Package: protGear
Type: Package
Title: Protein Micro Array Data Management and Interactive
        Visualization
Version: 1.17.0
Authors@R: c(person("Kennedy", "Mwai", role = c("cre", "aut"), email = "keniajin@gmail.com"),
             person(c("James", "Mburu"), role = "aut" , email = "mburuwanja@gmail.com") ,
             person(c("Jacqueline", "Waeni"), role = "ctb" , email = "jacqwaeni@gmail.com"))
Description: A generic three-step pre-processing package for protein microarray data. This package contains different data pre-processing procedures to allow comparison of their performance.These steps are background correction, the coefficient of variation (CV) based filtering,  batch correction and normalization.
License: GPL-3
URL: https://github.com/Keniajin/protGear
BugReports: https://github.com/Keniajin/protGear/issues
Depends: R (>= 4.2), dplyr (>= 0.8.0) , limma (>= 3.40.2) ,vsn (>=
        3.54.0)
Imports: magrittr (>= 1.5) , stats (>= 3.6) , ggplot2 (>= 3.3.0) ,
        tidyr (>= 1.1.3) , data.table (>= 1.14.0), ggpubr (>= 0.4.0),
        gtools (>= 3.8.2) , tibble (>= 3.1.0) , rmarkdown (>= 2.9) ,
        knitr (>= 1.33), utils (>= 3.6), genefilter (>= 1.74.0), readr
        (>= 2.0.1) , Biobase (>= 2.52.0), plyr (>= 1.8.6) , Kendall (>=
        2.2) , shiny (>= 1.0.0) , purrr (>= 0.3.4), plotly (>= 4.9.0) ,
        MASS (>= 7.3) , htmltools (>= 0.4.0) , flexdashboard (>= 0.5.2)
        , shinydashboard (>= 0.7.1) , GGally (>= 2.1.2) , pheatmap (>=
        1.0.12) , grid(>= 4.1.1), styler (>= 1.6.1) , factoextra (>=
        1.0.7) ,FactoMineR (>= 2.4) , rlang (>= 0.4.11), remotes (>=
        2.4.0)
Suggests: gridExtra (>= 2.3), png (>= 0.1-7) , magick (>= 2.7.3) ,
        ggplotify (>= 0.1.0) , scales (>= 1.1.1) , shinythemes (>=
        1.2.0) , shinyjs (>= 2.0.0) , shinyWidgets (>= 0.6.2) ,
        shinycssloaders (>= 1.0.0) , shinyalert (>= 3.0.0) , shinyFiles
        (>= 0.9.1) , shinyFeedback (>= 0.3.0)
biocViews: Microarray, OneChannel, Preprocessing ,
        BiomedicalInformatics , Proteomics , BatchEffect, Normalization
        , Bayesian, Clustering, Regression,SystemsBiology,
        ImmunoOncology
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/protGear
git_branch: devel
git_last_commit: d855f9b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:28:06 UTC; biocbuild
Author: Kennedy Mwai [cre, aut],
  James Mburu [aut],
  Jacqueline Waeni [ctb]
Maintainer: Kennedy Mwai <keniajin@gmail.com>

# Generated by roxygen2: do not edit by hand

export(array_vars)
export(best_CV_estimation)
export(bg_correct)
export(buffer_spots)
export(check_sampleID_files)
export(create_dir)
export(cv_by_sample_estimation)
export(cv_estimation)
export(extract_bg)
export(launch_protGear_interactive)
export(launch_select)
export(matrix_normalise)
export(merge_sampleID)
export(minpositive)
export(name_of_files)
export(output_trend_stats)
export(plot_FB)
export(plot_bg)
export(plot_buffer)
export(plot_normalised)
export(plot_normalised_antigen)
export(read_array_files)
export(read_array_visualize)
export(rlm_normalise)
export(rlm_normalise_matrix)
export(tag_subtract)
export(visualize_slide)
export(visualize_slide_2d)
import(FactoMineR)
import(GGally)
import(dplyr)
import(factoextra)
import(ggplot2)
import(ggpubr)
import(grid)
import(htmltools)
import(knitr)
import(limma)
import(pheatmap)
import(remotes)
import(rmarkdown)
import(shiny)
import(styler)
import(tibble)
import(tidyr)
import(vsn)
importFrom(Kendall,MannKendall)
importFrom(data.table,"%like%")
importFrom(data.table,fread)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_rows)
importFrom(dplyr,lag)
importFrom(dplyr,last)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(flexdashboard,renderValueBox)
importFrom(flexdashboard,valueBox)
importFrom(flexdashboard,valueBoxOutput)
importFrom(genefilter,rowSds)
importFrom(ggplot2,ggplot)
importFrom(gtools,mixedsort)
importFrom(magrittr,"%>%")
importFrom(plotly,ggplotly)
importFrom(plyr,.)
importFrom(purrr,set_names)
importFrom(readr,read_csv)
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,UQ)
importFrom(rlang,parse_expr)
importFrom(rlang,sym)
importFrom(shinydashboard,renderInfoBox)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(utils,read.csv)
importFrom(utils,write.table)

## General information
protGear is a package for protein micro-array data processing just before the main analysis. 

The package loads the '`gpr`' or '`txt`' file format extracted by the quantification software and merges this with the specific sample identifiers. The package processes multiple files extracted in a batch with their corresponding sample identifier file. The sample identifier file has 2 variables '`v1`' and '`v2`' which indicate the mini-array or block number and sample identifier respectively. The '`gpr`' file and the corresponding sample identifier file have the same file name.  protGear also provides a web based $Shiny^{(R)}$ platform for real time visualization of the data processing. 

There is a package vignette included with this package with a step by step use of the package. Check https://keniajin.github.io/protGear/ under articles. 

## Installing the devel package

``` r
library(remotes) 
remotes::install_github("keniajin/protGear")
```


## Installing from Bioc BioConductor

``` r
## install from BioConductor
if (!require("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}
BiocManager::install("protGear")
```

## To run the shiny application
To use the application locally on your compute install R and launch the app using the following command. 
`protGear::launch_protGear_interactive()`

To Install R: 
 - Open an Internet browser and go to www.r-project.org. 
 - Under "Getting Started" click the "download R" link
 - Select a CRAN location (a mirror site) and click the corresponding link.
 - Download R depending on your operating system. 

## protGear help pages

- The main page - https://keniajin.github.io/protGear/ 
- Vignette - https://keniajin.github.io/protGear/articles/vignette.html

## Contribute

Check the Github page for [source 
code](https://github.com/Keniajin/protGear/).


## Publication 

Mwai K, Kibinge N, Tuju J, Kamuyu G, Kimathi R, Mburu J, Chepsat E, Nyamako L, Chege T, Nkumama I, Kinyanjui S. et al. , protGear: A protein microarray data pre-processing suite. Computational and Structural Biotechnology Journal. 2021 Apr 24.

DOI: https://doi.org/10.1016/j.csbj.2021.04.044 

library(shiny)
library(shinyFiles)

# Define UI for application that draws a histogram
ui <- fluidPage(

  # Application title
  titlePanel("Example - Shiny Files Buttons Use"),

  # Sidebar with a slider input for number of bins
  sidebarLayout(
    sidebarPanel(
      p("First press this ShinyDir Button to locate an existing folder and set
        it to be your new working directory"),
      shinyDirButton("folderChoose","Choose Folder","Choose working directory"),

      p("In this second step, press the ShinyFiles Button that should point to the
        volumes (Just as the previous button does). In my R version, this is where it
        renders a white interface (see attached picture"),
      shinyFilesButton("filesChoose1","Files Chooser 1","Choose your files",
                       multiple=TRUE),

      p("In this third button, pI owuld like to link the ShinyFiles interface with
        the selected folder in button 1. This part I have not been able to solve"),
      shinyFilesButton("filesChoose2","Files Chooser 2","Choose your files",
                       multiple=TRUE)

      ),

    # Show a plot of the generated distribution
    mainPanel(
      verbatimTextOutput("path")
    )
      )
    )

# Define server logic required to draw a histogram
server <- function(input, output, session) {

  volumes <-  getVolumes()
  shinyDirChoose(input, "folderChoose", roots = volumes, session = session)
  sel_path <- reactive({return(print(parseDirPath(volumes, input$folderChoose)))})
  shinyFileChoose(input, "filesChoose1", roots = volumes, session = session)


  setWorkingDir<-eventReactive(input$folderChoose,{
    setwd(sel_path())
  })

  output$path<-renderText(sel_path())
}

# Run the application
shinyApp(ui = ui, server = server)

#' Extract buffer spots of data
#'
#' @param Data1 An object of the class data frame
#' @param buffer_spot A character string containing the name of the buffer
#' spots.
#' @import dplyr
#' @importFrom dplyr select  filter
#' @description A function to extract the buffer spots data. A buffer spot only
#'  has the solution for
#'  proprietary ingredients for stabilizing protein and minimizing evaporation.
#' @return A data frame of the buffer control spots
#' @export
#'
#' @examples
#' bg_correct_df <- readr::read_csv(system.file("extdata", "Data1_sample.csv",
#' package="protGear"))
#' buffer_spots(Data1 = bg_correct_df)
buffer_spots <- function(Data1 , buffer_spot = "buffer") {
 Data2_buffer <- Data1 %>%
    # within each Name count sampleID. We had grouped this earlier
    # n() - gives number of observations in the current group
    mutate(replicate = 1:n()) %>%
    # Select only relevant variables
    dplyr::select(sampleID,
                  antigen,
                  replicate,
                  FMedianBG_correct,
                  Block,
                  Column,
                  Row)
  
  if (buffer_spot == 'buffer') {
    Data2_buffer <- Data2_buffer %>%
      filter(grepl('^[bB][Uu][Ff][Ff][Ee][Rr]', antigen))
  } else {
    Data2_buffer <- Data2_buffer %>%
      filter(grepl(paste0('^', buffer_spot), tolower(antigen)))
  }
  
  
  Data2_buffer <- Data2_buffer %>%
    # combine Name and replicate
    unite(antigen, antigen, replicate)
  return(Data2_buffer)
}


#'  Plot the buffer values
#'
#' @param buffer_names A character string containing the name of the variable
#' with buffer spots. Default set to 'antigen'.
#' @param buffer_mfi A character string containing the name of the variable with
#' MFI value.Assuming background correction is done already.
#' Default to 'FMedianBG_correct'
#' @param slide_id  A character string containing the name of the slide/array
#' identifier variable.
#' @param df A data frame to be used to plot
#' @import dplyr ggplot2
#' @importFrom dplyr select  filter
#' @importFrom gtools mixedsort
#' @importFrom ggplot2 ggplot
#' @return plot of buffer spots
#' @export
#'
#' @examples
#'buffers <- readr::read_csv(system.file("extdata", "buffers_sample2.csv",
#'package="protGear"))
#' plot_buffer(df=buffers,buffer_names = "sampleID")
plot_buffer <- function(df = buffers,
                        buffer_names = "antigen",
                        buffer_mfi = "FMedianBG_correct",
                        slide_id = ".id") {
  x <- buffer_names
  y <- buffer_mfi
  df[[buffer_names]] <- factor(df[[buffer_names]] ,
                               levels = mixedsort(unique(
                                 as.character(df[[buffer_names]]))))
  
  
  p <- ggplot(data = df, aes_string(x = x, y = y)) +
    geom_jitter(aes_string(x = x, y = y, color = slide_id)) +
    geom_boxplot(aes_string(x = x, y = y), alpha = 0.2) +
    theme_light() +
    theme(axis.text.x = element_text(angle = 45, hjust = 1))
  return(p)
}

##________________________________________________________________________###
###creating a pipe operator to negate the %in% operator
#https://cran.r-project.org/web/packages/magrittr/vignettes/magrittr.html
# http://stackoverflow.com/questions/24941080/meaning-of-symbol-in-r
"%ni%" <- Negate("%in%")


#' Get the minimum positive value
#'
#' @param x  A numeric vector or variable
#'
#' @return Returns the minimum positive value in an object
#' @export
#'
#' @examples
#' minpositive(c(-1,-2,3,5,6,7,8,9,10))
minpositive <- function(x) {
  min(x[x > 0], na.rm = TRUE)
}



#' List the array structure variables
#'
#' @param channel A character indicating the channel that the data was scanned 
#' at. It is mostly included in the MFI variable names.
#' @param totsamples A numeric value indicating teh number of samples on a slide.
#' @param blockspersample A numeric value indicating the numer of blocks in a 
#' mini-array. The \code{".gal"} file can help in getting this
#' @param chip_path A character indicating the path of the folder  location with
#'  the array data.
#' @param sampleID_path A character indicating the path of the folder location 
#' with the sample identifiers matching the array structure.
#' @param mig_prefix Optional: A character indicating the identifier of an MIG
#'  dilution file
#' @param machine Optional:A character indicating the machine used to process
#'  the data in the folder
#' @param FG  Optional:A character indicating the name of the foreground
#'  variable name. if not specified its created as 
#'  \code{paste0("F",channel,".Median")}
#' @param BG Optional:A character indicating the name of the background 
#' variable name.  if not specified its created as 
#' \code{paste0("B",channel,".Median")}
#' @param FBG Optional:A character indicating the name of the 
#' foreground - background variable name.  if not specified its 
#' created as \code{paste0("F",channel,".Median...B",channel)}
#' @param date_process Optional:A character indicating the date
#'  when the samples were processed.
#'
#' @description A generic function returning a list with the data structure.
#' @importFrom rlang sym
#' @return a list of parameters required to process the data
#' @export
#'
#' @examples
#' ## specify the the parameters to process the data
#' genepix_vars <- array_vars(
#' ## the channel the data was processed in
#'   channel = "635",
#'   ## folder where the array data is stored
#'   chip_path = "data/array_data",
#'   ## the number of samples per slide or in as single run
#'   totsamples = 21,
#'   ## How many blocks each sample occupies
#'   blockspersample = 2,
#'   ## folder where the array data samples id files are stored
#'   sampleID_path = "data/array_sampleID/",
#'   ## optional
#'   mig_prefix = "_first",
#'   machine = 1,
#'   date_process = "0520"
#' )
#' genepix_vars
#' @return genepix_vars
#'
array_vars <- function(channel = "635",
                       totsamples ,
                       FG = "",
                       BG = "",
                       FBG = "",
                       blockspersample,
                       chip_path = "data/array_data",
                       sampleID_path = "data/array_sampleID/",
                       mig_prefix = "_first",
                       machine = "",
                       date_process = "") {
  ####List the directories with the CHIP data###############
  paths <- list.dirs(path = chip_path, recursive =  TRUE)
  ## remove the parent directory
  ## the folders with the chip data with the different batches is left
  paths <- paths[!grepl(paste0(chip_path, "$") , paths)]
  if (FG == "") {
    FG <- rlang::sym(paste0("F", channel, ".Median"))
  } else{
    FG <- rlang::sym(FG)
  }
  if (BG == "") {
    BG <- rlang::sym(paste0("B", channel, ".Median"))
  } else{
    BG <- rlang::sym(BG)
  }
  if (FBG == "") {
    FBG <- rlang::sym(paste0("F", channel, ".Median...B", channel))
  } else{
    FBG <- rlang::sym(FBG)
  }

  genepix_vars <-
    list(
      FG = FG,
      #rlang::sym(paste0("F",channel,".Median")),
      BG = BG,
      #rlang::sym(paste0("B",channel,".Median")) ,
      FBG = FBG,
      #rlang::sym(paste0("F",channel,".Median...B",channel)),
      paths = paths,
      chip_path = chip_path,
      sampleID_path = sampleID_path,
      mig_prefix = mig_prefix,
      machine = machine,
      date_process = date_process,
      totsamples = totsamples,
      blockspersample = blockspersample,
      mp = machine,
      dp = date_process
    )
  return(genepix_vars)
}





#' Title Create directory function
#'
#'
#' @param path folder location to create a directory
#' @description creating a directory
#' @return created directory
#' @export
#'
#' @examples
#' create_dir("data/sample_folder")
create_dir <- function(path) {
  if (!file.exists(paste0(path))) {
    dir.create(path)
  } else
    warning("The folder", path, " already exists")
}


###___________________________________________________

#' Object names of a list
#'
#' @param i - a list filenames with .txt or .gpr extension
#'
#' @return a list of file names
#' @export
#' @description A generic function returning a vector with the names of 
#' files in the same directory. Removes the file extension
#' @examples
#' name_of_files("KK2-06.txt")
#' @return name

name_of_files <- function(i) {
  name <- gsub("\\.txt*|\\.gpr*", "", i, perl = TRUE)
  name <- gsub(" repeat", "", name, perl = TRUE)
  name <- gsub(" ", "_", name, perl = TRUE)
  return(name)
}


#___________________________________________________
# Function to be called in case of replicated error

#'         \\\_Start_Function_For Error\\\         #
#'
#' @description A generic function to write into the log 
#' file with a replicate check error
#' @param iden An id for the file with replicates error
#' @return  a log file showing the replicate errors
#' @keywords internal
#'
error_replicates <- function(iden) {
  sink("log_replicates.txt" , append = TRUE)
  warning("The replicates per antigen per sample are more than expected for ",
          iden)
  sink()
}
#'         \\\_End_Function_\\\         #
#___________________________________________________




###
#' Check existing sample ID names
#' @param genepix_vars A list of specific definitions of the experiment design.
#'  See \code{\link{array_vars}}.
#' @description  A generic function to check if the file(s) 
#' with the MFI values have a corresponding sample ID file. Sample ID file is
#' a file with the identifiers for the samples in array file.
#' @return A file with missing corresponding sample ID files
#' @importFrom stats median quantile rnorm sd
#' @importFrom utils read.csv write.table
#' @export
#'
#' @examples
#' genepix_vars <- array_vars(
#' channel = "635",
#' chip_path = system.file("extdata", "array_data/machine1/", 
#' package="protGear"),
#' totsamples = 21,
#' blockspersample = 2,
#' mig_prefix = "_first",
#' machine = 1,
#' date_process = "0520"
#' )
#' check_sampleID_files(genepix_vars)
check_sampleID_files <- function(genepix_vars) {
  ## copy all sample ID with missing CSV file
  ##
  sid_files <-
    gsub(".csv", "", list.files(genepix_vars$sampleID_path))
  ## check if all the chip files have an existing sampleID file
  sid_check <-
    gsub(
      ".txt|.gpr",
      "",
      list.files(
        genepix_vars$chip_path ,
        recursive = TRUE,
        pattern = "*.txt|*.gpr",
        full.names = FALSE
      )
    )
  ## convert all the file names to caps to avoid merge errrors due to case
  sid_check <- toupper(sub(".*/(.*)", "\\1", sid_check))

  ## missing sampleID for any given file
  miss_id_file <- sid_check[sid_check  %ni%  toupper(sid_files)]



  if (length(miss_id_file) > 0) {
    write.table(miss_id_file, "missing_IDfile.txt")
  }

  if (length(miss_id_file) == 0) {
    warning("All array files have a corresponding sampleID file")
    return(0)
  } else{
    return(miss_id_file)
  }

}

#' Example data for protGear package.
#' dataC.csv
#' ------------------------------------
#' library(protGear)
## define the structure
# genepix_vars <- array_vars(channel="635" ,
#                            chip_path = system.file("extdata/array_data/", package="protGear"),
#                            totsamples = 21,
#                            blockspersample = 2,
#                            sampleID_path = system.file("extdata/array_sampleID/", package="protGear"),
#                            mig_prefix = "_first",
#                            machine =1,
#                            ## optional
#                            date_process = "0520")
#### read in all the datasets
### list all the file names under data folder
# filenames <- list.files(file.path(genepix_vars$paths[[1]]),
#                         pattern="*.txt$|*.gpr$", full.names=FALSE)
### read all the data files and save them in a list
# data_path <- paste0(genepix_vars$paths[[1]],"/")
# data_files <- purrr::map(.x = filenames,
#                          .f = read_array_files,
#                          data_path=data_path ,
#                          genepix_vars=genepix_vars)
# data_files <- set_names(data_files, purrr::map(filenames, name_of_files))
#
# ## this does the background correction after reading the
# ## genepix data file.
# sample_ID_merged_dfs <- purrr::map(.x=dfs, .f=merge_sampleID ,data_files=data_files ,
#                                    genepix_vars, method="subtract_local")
# sample_ID_merged_dfs <- set_names(sample_ID_merged_dfs, purrr::map(filenames, name_of_files))
# dataC <- sample_ID_merged_dfs[[1]]
#' bg_example.csv
#' ----------------------------------------
#' This is a sample of the allData_bg dataset
#' ---------------------------------
#' dfs <- names(data_files)
# allData_bg <- purrr::map(.x=dfs, .f=extract_bg,data_files=data_files,genepix_vars)
# allData_bg <- set_names(allData_bg, purrr::map(filenames, name_of_files))
# allData_bg <- plyr::ldply(allData_bg)
#
#'
#' Data1_sample.csv --
#' -----------------------------------
#' A sample of the dataC.csv file
#'
#'
#' Data1_bg_sample.csv
#'-----------------------------------
#' The data is generated from
#' Data1_bg_sample <- data_files[[1]]
#'
#' matrix_antigen.csv
#' -------------------------------
#' After creating the sample_ID_merged_dfs list
#' As indicated in the vignette
#' 1) Perform Coefficient of Variation (CV)
#' 2) Select the Best replicates
#' 3) If the proteins are tagged perform Tag subtraction
#' 4) Create the to_normalise data frame and then
###
# to_normalise <- df_to_normalise %>%
#   ungroup() %>% dplyr::select(-slide,-sampleID,-sample_array_ID) %>%
#   dplyr::select(antigen, sample_index, everything()) %>%
#   gather(variable, value, -(antigen:sample_index)) %>%
#   unite(temp, antigen ) %>%  dplyr::select(-variable) %>%
#   spread(temp, value) %>%
#   as.data.frame(.)
#
# ### get the row names of the machine data
# row.names(to_normalise) <- to_normalise$sample_index
# #batch_all <- as.factor(paste0(to_normalise$machine,"/",to_normalise$day))
# #machines <- as.factor(to_normalise$machine)
# #day_batches <- as.factor(to_normalise$day)
#
# ## create the matrix to normalise
# matrix_antigen <-  to_normalise %>%
#   dplyr::select(-sample_index) %>%
#   as.matrix(.)

#'
#' array_matrix.csv
#' --------------------------------
#'## create the matrix to hold the important parameters
## in place of AMA1 you use one of your features or antigen
# array_matrix <- df_to_normalise %>%
#   filter(antigen=="AMA1") %>%
#   ungroup() %>%
#   dplyr::select(sample_array_ID,sample_index,slide)
#'
#'
#' buffers_sample2.csv
#' ---------------------------------
#' This is a sample of buffers data frame below
# buffer_transp <- purrr::map(.x=sample_ID_merged_dfs,
# .f=buffer_spots ,  buffer_spot="buffer")
#
# buffer_transp <- set_names(buffer_transp, purrr::map(filenames, name_of_files))
#
# buffers <- plyr::ldply(buffer_transp)
#'
#'
#' TAG_antigens.csv
#' ----------------------
#' This is a file with all the proteins processed and their respective TAG's.
#' Tags are explained in detail under https://www.frontiersin.org/articles/10.3389/fimmu.2018.02866/full
#' Microarray Protein Map
#'
#' array_data/machine1
#' -----------------------------------
##'  KK2-06.txt and BRB001.txt
##'  This is data extracted using GenePix® Pro software (Molecular Devices).
##' This is data sets from from KILchip v1.0 (https://doi.org/10.3389/fimmu.2018.02866),
##' a protein microarray chip designed to enable the simultaneous detection of antibodies
##'  against > 100 Plasmodium falciparum proteins. The slides were printed in
##'  triplicate on a slide divided into mini-arrays, defined as the region allocated to
##'  a discrete sample and can be further divided into blocks.
##'
#'
#' @format An object of class 'ElistRaw' from limma package that
#' contains 5 entries: matrix E, data.frames targets, genes, source and printer.
#' E is a matrix of protein intensities of size 23232 by 28 (number of samples);
#' targets is a data.frame containing original sample file names;
#' genes is a data.frame with information about each protein on the Protoarray
#' source stores a name of method that has produced this dataset
#' and printer contains information about printer
#'
#"rawdata"

#' Normalize  Arrays
#'
#' @param matrix_antigen An object of class matrix with  features/proteins as 
#' columns and samples as the rows
#' @param method character string specifying the normalization method.
#'  Choices are \code{"none","log2","vsn","cyclic_loess"}
#' \code{"cyclic_loess_log" ,"rlm"}
#' @param batch_correct A logical value indicating whether batch
#'  correction should be done or not
#' @param batch_var1 A character or factor vector of size similar to rows 
#' of \code{matrix_antigen} indicating the first batch.
#' @param batch_var2 A character or factor vector of size similar to rows 
#' of \code{matrix_antigen} indicating the second batch.
#' @param return_plot A logical value indicating whether a plot is returned 
#' to show the results of normalisation.
#' @param control_antigens  logical vector specifying the subset of spots 
#' which are non-differentially-expressed control spots,
#' for use with \code{method="rlm"}
#' @param array_matrix  An object of class dataframe or matrix used with
#'  \code{method='rlm'} indicating the sample index and
#' @param plot_by_antigen Logical to indicate whether to plot by antigen or not
#' slide name for the matrix_antigen object.
#' @import limma tibble vsn
#' @return A data frame of normalised values
#' @export
#'
#' @examples
#' matrix_antigen <- readr::read_csv(system.file("extdata", 
#' "matrix_antigen.csv", package="protGear"))
#' #VSN
#' normlise_vsn <- matrix_normalise(as.matrix(matrix_antigen),
#' method = "vsn",
#' return_plot = TRUE
#' )
#' ## log
#' normlise_log <- matrix_normalise(as.matrix(matrix_antigen),
#' method = "log2",
#' return_plot = TRUE
#' )
#' ## cyclic_loess_log
#' normlise_cylic_log <- matrix_normalise(as.matrix(matrix_antigen),
#' method = "cyclic_loess_log",
#' return_plot = TRUE
#' )
matrix_normalise <-
  function(matrix_antigen,
           method = "log2",
           batch_correct = FALSE,
           batch_var1,
           batch_var2 = day_batches,
           return_plot = FALSE,
           plot_by_antigen = TRUE,
           control_antigens = NULL,
           array_matrix = NULL) {
    if (method == "log2") {
      exprs_log <- matrix_antigen
      ## since log can handle negative values , we convert all the 
     # negative values to a constant value
      # exprs_log[exprs_log<0] <- 1

      ## any value that is negative is allocated the smallest value in the batch
      min_pos <- minpositive(exprs_log)
      exprs_log <- replace(exprs_log, exprs_log < 1, min_pos)
      exprs_normalised <- log2(exprs_log)


    } else if (method == "none") {
      exprs_normalised <- matrix_antigen

    } else if (method == "vsn") {
      ## this approach utilises the VSN package
      ## structure the data to be normalised
      ## convert it to an Expression Set
      exprs_antigen <-
        Biobase::ExpressionSet(assayData = matrix_antigen)

      if (batch_correct == FALSE) {
        ## normalise without adjusting for the strata on VSN
        data_points <- dim(matrix_antigen)[[1]]
        exprs_normalised <-  vsn::justvsn(x = exprs_antigen ,
                                          minDataPointsPerStratum = data_points)
      } else if (batch_correct == TRUE) {
        exprs_normalised <-
          vsn::justvsn(
            x = exprs_antigen ,
            strata = machines ,
            minDataPointsPerStratum = data_points
          )
      }


    } else if (method == "cyclic_loess") {
      exprs_normalised <-  limma::normalizeCyclicLoess(
        matrix_antigen,
        weights = NULL,
        span = 0.7,
        iterations = 3,
        method = "fast"
      )
    } else if (method == "cyclic_loess_log") {
      exprs_normalised <-   limma::normalizeCyclicLoess(
        matrix_antigen,
        weights = NULL,
        span = 0.7,
        iterations = 3,
        method = "fast"
      )
      ## any value that is negative is allocated the smallest value in the batch
      min_pos <- minpositive(exprs_normalised)
      exprs_normalised <-
        replace(exprs_normalised, exprs_normalised < 1, min_pos)
      exprs_normalised <- log2(exprs_normalised)
    } else if (method == "rlm") {
      if (!is.null(control_antigens)) {
        exprs_df <-
          rlm_normalise_matrix(
            matrix_antigen = matrix_antigen,
            array_matrix = array_matrix,
            control_antigens = control_antigens
          )
        exprs_normalised <-   rlm_normalise(exprs_df)
        exprs_normalised <-  exprs_normalised  %>%
          gather(variable, value,-(antigen:sampleID2)) %>%
          unite(temp, antigen) %>% select(-variable) %>%
          spread(temp, value) %>%
          column_to_rownames(var = "sampleID2")
      } else if (is.null(control_antigens) | is.null(array_matrix)) {
        stop("Specify the control antigens or array_matrix to use RLM")
      }
    }

    cv_val <-
      round(sd(as.matrix(exprs_normalised), na.rm = TRUE) / 
              mean(as.matrix(exprs_normalised), na.rm = TRUE), 4) *
      100
    cv_val <- paste(cv_val, "%", sep = "")
    #exprs_normalised_df <- as.data.frame(exprs_normalised)

    ## return the data in the structure as it was supplied
    if (method == "vsn") {
      ## this kept if a different structure is considered
      matrix_antigen_normalised <- t(as.data.frame(exprs_normalised))
      row.names(matrix_antigen_normalised) <-
        as.numeric(gsub("X", "", row.names(matrix_antigen_normalised)))
      matrix_antigen_normalised <-
        as.data.frame(matrix_antigen_normalised)
    } else{
      matrix_antigen_normalised <-  as.data.frame(exprs_normalised)
      #matrix_antigen_normalised <- exprs_normalised_df
    }

    ## Whether to plot by sample or antigen
    if (plot_by_antigen == TRUE) {
      plot_normalisation <-
        plot_normalised_antigen(
          exprs_normalised_df = matrix_antigen_normalised,
          method = method,
          batch_correct = batch_correct
        )
    } else{
      plot_normalisation <-
        plot_normalised(
          exprs_normalised_df = matrix_antigen_normalised,
          method = method,
          batch_correct = batch_correct
        )
    }



    if (return_plot == TRUE) {
      return(
        list(
          matrix_antigen_normalised = matrix_antigen_normalised,
          plot_normalisation = plot_normalisation
        )
      )
    } else {
      return(matrix_antigen_normalised)
    }

  }









#' Nomrmalise using RLM
#' @param matrix_antigen A matrix with antigen data
#' @param array_matrix A matrix with control antigen data
#' @param control_antigens the control antigens for RLM normalisation
#' @description  A function for \code{method='rlm'} from 
#' \code{\link{matrix_normalise}}.
#' @return A RLM normalised data frame
#' @import dplyr
#' @export
#'
#' @examples
#' matrix_antigen <- readr::read_csv(system.file("extdata",
#'  "matrix_antigen.csv", package="protGear"))
#' # rlm_normalise_matrix(matrix_antigen=matrix_antigen,
#'  #array_matrix=array_matrix,
#' # control_antigens=control_antigens)
rlm_normalise_matrix <-
  function(matrix_antigen,
           array_matrix,
           control_antigens) {
    array_matrix$sample_index <-
      as.character(array_matrix$sample_index)

    array_matrix <- array_matrix %>%
      group_by(slide) %>%
      mutate(Array = group_indices())

    rlm_normalise_df <- as.data.frame.matrix(matrix_antigen) %>%
      #dplyr::mutate(sample_index=row.names(matrix_antigen))
      rownames_to_column(var = "sample_index")


    rlm_normalise_df <- rlm_normalise_df %>%
      select(sample_index, everything()) %>%
      gather(antigen, MFI_val, -sample_index) %>%
      left_join(array_matrix, by = "sample_index") %>%
      rename(Block = sample_array_ID) %>%
      mutate(Block = as.numeric(Block))


    ## create a variable to indicate the control antigens
    rlm_normalise_df <-   rlm_normalise_df %>%
      mutate(Description = ifelse(antigen %in% all_of(control_antigens) ,
                                  "Control", "Sample"))
    return(rlm_normalise_df)
  }


#' RLM normalisation
#'
#' @param rlm_normalise_df rlm normalised data frame
#' @description  A function for \code{method='rlm'} from 
#' \code{\link{matrix_normalise}}.
#' @return an elist of RLM normalisation to be utilised by 
#' \code{\link{rlm_normalise_matrix}}
#' @export
#' @keywords internal
#' @examples
#' matrix_antigen <- readr::read_csv(system.file("extdata", 
#' "matrix_antigen.csv", package="protGear"))
#' #rlm_normalise_df <- rlm_normalise_matrix(matrix_antigen=matrix_antigen, 
#' #array_matrix=array_matrix,
#' # control_antigens=control_antigens)
#' # rlm_normalise(rlm_normalise_df)
rlm_normalise <- function(rlm_normalise_df) {
  rlm_normalise_C <- rlm_normalise_df %>%
    filter(grepl("Control", Description))
  rlm_normalise_E <- rlm_normalise_df %>%
    filter(grepl("Sample", Description))


  ## create the matrix of the control antigens
  rlm_normalise_C <- rlm_normalise_C  %>%
    select(Array, Block, antigen, MFI_val) %>%
    spread(Array, MFI_val)

  ## create the matrix of the sample antigens
  rlm_normalise_E <- rlm_normalise_E  %>%
    select(Array, Block, antigen, MFI_val) %>%
    spread(Array, MFI_val)


  ## create Elist for the controls and samples
  sample_elist <- list(E = rlm_normalise_E %>%
                         select(-c(Block, antigen)) ,
                       genes =  rlm_normalise_E %>% select(Block:antigen))


  controls_elist <- list(E = rlm_normalise_C %>%
                           select(-c(Block, antigen)) ,
                         genes =  rlm_normalise_C %>%
                           select(Block:antigen))


  ## changing less values to the lowest positivie MFI value
  min_pos <- minpositive(controls_elist$E)
  controls_elist$E <-
    replace(controls_elist$E, controls_elist$E < 1, min_pos / 2)

  ## change the control EList to log2 -->
  ## RLM reccommends
  controls_elist$E  <- as.matrix(controls_elist$E)
  controls_elist$E <- log2(controls_elist$E)
  rownames(controls_elist$E) <- controls_elist$genes$antigen

  contr_names <- unique(rownames(controls_elist$E))
  contr_names_len <- length(contr_names)
  contr_mapping <- matrix(nrow = contr_names_len, ncol = 1)
  rownames(contr_mapping) <- contr_names
  contr_mapping[, 1] <- seq_len(contr_names_len)


  ##the number of not control antigens
  p_features <- nrow(sample_elist$E)
  p_controls <- nrow(controls_elist$E)
  n_arrays <- ncol(sample_elist$E)
  n_blocks <- max(controls_elist$genes$Block , na.rm = TRUE)
  y <- c(controls_elist$E)


  ## should be contr_names_len - 3 ... check
  ## should it be changed when the controls are more
  dummies <- matrix(0, ncol = {
    n_arrays + n_blocks + contr_names_len - 3
  }, nrow = length(y))


  rnames <- vector(length = length(y))
  a_cols <- paste("a", 1:{
    n_arrays - 1
  }, sep = "")


  b_cols <- paste("b", 1:{
    n_blocks - 1
  }, sep = "")



  ####changed
  t_cols <- paste("t", 1:{
    contr_names_len - 1
  }, sep = "")



  colnames(dummies) <- c(a_cols, b_cols, t_cols)

  ## matrix to hold the parameters per array for each of the control antigen
  a_params <- matrix(nrow = n_arrays, ncol = 2)
  rownames(a_params) <- colnames(controls_elist$E)

  ## matrix to hold the parameters for the Blocks
  b_params <- matrix(nrow = n_blocks, ncol = 2)
  idx <- 1

  ## putting values to the defined matrix
  # fill the matrix with 1's in such manner that for each row
  # the combination of 1's and 0's is unique
  for (i in seq_len(n_arrays)) {
    a_tmp <- paste("a", i, sep = "")
    a_params[i, 1] <- a_tmp
    for (j in seq_len(p_controls)) {
      rnames[idx] <- rownames(controls_elist$E)[j]
      b_idx <- controls_elist$genes$Block[j]
      b_tmp <- paste("b", b_idx, sep = "")
      b_params[b_idx, 1] <- b_tmp
      t_idx <- contr_mapping[rownames(controls_elist$E)[j],
                             1]
      t_tmp <- paste("t", t_idx, sep = "")
      if (i == n_arrays) {
        dummies[idx, a_cols] <- -1
      }
      else {
        dummies[idx, a_tmp] <- 1
      }
      if (b_idx == n_blocks) {
        dummies[idx, b_cols] <- -1
      }
      else {
        dummies[idx, b_tmp] <- 1
      }
      if (t_idx == contr_names_len) {
        dummies[idx, t_cols] <- -1
      }
      else {
        dummies[idx, t_tmp] <- 1
      }
      idx <- idx + 1
    }
  }

  ## bind with the control spots
  dummies <- cbind(y, dummies)
  rownames(dummies) <- rnames

  ## add the error term
  e <- rnorm(length(y), 0, 1)

  ## run the model of controls and a matrix created out of controls
  rlm_result <- MASS::rlm(y ~ . + e, data = data.frame(dummies))

  ### PAA approach
  a_params[-n_arrays, 2] <-
    rlm_result$coefficients[a_params[-n_arrays, 1]]
  a_params_sum <- sum(as.numeric(a_params[-n_arrays, 2]))
  a_params[n_arrays, 2] <- -a_params_sum


  b_params[-n_blocks, 2] <-
    rlm_result$coefficients[b_params[-n_blocks, 1]]
  b_params_sum <- sum(as.numeric(b_params[-n_blocks, 2]))
  b_params[n_blocks, 2] <- -b_params_sum


  a <- t(matrix(rep(as.numeric(a_params[, 2]), p_features),
                ncol = p_features))

  b <- matrix(mapply(function(i) {
    as.numeric(b_params[sample_elist$genes$Block[i], 2])
  }, rep(seq_len(p_features), n_arrays)), ncol = n_arrays)



  ### added by ken - to convert all the negatives in the sample antigens
 # to make them to the half of the lowest positive

  min_pos <- minpositive(sample_elist$E)
  sample_elist$E <-
    replace(sample_elist$E, sample_elist$E < 1, min_pos / 2)

  ###
  ## sample antigens matrix log 2
  sample_elist_log <- as.matrix(log2(sample_elist$E))

  sample_elist_normalised <- sample_elist_log - a - b
  row.names(sample_elist_normalised) <-
    paste0(sample_elist$genes$Block, "_", sample_elist$genes$antigen)


  sample_elist_normalised <-
    as.data.frame(sample_elist_normalised) %>%
    rownames_to_column("antigen_name") %>%
    mutate(
      Block = as.integer(sub('_.*$', '', antigen_name)) ,
      antigen = sub("[^_]*(.*)", "\\1", antigen_name),
      antigen = sub('.', '', antigen)
    ) %>%
    select(Block, antigen_name, antigen, everything())

  elist_normalised_df <- sample_elist_normalised %>%
    gather(Array, meanBest2_RLM, -c("Block", "antigen_name", "antigen")) %>%
    mutate(Array = as.numeric(Array)) %>%
    right_join(rlm_normalise_df, by = c("antigen", "Block", "Array"))


  elist_normalised_df <-
    elist_normalised_df %>% group_by(sample_index, slide)

  ## add the sample ID variable
  elist_normalised_df$sampleID2 <-
    group_indices(.data = elist_normalised_df)

  ##
  elist_normalised_df <- elist_normalised_df %>%
    ungroup() %>%
    select(antigen, sampleID2, meanBest2_RLM)

  return(elist_normalised_df)

}


#' Trend test using Cox–Stuart (C–S) and Mann–Kendall (M–K) trend tests
#'
#' @param name Name of the test
#' @param p_val p value from the test
#' @param z_val the Z value of the test
#'
#' @return A statistics of mean standard deviation trend
#' @export
#'
#' @examples
#' output_trend_stats(name="t.test",p_val=0.001, z_val=5)
output_trend_stats <- function(name, p_val, z_val) {
  if (p_val < 0.00001) {
    prop <- "<0.00001"
    z <- round(z_val, 4)
  } else{
    prop <- as.character(round(p_val, 4))
    z <- round(z_val, 4)
  }
  output <- paste0(name, ' Z-val = ', z, ' P-val = ', prop)
}



#' Comparison of normalised data by sample
#'
#' @param exprs_normalised_df a normalised data frame
#' @param method the method of normalisation used
#' @param batch_correct the batch correction
#'
#' @import dplyr   ggplot2
#' @importFrom  readr  read_csv
#' @importFrom Kendall MannKendall
#' @importFrom genefilter rowSds
#' @importFrom plyr .
#' @return A ggplot of normalised data
#' @export
#'
#' @examples
#' matrix_antigen <- readr::read_csv(system.file("extdata", 
#' "matrix_antigen.csv", package="protGear"))
#' normlise_vsn <- matrix_normalise(as.matrix(matrix_antigen),
#' method = "vsn",
#' return_plot = FALSE
#' )
#' plot_normalised(normlise_vsn,method="vsn",batch_correct=FALSE)
plot_normalised <-
  function(exprs_normalised_df,
           method,
           batch_correct) {
    exprs_normalised_df_plot <-  exprs_normalised_df %>%
      dplyr::mutate(
        mean_all_anti = rowMeans(., na.rm = TRUE),
        stdev_all_anti = genefilter::rowSds(as.matrix(.), na.rm = TRUE)
      ) %>%
      dplyr::mutate(
        rank_mean_all_anti = rank(mean_all_anti) ,
        method = method,
        batch_correct = batch_correct
      ) %>%
      arrange(rank_mean_all_anti)

    # perform the trend test using the Cox–Stuart (C–S) and Mann–Kendall (M–K) 
    #trend tests for the null hypothesis of no 
    #trend in the transformed standard deviations
    #under several transformation
    #cs_trend <- trend::cs.test(exprs_normalised_df_plot$stdev_all_anti)
    # mk_trend <- trend::mk.test(exprs_normalised_df_plot$stdev_all_anti)

    mk_trend <-
      Kendall::MannKendall(exprs_normalised_df_plot$stdev_all_anti)



    cs_stuart <- "Cox-Stuart"
    m_kendall <-
      output_trend_stats('Mann-Kendall (tau stats)', mk_trend$sl[[1]],
                         mk_trend$tau[[1]])



    normalisation_approaches <- c(
      "Log2" = "log2",
      "VSN" = "vsn",
      "Cyclic Loess" = "cyclic_loess",
      "Cyclic Loess (log)" = "cyclic_loess_log",
      "RLM" = "rlm"
    )
    norm_method <- names(which(normalisation_approaches == method))

    p_norm <-
      ggplot(exprs_normalised_df_plot ,
             aes(x = rank_mean_all_anti, y = stdev_all_anti)) +
      geom_jitter(color = "red") + theme_classic() +
      expand_limits(y = c(0, 10)) +  
      stat_cor() + geom_smooth(color = 'blue', se = FALSE, size = 0.5) +
      ggtitle(paste(norm_method, "Normalisation")) + 
      xlab("pooled mean rank (mean of features by sample)") +
      ylab("pooled SD") +
      labs(caption = paste0(m_kendall , "\n", cs_stuart)) +
      theme(plot.caption = element_text(
        hjust = 0,
        color = "black",
        face = "italic"
      ))
    return(p_norm)
  }



#' Comparison of normalised data by feature
#'
#' @param exprs_normalised_df a normalised data frame
#' @param method the method of normalisation used
#' @param batch_correct the batch correction
#'
#' @import dplyr
#' @importFrom Kendall MannKendall
#' @return A ggplot of various normalisation approaches
#' @export
#'
#' @examples
#' matrix_antigen <- readr::read_csv(system.file("extdata", 
#' "matrix_antigen.csv", package="protGear"))
#' normlise_vsn <- matrix_normalise(as.matrix(matrix_antigen),
#' method = "vsn",
#' return_plot = FALSE
#' )
#' plot_normalised_antigen(normlise_vsn,method="vsn",batch_correct=FALSE)
plot_normalised_antigen <-
  function(exprs_normalised_df,
           method,
           batch_correct) {
    ## how can we combine these two functions and make them the same
    antigen_summ <- exprs_normalised_df %>%
      gather(antigen, MFI) %>%
      group_by(antigen) %>%
      dplyr::summarise(mean_mfi = mean(MFI, na.rm = TRUE),
                       sd_mfi = sd(MFI, na.rm = TRUE)) %>%
      dplyr::mutate(
        rank_mean_all_anti = rank(mean_mfi) ,
        method = method,
        batch_correct = batch_correct
      ) %>%
      arrange(rank_mean_all_anti)

    # perform the trend test using the Cox–Stuart (C–S) and Mann–Kendall (M–K) 
   # trend tests for
    #for the null hypothesis of no trend in the transformed standard deviations
   # under several transformation
  #cs_trend2 <- trend::cs.test(antigen_summ$sd_mfi)
  #mk_trend2 <- trend::mk.test(antigen_summ$sd_mfi[!is.na(antigen_summ$sd_mfi)])

  #cs_stuart2 <- output_trend_stats("Cox-Stuart",
    #cs_trend2$p.value, cs_trend2$statistic)
  #m_kendall2 <- output_trend_stats('Mann-Kendall',
 #mk_trend2$p.value,mk_trend2$statistic )

    ## Changed here to use Kendall packages
    mk_trend2 <-
      Kendall::MannKendall(antigen_summ$sd_mfi[!is.na(antigen_summ$sd_mfi)])
    cs_stuart2 <-   "Cox-Stuart" 
    
  #output_trend_stats("Cox-Stuart",cs_trend2$p.value, cs_trend2$statistic[[1]])
    m_kendall2 <-
      output_trend_stats('Mann-Kendall (tau stats)', 
                         mk_trend2$sl[[1]], mk_trend2$tau[[1]])


    normalisation_approaches <- c(
      "Log2" = "log2",
      "VSN" = "vsn",
      "Cyclic Loess" = "cyclic_loess",
      "Cyclic Loess (log)" = "cyclic_loess_log",
      "RLM" = "rlm"
    )
    norm_method <- names(which(normalisation_approaches == method))

    p_norm <-
      ggplot(antigen_summ ,  aes(x = rank_mean_all_anti, y = sd_mfi)) +
      geom_jitter(color = "red") + theme_classic() +
      expand_limits(y = c(0, 10)) +  stat_cor() + geom_smooth(color = 'blue',
                                                              se = FALSE, size =
                                                                0.5) +
      ggtitle(paste(norm_method, "Normalisation")) + 
      xlab("Pooled mean rank (mean of features)") +
      ylab("pooled SD") +
      labs(caption = paste0(m_kendall2 , "\n", cs_stuart2)) +
      theme(plot.caption = element_text(
        hjust = 0,
        color = "black",
        face = "italic"
      ))

    return(p_norm)
  }

#' Summarise CV by samples
#' @title cv by sample
#'
#' @param cv_variable A character string containing the identifier 
#' of the variable with CV values.
#' @param lab_replicates A numeric value indicating the number of lab 
#' replicates.
#' @param  sampleID_var A character string containing the name of the 
#' sample identifier variable. Default set to 'sampleID'
#' @param dataCV A dataframe
#' @import dplyr tidyr
#' @importFrom tidyr gather
#' @importFrom dplyr lag filter
#' @description A function to give the summary of 
#' the CV's by the sampleID
#' @return A data frame of CV calculated by sample
#' @export
#'
#' @examples
#' dataC <- readr::read_csv(system.file("extdata", 
#' "dataC.csv", package="protGear"))
#' ## this file has 3 lab replicates and the default names
#' dataCV <- cv_estimation(dataC  ,lab_replicates=3)
#' cv_by_sample_estimation(dataCV, cv_variable = "cvCat_all",
#'  lab_replicates = 3)
cv_by_sample_estimation <-
  function(dataCV,
           cv_variable,
           lab_replicates,
           sampleID_var = 'sampleID') {
    ## creating a summary of the CV's by sampleID for each file
    ## helps in identifying samples with a high CV value
    if (lab_replicates > 1) {
      iden <- unique(dataCV$iden)
      dataC_cvSample <-  dataCV %>%
        group_by_at(c(sampleID_var, cv_variable)) %>%
        summarise(n = n()) %>%
        mutate(perc = round((n / sum(n)) * 100, 2)) %>%
        #rename(cvCat='get(cv_variable)') %>%
        ungroup() %>%
        gather(variable, value,-c(sampleID_var, cv_variable)) %>%
        unite(temp,!!(cv_variable), variable) %>%
        tidyr::spread(temp, value, fill = 0)
    } else{
      dataC_cvSample <- NULL
      warning("The experiment is specified not to have lab replicates")
    }

    return(dataC_cvSample)
  }

shinyUI(
  dashboardPage(#theme = shinytheme("sandstone") ,
    dashboardHeader(title = "Menu Select"),
    dashboardSidebar(width="20%",
                     column(12,
      shinyFiles::shinyDirButton("folderChoose",
                                 "Chose directory with the data",
                                 "Upload"),
      br(),
      uiOutput("batch_select"),
      h4("This will just show the folder named +sampleID+"),
      uiOutput("sampleID_select"),
      br(),
       sidebarMenu(
        menuItem("Dashboard", tabName = "dashboard"),
        menuItem("Raw data", tabName = "rawdata")
      )
    )),
    dashboardBody(
      tabItems(
        tabItem("dashboard",
                # this Row picks the parametes
                fluidRow(
                  column(2,
                         uiOutput('channel_output')
                         ),
                  column(2,
                         uiOutput('total_samples_output')
                  ), 
                  column(2,
                            uiOutput('blockspersample_output')
                  ),
                  column(2,
                         uiOutput('mig_prefix_output')
                  ),
                  column(2,
                         uiOutput('machine_output')
                  ),
                  column(2,
                         uiOutput('date_process_output')
                  )
                  
                ),
                fluidRow(
                  column(3,
                  uiOutput("files_in_batch")
                  ),
                  column(3,
                  uiOutput("check_sampleID_out")  
                    ),
                  column(3,
                    uiOutput("antigens_count")  
                  ),
                  column(3,
                    uiOutput("blocks_count")  
                  )
                ),
                fluidRow(
                  box(
                 #   width = 8, status = "info", solidHeader = TRUE,
                  #  title = "Popularity by package (last 5 min)",
                  #  bubblesOutput("packagePlot", width = "100%", height = 600)
                    div(class="span6" , "Buffer spots overview", 
                        plotOutput('data_process', width="100%"),
                        style = "height:80%;background-color: aqua;")
                  ),
                  box(
                  div(class="span6" , "Data overview", 
                        plotlyOutput('data_box', width="100%"),
                        style = "height:80%;background-color: aqua;"),
                  # slide bar for the 
                  sliderInput("slider_antigen", label = h3("Select the antigen range to view"), min = 0, 
                              max = 120, value = c(40, 60))
                  )
                )
        ),
        tabItem("rawdata",
              #  numericInput("maxrows", "Rows to show", 25),
                #verbatimTextOutput("rawtable"),
                #downloadButton("downloadCsv", "Download as CSV")
              DT::DTOutput("tbl")
        )
      )
    )
  )
)

library(shiny)
library(shinydashboard)
library(magrittr)
library(protGear)
"%ni%" <- Negate('%in%')

# HELP & INTRO DATA ---------------------------------------------------------------

#steps <- readr::read_csv2("help.csv")
steps <- system.file("shiny-examples/protGear_interactive/", "help.csv", package="protGear")
steps <- readr::read_csv2(steps)
#intro <- read_csv2("intro.csv")


# FLUID DESIGN FUNCTION ---------------------------------------------------

fluid_design <- function(id, w, x, y, z) {
  fluidRow(
    div(
      id = id,
      column(
        width = 6,
        uiOutput(w),
        uiOutput(y)
      ),
      column(
        width = 6,
        uiOutput(x),
        uiOutput(z)
      )
    )
  )
}

#' launch_protGear_interactive
#'
#' This is Function is to launch the shiny application
#' @return launches the shiny interactive protGear app
#' @import rmarkdown shiny GGally pheatmap  knitr
#' grid styler factoextra FactoMineR   remotes
#' @importFrom flexdashboard renderValueBox valueBoxOutput valueBox
#' @importFrom shinydashboard renderInfoBox
#' @importFrom  dplyr group_rows between first last
#' @export
#' @examples
#' app <- system.file("shiny-examples", "protGear_interactive",
#' "protGear_interactive.Rmd", package = "protGear")
#'  if (app!=""){
#'  ## run this
#'  #launch_protGear_interactive()
#'  }
#'
launch_protGear_interactive <- function() {
  appDir <-
    system.file("shiny-examples",
                "protGear_interactive",
                "protGear_interactive.Rmd",
                package = "protGear")
  if (appDir == "") {
    stop("Could not find example directory. Try re-installing `protGear`.",
         call. = FALSE)
  }

  rmarkdown::run(file = appDir)
}

#' launch_select
#'
#' This is Function is to launch mutiple shiny applications for protGear
#' @param theApp accepts one of the folders containing the shiny appplication
#' @return launches the app defined under theApp
#' @export
#' @examples
#' validExamples <-
#'  list.files(system.file("shiny-examples", package = "protGear"))
#' #launch_select(validExamples[[1]])
launch_select <- function(theApp) {
  # locate all the shiny app examples that exist
  validExamples <-
    list.files(system.file("shiny-examples", package = "protGear"))

  validExamplesMsg <-
    paste0("The available apps in `protGear` are: '",
           paste(validExamples, collapse = "', '"),
           "'")

  # if an invalid folder is given, throw an error
  if (missing(theApp) || !nzchar(theApp) ||
      !theApp %in% validExamples) {
    stop(
      'Please run `launch_select()` with a valid example app as an argument.\n',
      validExamplesMsg,
      call. = FALSE
    )
  }

  # find and launch the app
  appDir <-
    system.file("shiny-examples", theApp, package = "protGear")
  if (grepl('protGear_interactive', theApp)) {
    file_rmd <- 'protGear_interactive.Rmd'
  } else
    file_rmd <- 'index.Rmd'
  rmarkdown::run(file = paste0(appDir, "/", file_rmd))
}

#' Read multiple array files
#' @title Read array files
#' @param i  The name of the file which the data are to be read from.
#' @param data_path  The path where the file with the data  is located
#' @param genepix_vars  A list of specific definitions of the experiment design.
#' See \code{\link{array_vars}}.
#' @description This helps to read the chip file(s).
#' @return a number of data frames in the global environment
#' @importFrom  data.table fread %like%
#' @importFrom rlang parse_expr :=
#' @importFrom purrr set_names
#' @importFrom magrittr %>%
#' @export
#' @examples
#' ## Not run:
#' genepix_vars <- array_vars(
#' channel = "635",
#' chip_path = system.file("extdata", "array_data/machine1/", 
#' package="protGear"),
#' totsamples = 21,
#' blockspersample = 2,
#' mig_prefix = "_first",
#' machine = 1,
#' date_process = "0520"
#' )
#' file_read <- "KK2-06.txt"
#' read_array_files(i=file_read,
#' data_path=system.file("extdata", "array_data/machine1/",
#' package="protGear"), genepix_vars=genepix_vars)
#' ## End(Not run)
read_array_files <- function(i, data_path, genepix_vars) {
  ###loop through all the data to read
  # skip- the number of lines of the data file to skip before 
  #beginning to read data ---
  if (length(grep("Block.*Column|Column.*Block", 
                  readLines(file.path(data_path, i))) -
             1) == 1) {
    x <-
      grep("Block.*Column|Column.*Block", readLines(file.path(data_path, i))) -
      1
    #print(paste0(x,"_",i))
    d_f <-
      data.table::fread(file.path(data_path, i),
                        skip = x,
                        header = TRUE)
    ## arrange block to ensure the order is maintains
    d_f <- d_f %>% arrange(Block)
    ## calculate the different background  methods
    if (paste0(genepix_vars$BG) %ni% names(d_f)) {
      genepix_vars$BG <-
        names(d_f)[names(d_f) %like% paste0(genepix_vars$BG, '$')]
    }

    if (paste0(genepix_vars$FG) %ni% names(d_f)) {
      genepix_vars$FG <-
        names(d_f)[names(d_f) %like% paste0(genepix_vars$FG, '$')]
    }

    expression_med <-
      paste0("median(`", genepix_vars$BG, "`, na.rm = TRUE)")
    exp_minpos <-  paste0("minpositive(`", genepix_vars$BG, "`)")
    d_f <- d_f %>%

      mutate(global_BGMedian := !!parse_expr(expression_med)) %>%
      ## minimum BG per block and >1 -- Moving minimum approach per block
      group_by(Block) %>%

      mutate(minimum_BGMedian := !!parse_expr(exp_minpos)) %>%
      ungroup()


  }  else
    stop("File " , i, " does not have Block Column Specification")

}




#' Extract the background values
#' @title  extract bg
#' @param iden A character indicating the name of the object to be
#'  used under data_files.
#' @param data_files A list of data objects with names utilised by iden.
#' @param genepix_vars A list of specific definitions of the experiment design.
#'  See \code{\link{array_vars}}.
#' @description A generic function to extract the background 
#' data for micro array data.
#' @return A data frame of background values
#' @importFrom dplyr select arrange
#' @importFrom data.table %like%
#' @importFrom purrr set_names
#' @export
#'
#' @examples
#' ## Not run:
#' genepix_vars <- array_vars(
#' channel = "635",
#' chip_path = system.file("extdata", "array_data/machine1/", 
#' package="protGear"),
#' totsamples = 21,
#' blockspersample = 2,
#' mig_prefix = "_first",
#' machine = 1,
#' ## optional
#' date_process = "0520"
#' )
#' #Define the data path
#' data_path <- paste0(genepix_vars$chip_path)
#' # List the file names to use
#' filenames <- list.files(genepix_vars$chip_path,
#'                        pattern = '*.txt$|*.gpr$', full.names = FALSE
#' )
#' data_files <- purrr::map(
#'  .x = filenames,
#'   .f = read_array_files,
#'   data_path = data_path,
#'   genepix_vars = genepix_vars
#' )
#' data_files <- purrr::set_names(data_files, 
#' purrr::map(filenames, name_of_files))
#' names(data_files)
#' extract_bg(iden ="KK2-06" , data_files=data_files,genepix_vars=genepix_vars)
#' ## End(Not run)
extract_bg <- function(iden, data_files , genepix_vars = genepix_vars)
{
  ## read in the sample ID files
  ## this can be pulled from a mysql table
  ## if the sample ID is not available, we create an automated sampleID
  if (file.exists(file.path(genepix_vars$sampleID_path, 
                            paste0(iden , ".csv")))) {
    arraynames <-
      read.csv(
        file.path(genepix_vars$sampleID_path, paste0(iden , ".csv")) ,
        header = TRUE ,
        stringsAsFactors = FALSE ,
        colClasses = "character"
      )
  } else{
    warning(iden,
            " Not found in the sampleID files here",
            genepix_vars$sampleID_path)
    arraynames <- data.frame(
      v1 = (1:genepix_vars$totsamples) ,
      v2 = paste0("SID_gen", 1:genepix_vars$totsamples),
      barcode = iden
    )

  }


  ## replicate to the number of blocks
  ## make sure the block is arranged before merging with the data file
  arraynames <- arraynames %>%
    dplyr::select(v1, v2) %>%
    arrange(as.numeric(v1))

  ## capture errors for same sample ID in a slide
  if (length(unique(arraynames$v2)) < genepix_vars$totsamples) {
    sink("log_replicates.txt" , append = TRUE)
    warning("Most likely there is a repeated sample name for " , iden)
    sink()
    arraynames <- arraynames %>%
      group_by(v2) %>%
      mutate(index = 1:n()) %>%
      mutate(v2 = ifelse(index > 1, paste0(v2, "_", index), v2)) %>%
      select(-index)
  }


  ## get the data from the loop
  ## extract the specific data from the data files
  data <- data_files[[iden]]

  ## pick the spots per block from the data file
  spotsperblock <- table(data$Block)[[1]]
  sampleID <-
    rep(arraynames$v2, each = spotsperblock * genepix_vars$blockspersample)



  Data1 <- data %>%
    # assign respective sample number to each row
    mutate(
      sample = rep(
        seq_len(genepix_vars$totsamples),
        each = spotsperblock * genepix_vars$blockspersample
      ),
      # Bring in the sampleIDs..192 each sample
      sampleID = sampleID,
      # Abit of formating the Antigen names and concentration
      Name = gsub(':', '', Name),
      Name = gsub('\n', '', Name),
      Name = gsub(' ', '', Name)
    )  %>%
    ##remove uneccessary concs
    ## filter(!grepl('Landmark|Buffer|IgG', Name))   %>%
    # group by iden and antigen name
    group_by(sampleID, Name)

  if (paste0(genepix_vars$BG) %ni% names(Data1)) {
    genepix_vars$BG <-
      names(Data1)[names(Data1) %like% paste0(genepix_vars$BG, '$')]
  }

  if (paste0(genepix_vars$FG) %ni% names(Data1)) {
    genepix_vars$FG <-
      names(Data1)[names(Data1) %like% paste0(genepix_vars$FG, '$')]
  }

  #----------------------------------------------------------------------------
  ##save the MFI values of the Background
  data1_bg <-
    Data1 %>% dplyr::select(
      sampleID,
      antigen = Name,
      Block,
      FBG_Median = !!genepix_vars$FG ,
      BG_Median = !!genepix_vars$BG
    ) %>%
    mutate(replicate = 1:n()) %>%
    filter(
      !grepl(
        '^[Ll][Aa][Nn][Dd][Mm][Aa][Rr][Kk]|^[bB][Uu][Ff][Ff][Ee][Rr]',
        antigen
      )
    )
  # combine Name and replicate
  #----------------------------------------------------------------------------
  return(data1_bg)
}





#' @title Plot background
#'
#' @param df A default dataset to use for plot.
#' @param bg_MFI A numeric \code{variable} describing which is the 
#' background MFI
#' @param x_axis The variable on the x axis
#' @param log_mfi a logical value indicating whether the MFI values should be
#' log transformed or not.
#'
#' @description  A generic function for plotting of R objects.
#' @return A ggplot of background values
#' @export
#' @import ggpubr
#' @examples
#' ## Not run:
#' #After extracting the background using \code{\link{extract_bg}} 
#' #we plot the data using
#' allData_bg <- readr::read_csv(system.file("extdata", "bg_example.csv",
#'  package="protGear"))
#' plot_bg(allData_bg,
#' x_axis = "antigen",
#' bg_MFI = "BG_Median",  log_mfi = TRUE
#' )
#' ## End(Not run)

plot_bg <- function(df,
                    x_axis = "antigen",
                    bg_MFI = "BG_Median",
                    log_mfi = TRUE) {
  ## create an id to help in having a numeric sample ID to sort your data
  ## this is because all sampleIDs from the samples were not unique
  ## rename the original sampleID sampleID2
  bg_MFI_sys <- rlang::sym(bg_MFI)
  ### check if the .id exists and renane it to slide
  if ('.id' %in% names(df)) {
    df <- df %>%  dplyr::rename(slide = .id)
  } else
    df <- df %>%  dplyr::mutate(slide = "slide")

  bg_plot <- df %>%
    dplyr::mutate(log_bg =    log2(!!bg_MFI_sys))

  # bg_plot$sampleID <- group_indices(.data =bg_plot )

  if (log_mfi == TRUE) {
    p_pubr <- ggboxplot(
      data = bg_plot ,
      x = x_axis ,
      y = "log_bg",
      facet.by = "replicate",
      ncol = 1
    )
    p_pubr <- ggpar(
      p_pubr,
      font.tickslab = c(6, "#993333"),
      xtickslab.rt = 45 ,
      ylab = "Background of the replicates (log2)"
    )
    p_bg <-
      ggplot(data = bg_plot , aes_string(x = x_axis , y = "log_bg")) +
      geom_boxplot() +
      facet_wrap( ~ replicate, ncol = 1) +
      theme_light() +
      theme(axis.text.x = element_text(angle = 45 , hjust = 1))



  } else if (log_mfi == FALSE) {
    p_pubr <- ggboxplot(
      data = bg_plot ,
      x = x_axis ,
      y = bg_MFI,
      facet.by = "replicate",
      ncol = 1
    )
    p_pubr <- ggpar(
      p_pubr,
      font.tickslab = c(6, "#993333"),
      xtickslab.rt = 45 ,
      ylab = "Background of the replicates raw"
    )

    p_bg <-
      ggplot(data = bg_plot , aes_string(x = x_axis , y = bg_MFI)) +
      geom_boxplot() +
      facet_wrap( ~ replicate, ncol = 1) +
      theme_light() +
      theme(axis.text.x = element_text(angle = 45 , hjust = 1))
  } else {
    # p_pubr <- ggboxplot(data = bg_plot ,
    #                     x="antigen" , y=bg_MFI,
    #                     facet.by = "replicate",ncol=1)
    # p_pubr <- ggpar(p_pubr,font.tickslab = c(8,"#993333"),
    #                 xtickslab.rt = 45 , ylab = "Background of the 
    #replicates raw")

    p_bg <-
      ggplot(data = bg_plot , aes_string(x = x_axis , y = bg_MFI)) +
      geom_boxplot() +
      facet_wrap( ~ replicate, ncol = 1) +
      theme_light() +
      theme(axis.text.x = element_text(angle = 45 , hjust = 1))
  }
  return(p_pubr)
}

#___________________________________________________


#' Plot foreground and background values
#' @title  plot_FB
#' @param df An object containing the data to which the plot is done.
#' @param antigen_name The \code{variable} describing which features/proteins/
#' antibodies in the data should be used to plot
#' @param bg_MFI A numeric \code{variable} describing which is the 
#' background MFI
#' @param FG_MFI A numeric \code{variable} describing which is the 
#' foreground MFI
#' @param log_mfi a logical value indicating whether the MFI values should be
#' log transformed or not.
#' @description A generic function for plotting the background and foreground
#'  values.
#' @return a ggplot of foreground vs background MFI values
#' @export
#' @import ggplot2 dplyr
#' @examples
#' ## Not run:
#' #After extracting the background using \code{\link{extract_bg}} 
#' #we plot the data using
#' allData_bg <- readr::read_csv(system.file("extdata", 
#' "bg_example.csv", package="protGear"))
#' plot_FB(allData_bg,
#' antigen_name = "antigen",
#' bg_MFI = "BG_Median", FG_MFI = "FBG_Median", log = FALSE
#' )
#' ## End(Not run)
plot_FB <-
  function(df,
           antigen_name = "antigen",
           bg_MFI = "BG_Median",
           FG_MFI = "FBG_Median",
           log_mfi = FALSE) {
    ## create an id to help in having a numeric sample ID to sort your data
    ## this is because all sampleIDs from the samples were not unique
    ## rename the original sampleID sampleID2
    bg_MFI_sys <- rlang::sym(bg_MFI)
    FB_MFI_sys <- rlang::sym(FG_MFI)
    ### check if the .id exists and renane it to slide
    if ('.id' %in% names(df)) {
      df <- df %>%  dplyr::rename(slide = .id)
    } else
      df <- df %>%  dplyr::mutate(slide = "slide")

    bg_plot <- df %>%
      mutate(log_bg = log2(!!bg_MFI_sys),
             log_fb = log2(!!FB_MFI_sys))



    if (log_mfi == TRUE) {
      p <- ggplot(bg_plot , aes(
        log_fb,
        log_bg,
        text = paste(
          "Antigen: ",
          antigen,
          "<br>FG: $",
          FBG_Median,
          "<br>B: $",
          BG_Median
        )
      )) +
        xlab("Foreground MFI") + ylab("Background MFI") +
        geom_jitter() +
        theme_light()



    } else if (log_mfi == FALSE) {
      p <- ggplot(bg_plot , aes(
        FBG_Median,
        BG_Median,
        text = paste(
          "Antigen: ",
          antigen,
          "<br>FG: $",
          FBG_Median,
          "<br>B: $",
          BG_Median
        )
      )) +
        xlab("Foreground MFI") + ylab("Background MFI") +
        geom_jitter() +
        theme_light()
    } else {
      p <- ggplot(bg_plot , aes(
        FBG_Median,
        BG_Median,
        text = paste(
          "Antigen: ",
          antigen,
          "<br>FG: $",
          FBG_Median,
          "<br>B: $",
          BG_Median
        )
      )) +
        xlab("Foreground MFI") + ylab("Background MFI") +
        geom_jitter() +
        theme_light()
    }
    return(p)
  }


#' Background correction
#' @title bg_correct
#' @param iden A character indicating the name of the object to be 
#' used under Data1
#' @param Data1 A data frame with sample identifiers merged with micro 
#' array data.
#' @param genepix_vars A list of specific definitions of the experiment design.
#'  See \code{\link{array_vars}}.
#' @param method a description of the background correction to be used. 
#'  Possible values are \code{"none","subtract_local",
#' "subtract_global","movingmin_bg","minimum_half","edwards" or "normexp"}. 
#' The default is \code{"subtract_local"}.
#' @details  The function implements background correction methods developed 
#' by \code{\link[limma]{backgroundCorrect}}. But the
#' \code{minimum_half and movingmin_bg} uses the block of the protein array as
#' the grid. If method="movingmin_bg" the minimum
#' background value within a  block is subtracted.
#' If method="minimum_half" then any intensity which is negative after 
#' background subtraction is reset to be equal to half the 
#' minimum positive value in
#' a block.  If method="movingmin_value" then any intensity which is negative 
#' after background subtraction is reset to the minimum positive value
#' in a block. For \code{edwards} we implement a similar algorithm with 
#' \code{limma::backgroundCorrect(method="edwards")} and for \code{'normexp'}
#' we use  the saddle-point approximation to maximum likelihood, 
#' \code{\link[limma]{backgroundCorrect}} for more details.
#' @description  A generic function to perform background correction.
#' @return A data frame with background corrected data
#' @export
#' @import  dplyr limma
#' @importFrom rlang sym
#' @examples
#' ## Not Run
#' genepix_vars <- array_vars(
#'   channel = '635',
#'   chip_path = system.file('extdata', 'array_data/machine1/',
#'   package='protGear'),
#'   totsamples = 21,
#'   blockspersample = 2,
#'   mig_prefix = '_first',
#'   machine = 1,
#'   date_process = '0520')
#' raw_df <- readr::read_csv(system.file('extdata','Data1_bg_sample.csv',
#'  package='protGear'))
#' bg_correct(iden='iden', Data1 = raw_df, genepix_vars = genepix_vars,
#'  method='subtract_local')
#'  
bg_correct <-
  function(iden, Data1, genepix_vars, method = "subtract_local") {
    #--------------------------------------------------------------------------
    if (paste0(genepix_vars$BG) %ni% names(Data1)) {
      genepix_vars$BG <-
        names(Data1)[names(Data1) %like% paste0(genepix_vars$BG, '$')]
      genepix_vars$BG <- rlang::sym(genepix_vars$BG)
    }

    if (paste0(genepix_vars$FG) %ni% names(Data1)) {
      genepix_vars$FG <-
        names(Data1)[names(Data1) %like% paste0(genepix_vars$FG, '$')]
      genepix_vars$FG <- rlang::sym(genepix_vars$FG)
    }


    ##save the MFI values without formatting the background
    data1_full_bg <- Data1 %>%
      dplyr::select(sampleID,
                    antigen = Name,
                    FMedian = !!genepix_vars$FG) %>%
      mutate(replicate = 1:n()) %>%
      ## removing Land mark and Buffer
      filter(
        !grepl(
          '^[Ll][Aa][Nn][Dd][Mm][Aa][Rr][Kk]|^[bB][Uu][Ff][Ff][Ee][Rr]',
          antigen
        )
      )
    # %>%     spread(antigen, F635Median)
    # combine Name and replicate
    ## save in the background data folder files with higher background values
    file_ident <- paste0(iden, "_raw_MFI_BG", ".csv")

    #create_dir(path = system.file("processe_data/raw_MFI_BG/"))
    #write_csv(data1_full_bg ,system.file("processed_data/raw_MFI_BG/", 
    #'file_ident', package="protGear"))
    #-----------------------------------------------------------------------
    if (method == "none" | method == "") {
      #------------------------------------------------------------------------
      ##MFI values without subtracting the background
      Data1 <- Data1 %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG ,
          BGMedian = !!genepix_vars$BG,
          Block,
          Column,
          Row
        ) %>%
        mutate(FMedianBG_correct = FMedian) %>%
        mutate(replicate = 1:n()) 
      #------------------------------------------------------------------------

    } else if (method == "subtract_local") {
      ## this approach subtracts the local background estimated by 
      #the Array Jet Machine
      #-------------------------------------------------------------------------
      ##save the MFI values with subtracting the background
      Data1 <- Data1 %>%
        dplyr::mutate(FMedianBG_correct = (!!genepix_vars$FG) - 
                        (!!genepix_vars$BG)) %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          FMedianBG_correct,
          Block,
          Column,
          Row
        ) %>%
        dplyr::mutate(replicate = 1:n())
      #------------------------------------------------------------------------

    } else if (method == "subtract_global") {
      ## this approach subracts the median of the backgrounds in a slide
      #------------------------------------------------------------------------
      ##save the MFI values with subtracting the background
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG - global_BGMedian) %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          FMedianBG_correct,
          Block,
          Column,
          Row
        ) %>%
        mutate(replicate = 1:n())
      #------------------------------------------------------------------------
    } else if (method == "movingmin_bg") {
      ## this is subtracted
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG-!!genepix_vars$BG) %>%
      ## this is generated while reading the array files using 
        #read_array_files function
        mutate(FMedianBG_correct = !!genepix_vars$FG - minimum_BGMedian) %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          FMedianBG_correct,
          Block,
          Column,
          Row
        ) %>%
        mutate(replicate = 1:n())

    } else if (method == "minimum_half") {
      ## this approach ensures all the MFI values are positive
      ## if the MFI <0 after subtraction the MFI is set to the half of the 
      #minimum corrected intensities
      #------------------------------------------------------------------------
      ##save the MFI values with subtracting the background
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG-!!genepix_vars$BG) %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          FMedianBG_correct,
          Block,
          Column,
          Row
        )  %>%
        group_by(Block) %>%
        mutate(FMedianBG_correct = ifelse(
          FMedianBG_correct < 0.1,
          (minpositive(FMedianBG_correct) /
             2),
          FMedianBG_correct
        )) %>%
        group_by(sampleID, antigen) %>%
        mutate(replicate = 1:n())
      #------------------------------------------------------------------------
    } else if (method == "minimum_value") {
      ## this approach ensures all the MFI values are positive
      ## if the MFI <0 after subtraction the MFI is set to the minimum of 
      #the corrected intensities
      #-------------------------------------------------------------------------
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG-!!genepix_vars$BG) %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          FMedianBG_correct,
          Block,
          Column,
          Row
        )  %>%
        group_by(Block) %>%
        mutate(FMedianBG_correct = ifelse(FMedianBG_correct < 0.1,
                                          (minpositive(
                                            FMedianBG_correct
                                          )), FMedianBG_correct)) %>%
        group_by(sampleID, antigen) %>%
        mutate(replicate = 1:n())

      #------------------------------------------------------------------------
    } else if (method == "edwards") {
      #a log-linear interpolation method is used to adjust lower intensities as 
      #in Edwards (2003).
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG-!!genepix_vars$BG)
      one <- matrix(1, nrow(Data1), 1)
      delta.vec <- function(d, f = 0.1) {
        ##  mean(d < 1e-16, na.rm = TRUE) % of values that are negative
        ## mean(d < 1e-16, na.rm = TRUE) * (1 + f) the % of values just 
        #above the negative values
        ## gives the quartile cut off value of the threshold
        quantile(d,
                 probs = mean(d < 1e-16, na.rm = TRUE) * (1 + f),
                 na.rm = TRUE)
      }
      #delta <- one %*% apply(as.matrix(Data1[['FMedianBG_correct']]), 
      #2, delta.vec)
      ## no need to multiply with 1 since its returning the same value 
      #and we want to implement in a data frame
      delta <-
        apply(as.matrix(Data1[['FMedianBG_correct']]), 2, delta.vec)

      ## each value its given its own value accordingly
      ## this helps maintain the variation
      Data1 <- Data1 %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          FMedianBG_correct,
          BGMedian = !!genepix_vars$BG,
          Block,
          Column,
          Row
        ) %>%
        group_by(Block) %>%
        mutate(FMedianBG_correct = ifelse(FMedianBG_correct < delta,
                                          (delta * exp(
                                            1 - (BGMedian + delta) / FMedian
                                          )), FMedianBG_correct)) %>%
        group_by(sampleID, antigen) %>%
        dplyr::mutate(replicate = 1:n())
    } else if (method == "normexp") {
      ##a convolution of normal and exponential distributions is fitted to the 
      #foreground intensities using
      #the background intensities as a covariate, and the expected signal given
      #the observed foreground becomes
      #the corrected intensity. This results in a smooth monotonic 
      #transformation of the background subtracted
      #intensities such that all the corrected intensities are positive.
      ##Both norm exp are implemented in Limma for DNA micro array data
      Data1 <- Data1 %>%
        mutate(FMedianBG_correct = !!genepix_vars$FG-!!genepix_vars$BG)
      E <- as.matrix(Data1[['FMedianBG_correct']])
      rownames(E) <- rownames(Data1)

      ## here we use Here "saddle" gives the saddle-point approximation to 
      #maximum likelihood from
      # Ritchie et al (2007) and improved by Silver et al (2009) 
      #--> check limma for details
      ## can we use offset--> updates
      bg_correct <-
        limma::backgroundCorrect.matrix(
          E = E,
          method = "auto",
          offset = 0,
          printer = NULL,
          normexp.method = "saddle",
          verbose = TRUE
        )
      bg_correct <- data.frame(FMedianBG_correct = bg_correct)

      ## select the important variables and join with the background 
      #corrected data
      Data1 <- Data1 %>%
        dplyr::select(
          sampleID,
          sample_array_ID,
          antigen = Name,
          FMedian = !!genepix_vars$FG,
          BGMedian = !!genepix_vars$BG,
          Block,
          Column,
          Row
        ) %>%
        bind_cols(bg_correct) %>%
        group_by(sampleID, antigen) %>%
        dplyr::mutate(replicate = 1:n())
    }
    #Data1 <- Data1 %>% rename(F635MedianB635=F635.Median...B635)

    return(Data1)
  }




#' Merge sample ID with the array data
#' @param iden A character indicating the name of the object to be used under
#' data_files.
#' @param data_files A list of data objects with names utilised by iden.
#' @param genepix_vars A list of specific definitions of the experiment design.
#'  See \code{\link{array_vars}}.
#' @param method A description of the background correction to be used. 
#' See \code{\link{bg_correct}}.
#' @description  A generic function that merges the protein data with the
#'  sample identifiers or sample names. If the file
#' does not have sample identifiers the function generates it automatically.
#' @return a data frame merged with corresponding sample ID's.
#' The sample ID are specified in the sample ID files
#' @export
#' @import dplyr
#' @importFrom purrr set_names
#' @examples
#' ## Not run:
#' ### Define the genepix_vars
#' genepix_vars <- array_vars(
#'   channel = "635",
#'   chip_path = system.file("extdata", "array_data/machine1/",
#'    package="protGear"),
#'   totsamples = 21,
#'   blockspersample = 2,
#'   mig_prefix = "_first",
#'   machine = 1,
#'   ## optional
#'   date_process = "0520"
#' )
#'
#' ## the path where the micro-array data is located
#' data_path <- paste0(genepix_vars$chip_path)
#' filenames <- list.files(genepix_vars$chip_path,
#'                         pattern = "*.txt$|*.gpr$", full.names = FALSE
#' )
#' ## create a list of all the files
#' data_files <- purrr::map(
#'  .x = filenames,
#'   .f = read_array_files,
#'   data_path = data_path,
#'   genepix_vars = genepix_vars
#' )
#' data_files <- purrr::set_names(data_files, 
#' purrr::map(filenames, name_of_files))
#' ## merge the lab data with samples and perform bg correction
#' merge_sampleID(iden = "KK2-06", data_files = data_files,
#'                genepix_vars =genepix_vars,method = "subtract_global" )
#' ## End(Not run)
merge_sampleID <- function(iden, data_files, genepix_vars, method)
{
  ## read in the sample ID files
  ## this can be pulled from a mysql table
  if (file.exists(file.path(genepix_vars$sampleID_path,
                            paste0(iden , ".csv")))) {
    arraynames <-
      read.csv(
        file.path(genepix_vars$sampleID_path, paste0(iden , ".csv")) ,
        header = TRUE ,
        stringsAsFactors = FALSE ,
        colClasses = "character"
      )
  } else{
    warning(iden,
            ".csv Not found in the sampleID files",
            genepix_vars$sampleID_path)
    arraynames <-
      data.frame(
        v1 = (1:genepix_vars$totsamples) ,
        v2 = paste0("SID_gen", 1:genepix_vars$totsamples),
        barcode = iden
      )
  }

  ## replicate to the number of blocks
  ## make sure the block is arranged before merging with the data file
  arraynames <- arraynames %>%
    dplyr::select(v1, v2) %>%
    arrange(as.numeric(v1))

  ## capture errors for same sample ID in a slide
  if (length(unique(arraynames$v2)) < genepix_vars$totsamples) {
    sink("log_replicates.txt" , append = TRUE)
    warning("Most likely there is a repeated sample name for " , iden)
    sink()
    arraynames <- arraynames %>%
      group_by(v2) %>%
      mutate(index = 1:n()) %>%
      mutate(v2 = ifelse(index > 1, paste0(v2, "_", index), v2)) %>%
      select(-index)
  }


  ## get the data from the loop
  ## extract the specific data from the data files
  data <- data_files[[iden]]

  ## pick the spots per block from the data file
  spotsperblock <- table(data$Block)[[1]]
  sampleID <-
    rep(arraynames$v2, each = spotsperblock * genepix_vars$blockspersample)
  sample_array_ID <-
    rep(arraynames$v1, each = spotsperblock * genepix_vars$blockspersample)


  Data1 <- data %>%
    # assign respective sample number to each row
    mutate(
      sample = rep(
        1:genepix_vars$totsamples,
        each = spotsperblock * genepix_vars$blockspersample
      ),
      # Bring in the sampleIDs..192 each sample
      sampleID = sampleID,
      sample_array_ID = sample_array_ID,
      # Abit of formating the Antigen names and concentration
      Name = gsub(':', '', Name),
      Name = gsub('\n', '', Name),
      Name = gsub(' ', '', Name)
    )  %>%
    ##remove uneccessary concs
    ## filter(!grepl('Landmark|Buffer|IgG', Name))   %>%
    # group by iden and antigen name
    group_by(sampleID, Name)

  ## DO the background correction
  ## specify the
  Data1 <- bg_correct(iden, Data1 , genepix_vars, method = method)
  Data1 <- Data1 %>%  mutate(iden = iden)
  return(Data1)
}



#' Read a gpr file to visualize
#'
#' @param infile a .gpr file to be used to visualize the expression intensities
#'  of the slide spots
#'
#' @return a data frame to visualize the background or foreground values
#' @export
#' @importFrom  data.table fread
#' @examples
#' ## Not run:
#' read_array_visualize(infile = system.file("extdata",
#' "/array_data/machine1/KK2-06.txt", package="protGear"))
#' ## End(Not run)
read_array_visualize <- function(infile) {
  x <- grep('Block.*Column|Column.*Block', readLines(infile))
  # d_f <- read.table(inFile$datapath,skip=x-1, header = T)
  d_f <- data.table::fread(infile, skip = x - 1, header = TRUE)
  return(d_f)
}

#' Visualize the slide mimicking the original scan image.
#'
#' @param infile a .gpr file to be used to visualize the expression 
#' intensities of the slide spots
#' @param MFI_var the MFI variable to plot, can be either the
#'  background or foreground value
#' @param d_f  a data frame with array data
#' @param interactive a logical to specify whether an interactive 
#' graph is returned or not
#'
#' @import htmltools ggplot2
#' @importFrom plotly ggplotly
#' @return A ggplot of slide foreground values
#' @export
#'
#' @examples
#' ## Not run:
#' visualize_slide(
#' infile = system.file("extdata", "/array_data/machine1/KK2-06.txt",
#'  package="protGear"),
#' MFI_var = "B635 Median"
#' )
#' ## End(Not run)
visualize_slide <-
  function(infile,
           MFI_var,
           interactive = FALSE,
           d_f = NA) {
    ## d_f only used for the shiny app
    if (is.na(d_f)) {
      d_f <- read_array_visualize(infile)

      d_f <- d_f %>%
        group_by(Block) %>%
        mutate(
          meanX = mean(X),
          meanY = mean(Y),
          maxY = max(Y),
          maxX = max(X),
          minY = min(Y),
          minX = min(X)
        )
    }
    ## define mid points to put the block labels
    MFI_var_sys <- rlang::sym(MFI_var)
    mid <- median(log(d_f[[MFI_var]]))
    labels <- sprintf(
      "<b>%s</b><br> MFI= %s ",
      d_f$Name,
      formatC(d_f$`F635 Median`, format = "d", big.mark = ",")
    ) %>%
      lapply(htmltools::HTML)

    point_size <- 0.5
    if (interactive == FALSE)
      point_size <- 1
    ## plot the visual slide
    p <- ggplot(d_f, aes(x = X, y = -Y, text = labels)) +
      #geom_rect(aes(xmin = minX, xmax = maxX, ymin = -minY, ymax = -maxY),
      #color = "black",alpha=0.0001,fill="blue") +
      geom_point(size = point_size, 
                 aes_string(colour = sprintf("log(`%s`)", MFI_var))) +
      theme_void() +
      theme(legend.position = "none")  +
      scale_color_gradient2(
        midpoint = mid,
        low = "blue",
        mid = "white",
        high = "red",
        space = "Lab"
      ) +
      geom_text(aes(
        x = meanX,
        y = -meanY,
        label = paste("Block", Block)
      ),
      color = "black",
      size = 2)

    if (interactive == FALSE) {
      return(p)
    } else if (interactive == TRUE) {
      p <- ggplotly(p, tooltip = 'text')
      return(p)
    }
  }


#' Visualize the slide mimicking the original scan image using a 2d plot.
#'
#' @param infile - a .gpr file to be used to visualize the expression 
#' intensities of the slide spots
#' @param MFI_var the MFI variable to plot, can be either the 
#' background or foreground value
#' @param d_f a data frame with array data
#'
#' @return A 2d plot of either the background or foreground values
#' @export
#' @import ggplot2
#' @examples
#' ## Not run:
#' visualize_slide_2d(
#' infile = system.file("extdata", "/array_data/machine1/KK2-06.txt", 
#' package="protGear"),
#' MFI_var = "B635 Median"
#' )
#' ## End(Not run)
visualize_slide_2d <- function(infile, MFI_var , d_f = NA) {
  if (is.na(d_f)) {
    d_f <- read_array_visualize(infile)

    d_f <- d_f %>%
      group_by(Block) %>%
      mutate(
        meanX = mean(X),
        meanY = mean(Y),
        maxY = max(Y),
        maxX = max(X),
        minY = min(Y),
        minX = min(X)
      )
  }


  mid <- median(log(d_f[[MFI_var]]))
  ggplot(data = d_f,
         aes_string(
           x = 'X',
           y = sprintf("-%s", 'Y'),
           #-Y,
           z =  sprintf("log(`%s`)", MFI_var)
         )) + #log(`F635 Median`)
    ## we have 24 vs 8 per block
    #stat_summary_2d(fun = median ,binwidth = c(40,120)) +
    stat_summary_2d(fun = median) +
    scale_fill_gradient2(
      midpoint = mid,
      low = "blue",
      mid = "white",
      high = "red",
      space = "Lab"
    ) +
    theme_void() +
    theme(legend.position = "none") +
    geom_text(aes(
      x = meanX,
      y = -meanY,
      label = paste("Block", Block)
    ),
    color = "black",
    size = 4)
}

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(ggpubr)
library(gtools)
library(purrr)
library(scales)
library(pheatmap)
library(data.table)
library(gridExtra)
library(png)
library(knitr)
library(grid)
library(styler)
library(FactoMineR)
library(factoextra)
library(magick)
library(rlang)
library(GGally)
library(ggplotify)
library(remotes)
library(dplyr)
library(tidyr)
knitr::opts_chunk$set(echo = TRUE, message=FALSE,warning = FALSE,
                      fig.align = 'center',
                      dev = "png", 
                       tidy='styler', tidy.opts=list(strict=TRUE))


## Color Format
colFmt <- function(x,color) {
  
  outputFormat <- knitr::opts_knit$get("rmarkdown.pandoc.to")
  
  if(outputFormat == 'latex') {
    ret <- paste("\\textcolor{",color,"}{",x,"}",sep="")
  } else if(outputFormat == 'html') {
    ret <- paste("<font color='",color,"'>",x,"</font>",sep="")
  } else {
    ret <- x
  }

  return(ret)
}




## ----install_package, eval=FALSE----------------------------------------------
# ## install from BioConductor
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install('protGear')

## ----call_protGear------------------------------------------------------------
## load the package 
suppressWarnings(library(protGear))

## ----array_params-------------------------------------------------------------
## specify the the parameters to process the data
genepix_vars <- array_vars(channel="635" ,
                           chip_path = system.file("extdata/array_data/", package="protGear"),
                           totsamples = 21,
                           blockspersample = 2,
                           sampleID_path = system.file("extdata/array_sampleID/", package="protGear"),
                           mig_prefix = "_first",
                           machine =1,
                           ## optional 
                           date_process = "0520")

## ----gpr_header---------------------------------------------------------------
header_gpr <- readLines(system.file("extdata/array_data/machine1/KK2-06.txt", package="protGear"),
                        n=40)
header_gpr <- gsub("\"", "", header_gpr[1:32])
header_gpr[1:32]

## ----slide_struct, fig.align='left'-------------------------------------------
visualize_slide(infile=system.file("extdata/array_data/machine1/KK2-06.txt", package="protGear"),
                MFI_var ='F635 Median' )


## ----slide_struct_2d, fig.align='left'----------------------------------------
visualize_slide_2d(infile =system.file("extdata/array_data/machine1/KK2-06.txt", package="protGear"), 
                   MFI_var ='B635 Median' )

## ----array_files--------------------------------------------------------------
#### read in all the datasets
### list all the file names under data folder
filenames <- list.files(file.path(genepix_vars$paths[[1]]), 
                        pattern="*.txt$|*.gpr$", full.names=FALSE)
#' @___________________read_in_the_files_with_the_text_data_from_the_chip_____________________________
### read all the data files and save them in a list 
data_path <- paste0(genepix_vars$paths[[1]],"/") 
data_files <- purrr::map(.x = filenames, 
                         .f = read_array_files,
                         data_path=data_path ,
                         genepix_vars=genepix_vars)
data_files <- set_names(data_files, purrr::map(filenames, name_of_files))

## ----bg_data------------------------------------------------------------------
## utilising the map package we process a number of files  under data_files list
dfs <- names(data_files) 
allData_bg <- purrr::map(.x=dfs, .f=extract_bg,data_files=data_files,genepix_vars)
allData_bg <- set_names(allData_bg, purrr::map(filenames, name_of_files))
allData_bg <- plyr::ldply(allData_bg)

## ----bg_vs_fg-----------------------------------------------------------------
p1 <- plot_FB(allData_bg,antigen_name="antigen",
              bg_MFI="BG_Median",FG_MFI="FBG_Median",log=FALSE)


p1


## ----bg_plot------------------------------------------------------------------
p2 <- plot_bg(df=allData_bg, x_axis="Block",bg_MFI="BG_Median",
        log_mfi=TRUE) 
p2

## ----bg_correct---------------------------------------------------------------
sample_ID_merged_dfs <- purrr::map(.x=dfs, .f=merge_sampleID ,data_files=data_files , 
                             genepix_vars, method="subtract_local")
sample_ID_merged_dfs <- set_names(sample_ID_merged_dfs, purrr::map(filenames, name_of_files))

## ----buffer_spots_data--------------------------------------------------------
buffer_transp <- purrr::map(.x=sample_ID_merged_dfs, .f=buffer_spots ,  buffer_spot="buffer")

buffer_transp <- set_names(buffer_transp, purrr::map(filenames, name_of_files))

buffers <- plyr::ldply(buffer_transp)
plot_buffer(buffers,buffer_names="antigen",buffer_mfi="FMedianBG_correct",slide_id=".id")


## ----replicates_plot----------------------------------------------------------
#' @________________________________calculated_cv_for_each_data_file_______________________________________
#' data without the selected mean for the best 2 CVs 
dataCV <- purrr::map(.x=sample_ID_merged_dfs, .f=cv_estimation ,lab_replicates=3  ,
                     cv_cut_off=20,
                     sampleID_var='sampleID', antigen_var='antigen' ,replicate_var='replicate' ,
                     mfi_var='FMedianBG_correct')

lab_replicates=3
dataCV <- set_names(dataCV, purrr::map(filenames, name_of_files))

aa <- plyr::ldply(dataCV)
GGally::ggpairs(aa,aes(color=cvCat_all) ,
        columns = paste(seq_len(lab_replicates)), title = "",  axisLabels = "show") +
  theme_light()
   

## ----cv_summary---------------------------------------------------------------


#' @________________________________summary_of_cv_for_each_sample________________________________________ 
#' creates summaries by cv's greater than 20 and less than 20

cv_cut_off <- 20
dataCV_sample <- purrr::map(.x=dataCV, .f=protGear::cv_by_sample_estimation , cv_variable="cvCat_all",
                            lab_replicates=3)
dataCV_sample <- set_names(dataCV_sample, purrr::map(filenames, name_of_files))
all_cv_sample <- plyr::ldply(dataCV_sample)


## ----cv_by_sample-------------------------------------------------------------
less_20 <- rlang::sym(paste0("CV <= ",cv_cut_off, "_perc"))
gt_20 <- rlang::sym(paste0("CV > ",cv_cut_off, "_perc"))

less_20_per <-  rlang::sym(paste0("% CV <=",cv_cut_off))
gt_20_per <-  rlang::sym(paste0("% CV >",cv_cut_off))
ggplot(all_cv_sample)+
  geom_violin(aes(.id,`CV <= 20_perc`, color="% CV =<20")) +
  geom_violin(aes(.id,`CV > 20_perc`, color="% CV >20")) +
  geom_violin(aes(.id,Others_perc,color="Others")) +
  ylab("% of CV") +
  theme_minimal() +
  ggtitle("% of CV >20 or <=20 for each slide all repeats considered") 

## ----best_reps----------------------------------------------------------------
#' @________________________________data_with_selected_best_2_CV_______________________________________ 
#' data with the selected mean for the best 2 CVs
dataCV_best2 <- purrr::map(.x=dataCV, .f=best_CV_estimation , slide_id="iden",lab_replicates=3,
                           cv_cut_off=20)

## give the names to the returned list
dataCV_best2 <- set_names(dataCV_best2, purrr::map(filenames, name_of_files))


dataCV_sample_best2 <- purrr::map(.x=dataCV_best2, .f=cv_by_sample_estimation , 
                                  cv_variable="best_CV_cat",lab_replicates=3)
dataCV_sample_best2 <- set_names(dataCV_sample_best2, purrr::map(filenames, name_of_files))
all_cv_sample_best2 <- plyr::ldply(dataCV_sample_best2)

## ----best_cv_plot-------------------------------------------------------------
## plot only the CV perccentages
ggplot(all_cv_sample_best2)+
  geom_violin(aes(.id,`CV <= 20_perc`, color="% CV =<20")) +
  geom_violin(aes(.id,`CV > 20_perc`, color="% CV >20")) +
  geom_violin(aes(.id,Others_perc,color="Others")) +
  ylab("% of CV") +
  theme_minimal() +
  ggtitle("% of CV >20 or <=20 for each slide") 

## ----tag_file-----------------------------------------------------------------
tag_file <- read.csv(system.file("extdata/TAG_antigens.csv", package="protGear"))
tag_antigens <- c("CD4TAG" , "GST", "MBP")
batch_vars <- list(machine="m1", day="0520")

## ----tag_glimpse--------------------------------------------------------------
tb1 <- data.frame(head(tag_file, n=10))
tb1 %>% 
  knitr::kable() 

## ----tag_subtract-------------------------------------------------------------
#' @________________________________subtract_the_tag_values_______________________________________ 
#'
## tag subtract 
## read in the KILCHip TAG file to substract GST-1, MBP -2 and  CD4TAG - 0 file 
dataCV_tag <- purrr::map(.x=dataCV_best2, .f=tag_subtract , 
                         tag_antigens=tag_antigens, mean_best_CV_var="mean_best_CV",tag_file=tag_file,
                         antigen_var='antigen',
                         batch_vars=batch_vars)
dataCV_tag <- set_names(dataCV_tag, purrr::map(filenames, name_of_files))
dataCV_tag <- plyr::ldply(dataCV_tag)

## ----before_after_tag---------------------------------------------------------
aaa <- dataCV_tag %>% 
  filter(TAG_name=="GST") 

aaa <- aaa %>% 
dplyr::select(.id,sampleID,antigen,mean_best_CV,mean_best_CV_tag)

aaa <- aaa %>% 
  gather(measure,mfi,-c(.id:antigen))

ggplot(aaa,aes(as.factor(antigen),mfi,color=measure))  +
  geom_boxplot(aes(fill=measure),alpha=0.5)+
  theme_light() +
  xlab("antigen name")+
  ggtitle("Before and after TAG subtraction") +
  theme(axis.text.x = element_text(angle = 45, hjust = 1))

## ----normalisation------------------------------------------------------------
df_to_normalise <-  dataCV_tag  %>%  ungroup() %>%  
  dplyr::select(slide=.id,sampleID,sample_array_ID,antigen,mean_best_CV) %>%  
  group_by(sampleID, slide)
df_to_normalise$sample_index <- group_indices(.data =df_to_normalise )

### 
to_normalise <- df_to_normalise %>%
 ungroup() %>% dplyr::select(-slide,-sampleID,-sample_array_ID) %>% 
  dplyr::select(antigen, sample_index, everything()) %>%  
  gather(variable, value, -(antigen:sample_index)) %>%
  unite(temp, antigen ) %>%  dplyr::select(-variable) %>%
  spread(temp, value) %>% 
  as.data.frame(.)

### get the row names of the machine data
row.names(to_normalise) <- to_normalise$sample_index
#batch_all <- as.factor(paste0(to_normalise$machine,"/",to_normalise$day))
#machines <- as.factor(to_normalise$machine)
#day_batches <- as.factor(to_normalise$day)

## create the matrix to normalise
matrix_antigen <-  to_normalise %>% 
   dplyr::select(-sample_index) %>% 
  as.matrix(.)


## create the matrix to hold the important parameters 
## in place of AMA1 you use one of your features or antigen
array_matrix <- df_to_normalise %>% 
  filter(antigen=="AMA1") %>% 
  ungroup() %>% 
   dplyr::select(sample_array_ID,sample_index,slide)

control_antigens <- c("CommercialHumanIgG","CD4TAG")


## ----normalised_df------------------------------------------------------------
normlise_df <- matrix_normalise(matrix_antigen, method = "vsn", array_matrix=array_matrix,
                       return_plot = TRUE,control_antigens=control_antigens)

normlise_df$plot_normalisation


## ----compare_methods----------------------------------------------------------
control_antigens <- c("CommercialHumanIgG","CD4TAG")
## no normalisation
normalise_list_none <- matrix_normalise(matrix_antigen=matrix_antigen, 
                                         method = "none", 
                                         array_matrix=array_matrix,
                                         return_plot = TRUE,
                                         control_antigens=control_antigens)
  names(normalise_list_none) <- c("matrix_antigen_none" ,"plot_none")
## log2 normalisation
  normalise_list_log <- matrix_normalise(matrix_antigen=matrix_antigen, 
                                           method = "log2", 
                                           array_matrix=array_matrix,
                                           return_plot = TRUE,
                                           control_antigens=control_antigens)
  names(normalise_list_log) <- c("matrix_antigen_log" ,"plot_log")
## vsn normalisation
   normalise_list_vsn <- matrix_normalise(matrix_antigen=matrix_antigen, 
                                           method = "vsn", 
                                           array_matrix=array_matrix,
                                           return_plot = TRUE,
                                           control_antigens=control_antigens)
    names(normalise_list_vsn) <- c("matrix_antigen_vsn" ,"plot_vsn")
  ## cyclic loess with log
     normalise_list_cyclic_loess_log <- matrix_normalise(matrix_antigen=matrix_antigen, 
                                                        method = "cyclic_loess_log", 
                                                        array_matrix=array_matrix,
                                                        return_plot = TRUE,
                                                        control_antigens=control_antigens)
    names(normalise_list_cyclic_loess_log) <- c("matrix_antigen_cyclic_loess_log" ,
                                                "plot_cyclic_loess_log")
    

     normalise_list_rlm <- matrix_normalise(matrix_antigen=matrix_antigen, 
                                                  method = "rlm", 
                                                  array_matrix=array_matrix,
                                                  return_plot = TRUE,
                                                  control_antigens=control_antigens)
    names(normalise_list_rlm) <- c("matrix_antigen_rlm" ,"plot_rlm")
    
  
    ## create a list after normalisation
 normalised_list <- c(normalise_list_none , 
                      normalise_list_log,
                      normalise_list_vsn,
                      normalise_list_cyclic_loess_log,
                      normalise_list_rlm)
  ##
 normalised_list_plot <- normalised_list[grepl("plot",names(normalised_list))]

  

## ----plot_comparison, fig.align='center', fig.width=12, fig.height=15---------
p <- do.call("grid.arrange", c(normalised_list_plot, ncol=2))

## ----heat_norm----------------------------------------------------------------
norm_df <- normlise_df$matrix_antigen_normalised
norm_df <- norm_df %>% 
   dplyr::select(-control_antigens)
p3 <- pheatmap::pheatmap(norm_df ,scale = "none", cluster_rows = FALSE,
                            main=paste('VSN',"Normalised Data"),
                            silent = TRUE)
#-------
## if you want to save the file
# p3 <- ggplotify::as.ggplot(p3)
# p <- p3 +  theme_void()
# ggsave(p ,
#          filename ="heatmap.PNG" ,
#          width = 16 , height = 12 , 
#          limitsize = FALSE,
#          dpi=200 )
#-------
p3

## ----pca, fig.align='center',fig.width=16,fig.height=12-----------------------
norm_df <- normlise_df$matrix_antigen_normalised
res_pca <- prcomp( norm_df, scale = TRUE)
var <- get_pca_var(res_pca)
vars_visualise=20
#Visualize the PCA
## individuals contributing to the PCA
p1 <- fviz_pca_ind(res_pca,
                    col.var = "contrib", # Color by contributions to the PC
                    gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
                    repel = TRUE     # Avoid text overlapping
  )+    theme_minimal()
  
# Select the top vars_visualise contributing variables
p2 <-fviz_pca_biplot(res_pca, label="var",
                    select.var = list(contrib = vars_visualise)) +
    theme_minimal()
  
  
# Total cos2 of variables on Dim.1 and Dim.2
p3 <-     fviz_cos2(res_pca, choice = "var", axes = 1:2 , top = vars_visualise)

  
# Color by cos2 values: quality on the factor map
p4 <-  fviz_pca_var(res_pca, col.var = "cos2",
               gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
               select.var = list(contrib = vars_visualise),
               repel = TRUE # Avoid text overlapping
  )

 
## combine the plots into one grid
## combine the plots into one grid
## combine the plots into one grid
p_pca <- gridExtra::grid.arrange(p1,p2,p3,p4, ncol=2 )
## If you want to save the file
# ggsave(p_pca ,
#          filename ="p_pca.PNG" ,
#          width = 16 , height = 12 , 
#           units = "in",
#          limitsize = FALSE,
#          dpi=300)
p_pca 

## ----shiny_launch, eval=FALSE-------------------------------------------------
# protGear::launch_protGear_interactive()

## -----------------------------------------------------------------------------
sessionInfo()

#library(shinySignals)   # devtools::install_github("hadley/shinySignals")
#library(bubbles)        # devtools::install_github("jcheng5/bubbles")


## document -->http://www.cryer.co.uk/file-types/a/atf/genepix_file_formats.htm
## http://www.cryer.co.uk/file-types/a/atf/genepix_file_formats.htm#example

## this loads packages from CRAN
pac_loads <- c('shiny','shinyFiles','DT','tidyverse','shinydashboard','shinyjs',
               'factoextra','FactoMineR','shinySignals','bubbles','ggplotify','shinythemes')


sapply(pac_loads, require, character.only = TRUE)
## this loads or installs packages from github
#pacman::p_load_gh("hadley/shinySignals","jcheng5/bubbles","GuangchuangYu/ggplotify")



#' Title
#'
#' @param input
#' @param output
#' @param session
#'
#' @return
#' @export
#'
#' @examples
shinyServer(function(input, output, session) {
  js$hidehead('none')
  wd_this <- getwd()
  volumes <- getVolumes()()
  dir.create("processed_data")

  shinyFiles::shinyDirChoose(input, "folderChoose",  roots = volumes, #c(home = wd_this),
                 session = session)
## this works where
sel_path <- reactive({
    return(print(parseDirPath(volumes, input$folderChoose)))
})

#'@_______________structure_of_array_data____________________________________
observe({ # called only once at app init
    updateTabItems(session, "structure_view", "structure")
})


#'@______________________load_intro_________________________
#show intro modal
observeEvent("", {
  showModal(modalDialog(
       includeHTML(system.file("shiny-examples/protGear_interactive/", "intro_text.html", package="protGear" ,
                             mustWork = TRUE)),
   
    easyClose = TRUE,
    footer = tagList(
      actionButton(inputId = "intro", label = "DISMISS (INTRODUCTION TOUR)", icon = icon("info-circle"))
    )
  ))
})


observeEvent(input$intro,{
  removeModal()
})

#'@_____________________________check_this___________________
# show intro tour
## to be activated
# observeEvent(input$intro,
#              ## check on radar example
#             # introjs(session, options = list("nextLabel" = "Continue",
#                                             # "prevLabel" = "Previous",
#                                             # "doneLabel" = "Alright. Let's go"))
#             showModal(
#               modalDialog(title = "Just clicked on intro tour!",
#                           p("Spanner in the works.Coming soon!!"))
#             )
#          #  shinyalert::shinyalert("Just clicked on intro tour!", "Spanner in the works.Coming soon!!", type = "warning")
# )


# use action buttons as tab selectors
update_all <- function(x) {
  updateSelectInput(session, "tab",
                    choices = c("Structure overview", "Overview image"),
                    label = "",
                    selected = x
  )
}

observeEvent(input$structure, {
  update_all("Structure overview")
})
observeEvent(input$structure_img, {
  update_all("Overview image")
})


# update confirm button

observeEvent(input$confirm, {
  updateButton(
    session,
    inputId = "confirm",
    label = "CONFIRM SELECTION",
    icon = icon("bar-chart-o"),
    style = "primary")
})

# hide the underlying selectInput in sidebar for better design
observeEvent("", {
  hide("tab")
})

# initiate reactive value storage
rv <- reactiveValues()

observeEvent(input$all_tabs, {
  # store last tab
  rv$last_tab <- rv$current_tab
  rv$current_tab <- input$all_tabs

  if (input$all_tabs == "Overview") {
  showModal(
    modalDialog(title = "Overview",
                headerText = strong("What to do"),
                icon = icon("question"),
                badgeStatus = NULL,
                notificationItem(
                  text = (steps$text[1]),
                  icon = icon("spinner")
                ),
                br(),
                notificationItem(
                  text = steps$text[2],
                  icon = icon("address-card")
                ),
                br(),
                notificationItem(
                  text = steps$text[3],
                  icon = icon("calendar")
                ),
                br(),
                notificationItem(
                  text = steps$text[4],
                  icon = icon("user-md")
                ),
                br(),
                notificationItem(
                  text = steps$text[5],
                  icon = icon("ambulance")
                ),
                br(),
                notificationItem(
                  text = steps$text[6],
                  icon = icon("flask")
                ),
                br(),
                notificationItem(
                  text = strong(steps$text[7]),
                  icon = icon("exclamation")
                ))
  )
    # immediately navigate back to previous tab
    updateTabsetPanel(session, "all_tabs",
                      selected = rv$last_tab)
  }## end if
})


#'@-------------------------------
## a button to load the file
output$gpr_file_load <- renderUI({
    fileInput("gpr_file", "Choose one of the GPR files",
              buttonLabel = "Browse...",
              accept = c(
                "text/csv",
                "text/comma-separated-values,text/plain",
                ".csv")

    )
    # shinyFileChoose(input, 'gpr_file', defaultRoot = 'wd',
    #                   filetypes = c('gpr', "txt"))
})


## read file for the gpr
gpr_file_reactive <- reactive({
  # input$file will be NULL initially. After the user selects
  # and uploads a file, it will be a data frame with 'name',
  # 'size', 'type', and 'datapath' columns.
  inFile <- input$gpr_file

  if (is.null(inFile)){
    return(NULL)
  }else{
    x <- grep('Block.*Column|Column.*Block', readLines(inFile$datapath))
    # d_f <- read.table(inFile$datapath,skip=x-1, header = T)
    d_f <- data.table::fread(inFile$datapath,skip=x-1, header=TRUE)
    d_f <- d_f %>%
      group_by(Block) %>%
      mutate(meanX=mean(X),meanY=mean(Y),maxY=max(Y),maxX=max(X),minY=min(Y),minX=min(X))
    return(d_f)
  }

})


## reading the gpr file
gpr_header_reactive <- reactive({
  #inFile <- input$gpr_file
  inFile <- input$gpr_file
  if (is.null(inFile))    return(NULL)
  header_gpr <- readGPRHeader(file=inFile$datapath)


  return(header_gpr)

})


## empty structure
empty_strucuture <- reactive({
  text <- paste("\nLoad one of the file that has MFI data\n",
               "       to visualize/define the structure ")
  plot_empty2 <- ggplot() +
    annotate("text", x = 4, y = 25, size=8, label = text) +
    theme_bw() +
    theme(panel.grid.major=element_blank(),
          panel.grid.minor=element_blank())
  return(plot_empty2)
})


## a plot for the structure of the arrays
output$structure_plot <- renderPlot({
  gpr_file_reactive <- gpr_file_reactive()

  if(!is.null(gpr_file_reactive)){
    gpr_header <- gpr_header_reactive()

    gpr_header_wl <- unlist(strsplit(gpr_header$Wavelengths,"\\r|\\n|\\t"))
    gpr_header_wl <- gsub('\"',"",gpr_header_wl, fixed=TRUE)
    gpr_header_wl <- gpr_header_wl[gpr_header_wl!=""]
      p <- ggplot(gpr_file_reactive, aes(x=X, y=-Y, colour=as.factor(Block))) +
        #geom_rect(aes(xmin = minX, xmax = maxX, ymin = -minY, ymax = -maxY),color = "black",alpha=0.0001,fill="blue") +
        geom_point(size=.1) +
        theme_void()+
        theme(legend.position = "none") +
        # scale_color_brewer(palette="Set3") +
        geom_text(aes(x=meanX, y=-meanY, label=paste("Block",Block)), color="black",size=4) +
        ggtitle(paste("Scanned at", gpr_header_wl ,"Wavelength(s)"))


    }else if(is.null(input$gpr_file)){
      p <- empty_strucuture()

    }else{
      p <- empty_strucuture()

    }
  return(p)
})



## select the variable to plot
## selectig the spatial variable to plot
output$select_spatial_var <- renderUI({
  gpr_header <- gpr_header_reactive()
  gpr_file_reactive <- gpr_file_reactive()

  if(!is.null(gpr_file_reactive)){
    var_names <- names(gpr_file_reactive)
    var_names <- var_names[grepl("[Mm]edian|[Mm]ean",var_names)]
    tagList(
      useShinyFeedback(), # include shinyFeedback  # inclusion here is ideal; b/c inside module
      selectInput(inputId = "select_spatial",
                  "Select the variable to visualize",
                  choices = var_names,
                  selected = var_names[grep(paste0("^[B]635.*Median$"),var_names)]
      )
    )
  }
})



output$select_spatial_type <- renderUI({
  graphs <- c("Point graph"='point',
              "Array 2D graph"='2d_array')
  # graphs_id <- c("bar_chart","ridge_plot")
  prettyRadioButtons(inputId="spatial_type",
                     label = 'Plot an array 2D plot or a point graph:',
                     choices =  graphs,
                     inline= TRUE, animation = "jelly",
                     status = "default",
                     shape = "curve",bigger = TRUE)

})


## a plot for the spatial structure of the arrays
output$spatial_structure_plot <- renderPlotly({
  gpr_file_reactive <- gpr_file_reactive()
  inFile <- input$gpr_file
  MFI_var <- input$select_spatial
  if(!is.null(gpr_file_reactive)){
    m <- list(
      l = 20,
      r = 50,
      b = 50,
      t = 50,
      pad = 20
    )

      p <- visualize_slide(infile=inFile,
                           MFI_var =MFI_var, interactive=TRUE ,d_f=gpr_file_reactive)
      p <- p  %>%
        layout(title = paste("Spatial visualization of", MFI_var ," MFI"),
               margin = m)
      #ggtitle(paste("Spatial visualization of", MFI_var ," MFI"))


  }else if(is.null(input$gpr_file)){
    p <- empty_strucuture()
    p <- ggplotly(p)
  }else{
    p <- empty_strucuture()
    p <- ggplotly(p)
  }
  return(p)
})




output$spatial_structure_plot_2d <- renderPlot({
  gpr_file_reactive <- gpr_file_reactive()
  inFile <- input$gpr_file
  MFI_var <- input$select_spatial
  if(!is.null(gpr_file_reactive)){


      p <- visualize_slide_2d(infile=inFile,
                              MFI_var =MFI_var,d_f=gpr_file_reactive)
      p <- p  + ggtitle(paste("Spatial visualization of", MFI_var ," MFI")) #%>% layout(title = paste("Spatial visualization of", MFI_var ," MFI"))


  }else if(is.null(input$gpr_file)){
    p <- empty_strucuture()

  }else{
    p <- empty_strucuture()

  }
  return(p)
})


## indicating the scanning wavelength
output$wavelength_struct <- renderInfoBox({
  gpr_header <- gpr_header_reactive()

  if(is.null(gpr_header)){
   infoBox("Select the a gpr/text",
            subtitle = paste0("LOAD A FILE") ,
            # icon = shiny::icon("user-md"),
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = TRUE)
  }else{
    gpr_header_wl <- unlist(strsplit(gpr_header$Wavelengths,"\\r|\\n|\\t"))
    gpr_header_wl <- gsub('\"',"",gpr_header_wl, fixed=TRUE)
    gpr_header_wl <- gpr_header_wl[gpr_header_wl!=""]
    infoBox("The file(s) were scanned at:", paste(gpr_header_wl,collapse = ';'),
            subtitle = paste0("Wavelength(s)") ,
            icon = shiny::icon("check-circle"),
            color = "orange",width = 4,fill = TRUE)
  }

})

## miniarray
output$total_miniarray_struct <- renderUI({
  gpr_header <- gpr_header_reactive()
  if(!is.null(gpr_header)){
    tagList(
      useShinyFeedback(), # include shinyFeedback  # inclusion here is ideal; b/c inside module
      numericInput(inputId = "total_miniarray",
                   "Input the total number of mini-arrays as per the graph below . Each miniarray appears as a rectangle",
                   value = ''
      )
    )
  }
})

#### warning for the selecting the number of mini arrays
observeEvent(input$total_miniarray, {
  req(input$total_miniarray)
  gpr_file_reactive <- gpr_file_reactive()

  if(!is.null(gpr_file_reactive)){
    blocks <- max(gpr_file_reactive$Block, na.rm = TRUE)
  }else{
    blocks <- 0
  }

  if (input$total_miniarray >blocks |input$total_miniarray <0  ) {
      showFeedbackWarning(inputId = "total_miniarray",
                          text = "The value is more or less than available blocks",color = "#FF0000")
    } else {
      hideFeedback("total_miniarray")
    }
})


## observer event for foreground  name
observeEvent(input$select_F, {
  req(input$select_F)
  if (!grepl('^F',input$select_F)  ) {
    showFeedbackWarning(inputId = "select_F",
                        text = "The variable selected does not start with F",color = "#FF0000")
  } else {
    hideFeedback("select_F")
  }
})


## observer event for background name
observeEvent(input$select_B, {
  req(input$select_B)
  if (!grepl('^B',input$select_B)  ) {
    showFeedbackWarning(inputId = "select_B",
                        text = "The variable selected does not start with B",color = "#FF0000")
  } else {
    hideFeedback("select_B")
  }
})


## observer event for input of samples per block
observeEvent(input$blockspersample_param, {
  req(input$blockspersample_param)
  req(input$total_miniarray)
  gpr_file_reactive <- gpr_file_reactive()
  if(!is.null(gpr_file_reactive)){
    mini_arrays <- input$total_miniarray
    blocks <- max(gpr_file_reactive$Block, na.rm = TRUE)
    value_check <- blocks/mini_arrays

  }else{
    value_check <- 0
  }
  #alue_check=2
  if (input$blockspersample_param>value_check |input$blockspersample_param<value_check    ) {
    showFeedbackWarning(inputId = "blockspersample_param",
                        text = "This does not match well with the mini array",color = "#FF0000")
  } else {
    hideFeedback("blockspersample_param")
  }
})



## select the input for the foreground var
output$select_F_var <- renderUI({
  gpr_header <- gpr_header_reactive()
  gpr_file_reactive <- gpr_file_reactive()

  if(!is.null(gpr_file_reactive)){
    var_names <- names(gpr_file_reactive)
    var_names <- var_names[grepl("[Mm]edian",var_names)]
    tagList(
      useShinyFeedback(), # include shinyFeedback  # inclusion here is ideal; b/c inside module
      selectInput(inputId = "select_F",
                   "Select the FOREGROUND variable, mostly the variable starts with F followed by wavelength",
                  choices =var_names,
                  selected = var_names[grep(paste0("^[F]635.*Median$"),var_names)]
      )
    )
  }
})

## selectig the background variable
output$select_B_var <- renderUI({
  gpr_header <- gpr_header_reactive()
  gpr_file_reactive <- gpr_file_reactive()

  if(!is.null(gpr_file_reactive)){
    var_names <- names(gpr_file_reactive)
    var_names <- var_names[grepl("[Mm]edian",var_names)]
    tagList(
      useShinyFeedback(), # include shinyFeedback  # inclusion here is ideal; b/c inside module
      selectInput(inputId = "select_B",
                  "Select the BACKGROUND variable, the variable starts with B followed by wavelength",
                  choices = var_names,
                  selected = var_names[grep(paste0("^[B]635.*Median$"),var_names)]
      )
    )
  }
})


## input the numbe of blocks
output$blockspersample_output <- renderUI({
  gpr_file_reactive <- gpr_file_reactive()
 if(!is.null(input$total_miniarray)){
   mini_arrays <- input$total_miniarray
   blocks <- max(gpr_file_reactive$Block, na.rm = TRUE)
   value_check <- blocks/mini_arrays

 }else{
   value_check <- 1
 }
  if(!is.null(gpr_file_reactive)){

    tagList(
      useShinyFeedback(), # include shinyFeedback  # inclusion here is ideal; b/c inside module
      numericInput("blockspersample_param",
        "Input the number blocks per sample (rectangle) as per the graph below",
        value = value_check
      )
    )
  }


})


#'@_______________end_structure_of_array_data____________________________________


#'@_______________parameters_of_array_data____________________________________
#'
## input for the parameters
## select  the folders



output$dir_files <- renderUI({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
    if(!is.na(input$total_miniarray) | !is.null(input$total_miniarray)){
      shinyFiles::shinyDirButton("folderChoose",
                                 "Chose directory with the data",
                                 "Upload")
    }else{
      stop("Define the mini-array number to load files")
    }
  }else{
    stop("Define the array structure to load files")
  }
  #shinyalert("Oops!", "Something went wrong.", type = "error")
})

## check why disable button not working????
# observe({
#   useShinyjs()
#   #if(!is.null(input$gpr_file)){
#    # shinyjs::enable("folderChoose")
#   #} else {
#     shinyjs::disable("folderChoose")
#   #}
# })

## define the tab to load on intialisation
observe({ # called only once at app init
  updateTabItems(session, "data_load_tabs", "dashboard")
})


## observer event for input of the channel
observeEvent(input$channel_param, {
  req(input$channel_param)
  req(gpr_header_reactive())
  gpr_header <- gpr_header_reactive()
  gpr_header_wl <- unlist(strsplit(gpr_header$Wavelengths,"\\r|\\n|\\t"))
  gpr_header_wl <- gsub('\"',"",gpr_header_wl, fixed=TRUE)
  gpr_header_wl <- gpr_header_wl[gpr_header_wl!=""]
if (input$channel_param %ni% gpr_header_wl &!is.null(gpr_header)  ) {
  showFeedbackDanger(inputId = "channel_param",
                        text = "This does not match with what was entered on the structure",color = "#FF0000")
  } else {
    hideFeedback("channel_param")
  }
})


## observer event for input of the samples per slide
observeEvent(input$totsamples_param, {
  req(input$totsamples_param)
  req(input$total_miniarray)

  if (input$totsamples_param!=input$total_miniarray  ) {
    showFeedbackDanger(inputId = "totsamples_param",
                       text = "This does not match with what was entered on the structure. Expects each sample per mini-array",color = "#FF0000")
  } else {
    hideFeedback("totsamples_param")
  }
})

## if the intial file is not laoded the user has the freedom of selecting the wavelength
output$channel_output <- renderUI({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
    gpr_header <- gpr_header_reactive()
    gpr_header_wl <- unlist(strsplit(gpr_header$Wavelengths,"\\r|\\n|\\t"))
    gpr_header_wl <- gsub('\"',"",gpr_header_wl, fixed=TRUE)
    gpr_header_wl <- gpr_header_wl[gpr_header_wl!=""]
    if(!is.null(gpr_header)){
      tagList(
        useShinyFeedback(),
        selectInput("channel_param",
                    h4("Input the channel used for scanning the data"),
                    c("635" = "635",
                      "532" = "532",
                      "488" = "488",
                      "594 " = "594 "),
                    selectize = FALSE,
                    selected =   gpr_header_wl[[1]]))
    }
  }else {
    tagList(
      useShinyFeedback(),
      selectInput("channel_param",
                  h4("Input the channel used for scanning the data"),
                  choices =  c("635" = "635",
                               "532" = "532",
                               "488" = "488",
                               "594 " = "594 "),
                  selectize = FALSE)
    )
    }
})

## the total samples per slide
output$total_samples_output <- renderUI({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
    if(is.null(input$total_miniarray)){
      numericInput(
        "totsamples_param",
        h4("Input the  total number of samples per slide "),
        value = ''
      )
    }else{
      numericInput(
        "totsamples_param",
        h4("Input the  total number of samples per slide "),
        value = input$total_miniarray
      )
    }

  }else{
    numericInput(
      "totsamples_param",
      h4("Input the  total number of samples per slide "),
      value = ''
    )
  }


  })


output$mig_prefix_output <- renderUI({
  textInput(
    "mig_suffix_param",
    h4("Input the prefix used to identify the MIG files"),
    placeholder = "_first"
  )
})

output$machine_output <- renderUI({
  textInput(
    "machine_param",
    h4("Hybridization machine used before expression"),
    placeholder = "m1"
  )
})


output$date_process_output <- renderUI({
  textInput(
    "date_process_param",
    h4("Input the month day and month of processing"),
    placeholder = "0505"
  )
})




#'@_______________End_definition_of_parameters____________________________________
##################################################################################


#'@_______________Path_of_data_definition_and_working_directory____________________________________
# sel_path <- reactive({
#     return(print(parseDirPath( c(home = wd_this) , input$folderChoose)))
#   })
#
setWorkingDir<- eventReactive(input$folderChoose,{
    setwd(sel_path())
})


## output all the directory
output$batch_select <- renderUI({
  ## the path
  folder_choose <- parseDirPath(c(home = wd_this) ,input$folderChoose)
  paths <- list.dirs(path =sel_path() , full.names = TRUE)
  #paths <- paths[grepl("machine" , paths)]
  path_toutput <- gsub(paste0(getwd(),"/"),"", paths)

  selectInput(inputId='chip_path_param',
              label='Select the batch of data to be processed:',
              choices=path_toutput)

})


output$sampleID_select <- renderUI({
  ## the path
  folder_choose <- parseDirPath(c(home = wd_this) ,input$folderChoose)
  paths <- list.dirs(path =sel_path() , full.names = TRUE)
  #paths <- paths[grepl("sample[Ii][Dd]" , paths)]
  path_toutput <- gsub(paste0(getwd(),"/"),"", paths)

  selectInput('sampleID_path_param',
              'Select the folder with sampleID\'s :',
              path_toutput)

})



#'@_______________End_Path_of_data_definition____________________________________
  ##################################################################################

#'@_______________function_to_collect_all_the_parametes____________________________________
  ## specify the the parameters to process the data
genepix_vars <- reactive({
  genepix_vars_ls <- array_vars(channel=input$channel_param ,
                            FG= input$select_F,
                            BG=input$select_B,
                             chip_path = input$chip_path_param , #sel_path , #"data/array_data",
                             totsamples = input$totsamples_param,
                             blockspersample = input$blockspersample_param,  # 2,
                             sampleID_path = input$sampleID_path_param ,#"data/array_sampleID/",
                             mig_prefix = input$mig_suffix_param,
                             machine = input$machine_param ,#1,
                             date_process = input$date_process_param )
    return(genepix_vars_ls)
})

#'@_______________end_all_parameter_collection____________________________________
##################################################################################


#'@_______________count_all_files_in_the_path_selected____________________________________
#'
#'
## first check all files have sample ID

check_sampleIDs <- reactive({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
    if(input$sampleID_path_param=="" | is.null(input$sampleID_path_param ) ){
      check_result <- NULL
    }else(
      check_result <- check_sampleID_files(genepix_vars())

    )

  }else{
    check_result <- NULL
  }


})

output$check_sampleID_out <- renderInfoBox({

  missing_id <- length(check_sampleIDs())
  if(missing_id==0){
    infoBox("Select the directory",
            subtitle = paste0("with sampleID's") ,
           # icon = shiny::icon("user-md"),
           icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = TRUE)
  }else{
    infoBox("Files with missing sampleID .csv:", missing_id,
            subtitle = paste0("The files are saved in the log file") ,
            icon = shiny::icon("user-md"),
            color = "fuchsia",width = 4,fill = TRUE)
  }

})


## lists all the files in the selected directory
all_files <- reactive({
  inFile <- input$gpr_file

  if(!is.null(inFile)){
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
   filesInDir <- NULL
 }else if(!is.null(input$chip_path_param)){
   pth_full <- file.path(input$chip_path_param)
   pth <-input$chip_path_param
   #path_toutput <- gsub(paste0(getwd(),"/"),"", input$chip_path_param)
   ### list the files in the directory
   filesInDir <- list.files(pth_full)

 }
  }else{
    filesInDir <- NULL
  }
   return(filesInDir)
})


##total number of files
output$files_in_batch <- shinydashboard::renderInfoBox({
  total_files <- length(all_files())
  if(total_files==0){
    infoBox("Select the folder with array data",
            paste0("NA"),
            icon = icon("exclamation-triangle"),
            color = "red")
  }else if(total_files>0){
    infoBox("Total files in batch",
      paste0(total_files),
      subtitle = paste0("Files ending with .txt or .gpr"),
      icon = icon("list-ul"),
      color = "olive"
    )
  }


})

#'@_______________end_all_count_of_files____________________________________
##################################################################################


#'@_______________Display_some_key_info____________________________________
#'
#'
#### read in all the datasets
### list all the file names under data folder
filenames_reactive <- reactive({
  genepix_vars <- genepix_vars()
  filenames_return <- list.files(file.path(genepix_vars$chip_path),
                        pattern="*.txt$|*.gpr$", full.names=FALSE)


  return(filenames_return)
})


data_files_reactive  <-  reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    data_files <- NULL
  }else {
    genepix_vars <- genepix_vars()
    filenames <- filenames_reactive()

    data_files <- purrr::map(.x = filenames,
                             .f = read_array_files,
                             data_path=genepix_vars$chip_path ,
                             genepix_vars=genepix_vars)
    data_files <- set_names(data_files, purrr::map(filenames, name_of_files))
  }

  return(data_files)
})

dfs <- reactive({
  dfs_names <- names(data_files_reactive())
  return(dfs_names)
})




params_display <- reactive({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    params_df <- NULL
  }else{
    data_files <- data_files_reactive()
    genepix_vars <- genepix_vars()
    df_1 <- data_files[[1]]

    blocks <- max(df_1$Block)
    spotsperblock <- table(df_1$Block)[[1]]
    antigens <- length(unique(df_1$Name))
    antigen_list <- c(unique(as.character(df_1$Name)))
    antigen_list <- gsub(" ","",antigen_list)
    params_df <- list(blocks=blocks,
                      spotsperblock=spotsperblock,
                      antigens=antigens,
                      antigen_list=antigen_list)
  }
    }else{
      params_df <- NULL
  }

 return(params_df)
})


output$blocks_count <- renderInfoBox({
  #req(input$chip_path_param)
  params_df <- params_display()
  if(is.null(params_df) ){
    infoBox("Select the directory",
            subtitle = paste0("with array data") ,
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = TRUE)
  }else{
    infoBox("Each array file has : ", paste(params_df$blocks, "blocks"),
            subtitle = paste("with",params_df$spotsperblock, "spots per block") ,
            icon = icon("th",lib = "font-awesome"),
            color = "red",width = 4,fill = FALSE)
  }

})


output$antigens_count  <- renderInfoBox({
  params_df <- params_display()
  if(is.null(params_df)){
    infoBox("Select the directory",
            subtitle = paste0("with array data") ,
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = TRUE)
  }else{
    infoBox("Each array file has: ", params_df$antigens,
            subtitle = paste0("antigens") ,
            icon = icon("allergies",lib = "font-awesome"),
            color = "red",width = 4,fill = FALSE)
  }

})




## empty plot for adding to before data load
empty_plot <- reactive({
  inFile <- input$gpr_file
  if(!is.null(inFile)){

    if(is.na(input$total_miniarray)){
      x <- 0
    }else {x <- 1}

    if(x==0){
      text <- paste("\nPlot will appear after\n",
                   " a) defining the number of mini-arrays in design structure\n",
                   " b) selecting folder with data")
    }else  {
      text <- paste("\nPlot will appear after\n",
                   " a) selecting folder with data\n"
      )
    }

  }else{
    text <- paste("\nPlot will appear after\n",
                 " a) defining the array design structure then \n",
                 " b) selecting folder with data")
  }

   plot_empty <- ggplot() +
     annotate("text", x = 4, y = 25, size=8, label = text) +
     theme_bw() +
     theme(panel.grid.major=element_blank(),
           panel.grid.minor=element_blank())
   return(plot_empty)
})


empty_plot_error <- reactive({
  text <- paste("\nCheck data stucture well.\n",
               "       replicates definition or ")
  plot_empty <- ggplot() +
    annotate("text", x = 4, y = 25, size=8, label = text) +
    theme_bw() +
    theme(panel.grid.major=element_blank(),
          panel.grid.minor=element_blank())
  return(plot_empty)
})


empty_plot_NS <- reactive({
  text <- paste("\nNot selected.\n",
               " ")
  plot_empty <- ggplot() +
    annotate("text", x = 4, y = 25, size=8, label = text) +
    theme_bw() +
    theme(panel.grid.major=element_blank(),
          panel.grid.minor=element_blank())
  return(plot_empty)
})


empty_plot_no_rep <- reactive({
  text <- paste("\nThe experiment did not have\n",
               "       lab replicates or \n",
               "Select the no of lab replicates")
  plot_empty <- ggplot() +
    annotate("text", x = 4, y = 25, size=8, label = text) +
    theme_bw() +
    theme(panel.grid.major=element_blank(),
          panel.grid.minor=element_blank())
  return(plot_empty)
})

### plot of the buffer spots
## filenams
merged_dfs_reactive <- reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    sample_ID_merged_dfs <- NULL
  }else {
    genepix_vars <- genepix_vars()
    filenames <- filenames_reactive()
    data_files <-  data_files_reactive()
    dfs <- dfs()
    sample_ID_merged_dfs <- purrr::map(.x=dfs, .f=merge_sampleID ,data_files=data_files ,
                                         genepix_vars, method="subtract_local")
    sample_ID_merged_dfs <- set_names(sample_ID_merged_dfs, purrr::map(filenames, name_of_files))



  }

  return(sample_ID_merged_dfs)
})



# table of a sample dataset of the file
output$tbl_all_data <-  DT::renderDT({
  folder_choose <- parseDirPath(c(home = wd_this) ,input$folderChoose)
  if(!rlang::is_empty(sel_path) ){#input$folderChoose!=''
    all_datas <- list.files(sel_path() , recursive = TRUE)
    sample_ID_merged_dfs <-  merged_dfs_reactive()
    d_f <- plyr::ldply(sample_ID_merged_dfs)
    write_csv(d_f, paste("processed_data/raw_data-", Sys.Date(), ".csv", sep=""))
    df <- DT::datatable(d_f)

  }
  #options = list(lengthChange = FALSE,
  #  initComplete = JS('function(setting, json) { alert("done"); }'))
  return( df)
})

## download all data
output$download_Raw_Data <- downloadHandler(
  filename = function() {
    paste("raw_data-", Sys.Date(), ".csv", sep="")
  },
  content = function(file) {
    sample_ID_merged_dfs <-  merged_dfs_reactive()
    d_f <- plyr::ldply(sample_ID_merged_dfs)
    write_csv(d_f, file)
  }
)



dfs_reactive_bg_correct <- reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    sample_ID_merged_dfs <- NULL
  }else {
    genepix_vars <- genepix_vars()
    filenames <- filenames_reactive()
    data_files <-  data_files_reactive()
    dfs <- dfs()
    sample_ID_merged_dfs <- purrr::map(.x=dfs, .f=merge_sampleID ,data_files=data_files ,
                                       genepix_vars, method=input$bg_correct_1)
    sample_ID_merged_dfs <- set_names(sample_ID_merged_dfs, purrr::map(filenames, name_of_files))
}

  return(sample_ID_merged_dfs)
})


## function with bg correct input under CV tab
merged_dfs_reactive_bg_correct <- reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    sample_ID_merged_dfs <- NULL
  }else {
    genepix_vars <- genepix_vars()
    filenames <- filenames_reactive()
    data_files <-  data_files_reactive()
    dfs <- dfs()
   sample_ID_merged_dfs <- purrr::map(.x=dfs, .f=merge_sampleID ,data_files=data_files ,
                                         genepix_vars, method=input$bg_correct)
  sample_ID_merged_dfs <- set_names(sample_ID_merged_dfs, purrr::map(filenames, name_of_files))



  }

  return(sample_ID_merged_dfs)
})


all_data_reactive <- reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    allData <- NULL
  }else {
    dfs <- dfs()
    filenames <- filenames_reactive()
    data_files <-  data_files_reactive()
    genepix_vars <- genepix_vars()

    allData <- purrr::map(.x=dfs, .f=extract_bg,data_files=data_files,genepix_vars)
    allData <- set_names(allData, purrr::map(filenames, name_of_files))
    allData <- plyr::ldply(allData)
  }
 return(allData)
})


buffers_dfs_reactive <- reactive({
  if(is.null(input$chip_path_param) | input$chip_path_param==""){
    buffers <- NULL
  }else {
    sample_ID_merged_dfs <-  merged_dfs_reactive()
    filenames <- filenames_reactive()
    buffer_transp <- purrr::map(.x=sample_ID_merged_dfs, .f=buffer_spots)

    buffer_transp <- set_names(buffer_transp, purrr::map(filenames, name_of_files))

    buffers <- plyr::ldply(buffer_transp)
  }

  return(buffers)
})

 ## plot the buffer spots
output$data_process <- renderPlot({#DT::renderDT
  inFile <- input$gpr_file
  if(!is.null(inFile)){
  if(input$chip_path_param!="") {

    buffers <-  buffers_dfs_reactive()
    merged_dfs <- merged_dfs_reactive()
    dfs <- dfs()

    p <-  plot_buffer(buffers,buffer_names="antigen",buffer_mfi="FMedianBG_correct",slide_id=".id")

     }else{
     p <- empty_plot()

   }
  }else{
    p <- empty_plot()

  }
  return(p)
 })


output$data_box <- renderPlotly({
  inFile <- input$gpr_file
  if(!is.null(inFile)){
  if(input$chip_path_param!="") {
    sample_ID_merged_dfs <-  merged_dfs_reactive()

    merged_dfs <- plyr::ldply(sample_ID_merged_dfs)
    merged_dfs <- merged_dfs %>%
      filter(!grepl("[Bb][Uu][Ff][Ff][Ee][Rr]",antigen))

    antigens_list <- unique(merged_dfs[["antigen"]])
    antigens_list_plot <- antigens_list[input$slider_antigen[[1]]:input$slider_antigen[[2]]]

    merged_dfs <- merged_dfs %>%
      filter(antigen %in% antigens_list_plot)

    p <- plot_ly(merged_dfs, y = ~FMedianBG_correct, color = I("blue"),
                 alpha = 0.1, boxpoints = "suspectedoutliers",width=0.1 )
    p <- p %>%
      add_boxplot(x = ~antigen)  %>%
      layout(xaxis = list(title = "", tickangle = -45))
    p
  }else{
    p <- empty_plot()
    p <- ggplotly(p)
  p
  }
  }else{
    p <- empty_plot()
    p <- ggplotly(p)
    p
  }
})
#'@_______________End_display_some_key_info____________________________________

#'@_______________Background_correction____________________________________

## define which tab to load on opening the tab items
observe({ # called only once at app init
  updateTabItems(session, "bg_tabs", "bg_graphs")
})


output$slide_names_bg <- renderUI({
  if(input$chip_path_param!="") {
 dfs_names <- list('Slides'=dfs())
 dfs <- dfs()
 pickerInput(inputId="slide",
             label="Choose a slide:",
             choices= dfs,
             options = list(
               `actions-box` = TRUE,
               size = 10,
               `selected-text-format` = "count > 3"
             ),
             selected = paste(dfs),
             multiple = TRUE)
  }
})


output$select_log_MFI <- renderUI({
  graphs <- c("Log the MFI"=TRUE,
              "Raw MFI"=FALSE)
  # graphs_id <- c("bar_chart","ridge_plot")
  prettyRadioButtons(inputId="log_mfi",
                     label = 'Plot with log or raw MFI:',
                     choices =  graphs,
                     inline=TRUE, animation = "jelly",
                     status = "default",
                     shape = "curve",bigger = TRUE)

})

output$select_block_antigen <- renderUI({
  graphs <- c("Block"="Block",
              "Antigen"="antigen")
  awesomeRadio("block_antigen",
               label = 'Select block or antigen:',
               choices =  graphs,
               # choiceValues=all_var_prefix,
               #selected = character(0),
               inline=TRUE)

})

output$value_block <- renderUI({
  paste(input$block_antigen)
})


observe({
  x <- input$block_antigen
  y <- 'Background plot by'

  updateTextInput(session, "value_block", value = paste(y, x))


})

output$bg_plots <- renderPlotly({
  if(input$chip_path_param!="") {
    ## load the default parameted
    allData_bg <-  all_data_reactive()
    allData_bg <- allData_bg %>%
      filter(.id %in% input$slide)

    p<- plot_bg(df=allData_bg, x_axis=input$block_antigen,bg_MFI="BG_Median",
            log_mfi=input$log_mfi)
    p <- ggplotly(p) %>%  layout(height = 800)
    p
  }else{
    p <- empty_plot()
    p <- ggplotly(p)
    p
  }

})


output$fg_bg_plots <- renderPlotly({
  if(input$chip_path_param!="") {
    ## load the default parameted
    allData_bg <-  all_data_reactive()

    allData_bg <- allData_bg %>%
      filter(.id %in% input$slide)
    p <- plot_FB(allData_bg,antigen_name="antigen",bg_MFI="BG_Median",FG_MFI="FBG_Median")
    p <- ggplotly(p, tooltip = "text")%>%  layout(height = 800)
    p
  }else{
    p <- empty_plot()
    p <- ggplotly(p)
    p
  }

})




output$bg_correction_select_1 <- renderUI({
  bg_correct_approaches <- c("None"="none",
                             "Local bakground subtraction"="subtract_local",
                             "Global background subtraction"="subtract_global",
                             "Moving mininimum"="movingmin_bg",
                             "Half Moving mininimum"="minimum_half",
                             "Log-linear background correction"="edwards",
                             "Normal Exponential (normexp)"="normexp"
                             )
  selectInput(inputId = 'bg_correct_1',
              label = 'Background correction',
              selected = 'subtract_local',
              choices = bg_correct_approaches)
})


output$bg_correct_graphs <- renderPlot({
  if(input$chip_path_param!="") {
    bg_correct_df <- dfs_reactive_bg_correct()
    bg_correct_df <- plyr::ldply(bg_correct_df)
    all_df <- bg_correct_df %>%
      select(antigen,FMedian,FMedianBG_correct) %>%
      gather(var,mfi,-antigen)



    # New facet label names for var variable
   supp_labs <- c("FMedian before background correction", "FMedian after background correction")
    names(supp_labs) <- c("FMedian", "FMedianBG_correct")

    p <- ggplot(all_df,aes(x=antigen, y=mfi))+
      geom_boxplot()+
      facet_wrap(~var, nrow = 2,labeller = labeller(var = supp_labs))+
      geom_hline(yintercept = 0, color='red')+
      theme_light()+
    #  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)) +
      ylab("MFI") +
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),
            strip.background = element_rect(fill="black",size = 3),
            strip.text.x = element_text(size=12, color="white",
                                        face="bold.italic")) +
      ggtitle(paste0(input$bg_correct_1," background correction"))


  }else{
    p <- empty_plot()

  }
  return(p)
},height=800)

#'@_______________End_Background_correction____________________________________
#'
#'
#'
#'@_______________CV_calculation____________________________________

observe({ # called only once at app init
  updateTabItems(session, "cv_tabs", "bg_correct_cv")
})


output$bg_correction_drop_down <- renderUI({
  bg_correct_approaches <- c("None"="none",
                             "Local bakground subtraction"="subtract_local",
                             "Global background subtraction"="subtract_global",
                             "Moving mininimum"="movingmin_bg",
                             "Half Moving mininimum"="minimum_half",
                             "Log-linear background correction"="edwards",
                             "Normal Exponential (normexp)"="normexp")


  dropdownButton(
    tags$h3("List of Inputs"),
    selectInput(inputId = 'bg_correct', label = 'Background correction', choices = bg_correct_approaches),
    selectInput(inputId = 'ycol', label = 'Y Variable', choices = names(iris), selected = names(iris)[[2]]),
    sliderInput(inputId = 'clusters', label = 'Cluster count', value = 3, min = 1, max = 9),
  circle = TRUE, status = "danger", icon = icon("gear"), width = "300px",
    tooltip = tooltipOptions(title = "Click to see inputs !")
  )
})



output$bg_correction_select <-  renderUI({
  if(!is.null(input$bg_correct_1)){
    selected_bg <- input$bg_correct_1
  }else{
    selected_bg <- 'subtract_local'
  }
  bg_correct_approaches <- c("None"="none",
                             "Local bakground subtraction"="subtract_local",
                             "Global background subtraction"="subtract_global",
                             "Moving mininimum"="movingmin_bg",
                             "Half Moving mininimum"="minimum_half",
                             "Log-linear background correction"="edwards",
                             "Normal Exponential (normexp)"="normexp")
  selectInput(inputId = 'bg_correct',
              label = 'Background correction',
              selected = selected_bg,
              choices = bg_correct_approaches)
})

output$cv_value_select <- renderUI({
  sliderInput(inputId = 'cv_value', label = 'Select CV cut off', value = 20, min = 1, max = 100)
})

output$cv_ui <- renderText({ input$cv_value })


output$minimum_mfi_select <- renderUI({
  sliderInput(inputId = 'cv_value_mfi', label = 'Select MFI for CV cut off', value = 1000, min = 100, max = 10000)
})



output$replicates_select <- renderUI({
  sliderInput(inputId = 'lab_replicates',
              label = 'Select the number of lab replicates',
              value = 1, min = 1, max = 5)
})


data_CV_reactive <- reactive({
  if(input$chip_path_param!="") {
  sample_ID_merged_dfs <-  merged_dfs_reactive_bg_correct()
  filenames <- filenames_reactive()
  dataCV <- purrr::map(.x=sample_ID_merged_dfs, .f=cv_estimation ,lab_replicates=input$lab_replicates,
                       cv_cut_off=input$cv_value)

  dataCV <- set_names(dataCV, purrr::map(filenames, name_of_files))
  }else {
    dataCV <- NULL
  }
  return(dataCV)
})


data_CV_best2_reactive <- reactive({
  if(input$chip_path_param!="") {
    dataCV <- data_CV_reactive()
    filenames <- filenames_reactive()
    dataCV_best2 <- purrr::map(.x=dataCV, .f=best_CV_estimation , slide_id="iden" ,
                               lab_replicates=input$lab_replicates,cv_cut_off=input$cv_value)


# # give the names to the returned list
    dataCV_best2 <- set_names(dataCV_best2, purrr::map(filenames, name_of_files))
  }else {
    dataCV_best2 <- NULL
  }
  return(dataCV_best2)
})

output$cv_corr_plot <- renderPlot({
  dataCV <- data_CV_reactive()
  dataCV <- plyr::ldply(dataCV)
  if(input$chip_path_param!="") {
  if(max(dataCV$replicates) <=input$lab_replicates & input$lab_replicates!=1){

    ggpairs(dataCV,aes(color=cvCat_all) ,
            columns = paste(1:input$lab_replicates), title = "",  axisLabels = "show") +
      theme_light()
  }else if(input$lab_replicates==1){
    p <- empty_plot_no_rep()
    p
  }else{
    p <- empty_plot_error()
    p
  }
  }else{
    p <- empty_plot()
    p
  }

})

dataCV_all_sample_reactive <- reactive({
  if(input$lab_replicates>1) {
  dataCV <- data_CV_reactive()
  filenames <- filenames_reactive()

  dataCV_sample <- purrr::map(.x=dataCV, .f=cv_by_sample_estimation , cv_variable="cvCat_all" ,
                              lab_replicates=input$lab_replicates)
  dataCV_sample <- set_names(dataCV_sample, purrr::map(filenames, name_of_files))
  all_cv_sample <- plyr::ldply(dataCV_sample)
  }else {
  all_cv_sample <- NULL
  }
  return(all_cv_sample)
})



dataCV_sample_best2_reactive <- reactive({
  if(input$lab_replicates>1) {
    dataCV_best2 <- data_CV_best2_reactive()
    filenames <- filenames_reactive()

    dataCV_sample_best2 <- purrr::map(.x=dataCV_best2, .f=cv_by_sample_estimation , cv_variable="best_CV_cat" ,
                                      lab_replicates=input$lab_replicates)
    dataCV_sample_best2 <- set_names(dataCV_sample_best2, purrr::map(filenames, name_of_files))
    all_cv_sample_best2 <- plyr::ldply(dataCV_sample_best2)
  }else {
    all_cv_sample_best2 <- NULL
  }
  return(all_cv_sample_best2)
})


output$cv_violin_plot <- renderPlot({
  all_cv_sample <- dataCV_all_sample_reactive()
  cv_cut_off <- input$cv_value

  if(input$chip_path_param!="") {
    if(!is.null(all_cv_sample) & !plyr::empty(all_cv_sample)){
      ## plot only the CV perccentages
      less_20 <- rlang::sym(paste0("CV <= ",cv_cut_off, "_perc"))
      gt_20 <- rlang::sym(paste0("CV > ",cv_cut_off, "_perc"))

      less_20_per <-  rlang::sym(paste0("% CV <=",cv_cut_off))
      gt_20_per <-  rlang::sym(paste0("% CV >",cv_cut_off))

      p <- ggplot(all_cv_sample)+
        geom_violin(aes_string(".id",less_20,color=shQuote(less_20_per))) +
        geom_violin(aes_string('.id',gt_20, color=shQuote(gt_20_per))) +
        geom_violin(aes(.id,Others_perc,color="Others")) +
        ylab("% of CV") +
        theme_light() +
        ggtitle(paste0("% of CV >", input$cv_value ,"or <=",input$cv_value," for each sample all repeats considered"))


    }else if(plyr::empty(all_cv_sample)){
      p <- empty_plot_no_rep()

    }else{
      p <- empty_plot_error()

    }
  }else{
    p <- empty_plot()

  }
  return(p)
})


output$cv_violin_plot_best2 <- renderPlot({
  all_cv_sample_best2 <- dataCV_sample_best2_reactive()

  cv_cut_off <- input$cv_value

  if(input$chip_path_param!="") {
    if(!is.null(all_cv_sample_best2) & !plyr::empty(all_cv_sample_best2)){
      ## plot only the CV perccentages
      if('Others_perc' %ni% names(all_cv_sample_best2)){
        all_cv_sample_best2['Others_perc'] <- 0
        all_cv_sample_best2['Others_n'] <- 0
      }
      less_20 <- rlang::sym(paste0("CV <= ",cv_cut_off, "_perc"))
      gt_20 <- rlang::sym(paste0("CV > ",cv_cut_off, "_perc"))

      less_20_per <-  rlang::sym(paste0("% CV <=",cv_cut_off))
      gt_20_per <-  rlang::sym(paste0("% CV >",cv_cut_off))
      p <- ggplot(all_cv_sample_best2)+
        geom_violin(aes_string(".id",less_20,color=shQuote(less_20_per))) +
        geom_violin(aes_string('.id',gt_20, color=shQuote(gt_20_per))) +
        geom_violin(aes(.id,Others_perc,color="Others")) +
        ylab("% of CV") +
        theme_minimal() +
        ggtitle(paste0("% of CV >", input$cv_value ,"or <=",input$cv_value," for each sample all repeats considered"))
      p

    }else if(plyr::empty(all_cv_sample_best2)){
      p <- empty_plot_no_rep()
      p
    }else{
      p <- empty_plot_error()
      p
    }
  }else{
    p <- empty_plot()
    p
  }

})



# table of a sample dataset of the file
output$sample_best2_reactive_tbl <-  DT::renderDT({
  dataCV_sample_best2 <- dataCV_sample_best2_reactive()
 if(!rlang::is_empty(sel_path) ){#input$folderChoose!=""

   all_vars <- names(dataCV_sample_best2)
   percs <- all_vars[grepl("perc",all_vars)]

   divide_100 <- function(x, na.rm = FALSE) (x/100)
   dataCV_sample_best2 <- dataCV_sample_best2 %>% mutate_at(percs,divide_100 )

   sample_best2_df <-  datatable(dataCV_sample_best2)  %>%
     formatPercentage( columns = percs, 2) %>%
     formatStyle(
       columns = percs,
       background = styleColorBar(0:1, 'red'),
       backgroundSize = '100% 90%',
       backgroundRepeat = 'no-repeat',
       backgroundPosition = 'center'
     )
    sample_best2_df
 }else {
   sample_best2_df <- data.frame()
  }

})
#'@_______________End_CV_calculation____________________________________



#' @________________________________subtract_the_tag_values_______________________________________

output$tag_subtract_select <- renderUI({
  #materialSwitch(inputId = "tag_subtract_btn", label = "Tag", status = "danger")
  # switchInput(
  #   inputId = "tag_subtract_btn",
  #   value = FALSE,
  #   size = "mini",
  #   #label = "Purification TAGS",
  #   onLabel = "TAG",
  #   offLabel = "No TAG",
  #   onStatus = "success",
  #   offStatus = "danger",
  #   #labelWidth = "100px",
  #   width='auto'
  # )

  materialSwitch(inputId = "tag_subtract_btn",
                 label = "Select if purification TAG(s) was used",
                 right = TRUE,
                 value = FALSE,
                 status = "success"
                 )


})



output$tag_file_load <- renderUI({
  if(input$tag_subtract_btn==TRUE){
    fileInput("tag_file", "Choose CSV File with tag antigens",
              buttonLabel = "Browse...",
              accept = c(
                "text/csv",
                "text/comma-separated-values,text/plain",
                ".csv"))
  }else {
    warning("No purification TAGs used")
  }

 # shinyFileChoose(input, 'tag_file', defaultRoot = 'wd',
 #                   filetypes = c('csv', "txt"))
})


tag_file_reactive <- reactive({
  # input$file1 will be NULL initially. After the user selects
  # and uploads a file, it will be a data frame with 'name',
  # 'size', 'type', and 'datapath' columns. The 'datapath'
  # column will contain the local filenames where the data can
  # be found.
  inFile <- input$tag_file

  if (is.null(inFile))
    return(NULL)
  df_tag <- read_csv(inFile$datapath)
  return(df_tag)
})





output$tag_antigens_select <- renderUI({
  tag_file <- tag_file_reactive()
 if(!is.null(tag_file)) {
   params_df <- params_display()
   antigens <-   params_df$antigen_list

   if('TAG_name' %in% names(tag_file)){
     list_tag <- antigens[antigens %in% unique(tag_file$TAG_name)]
   }else{
     list_tag <- ""
   }
   antigen_list  <- list('Antigens'=antigens)
    selectInput(inputId="tag_antigens",
                  label="Select the TAG antigens:",
                  choices= antigen_list,
                  selected = list_tag ,
                  multiple = TRUE)
    }
})

output$sample_ID_select <- renderUI({
  if(input$chip_path_param!="") {

    tag_data <- tag_data_reactive()
     sample_list <- unique(tag_data$sampleID)
    sample_list  <- list('Antigens'=sample_list)
    selectInput(inputId="sample_tag",
                label="Choose a sample to compare TAG values:",
                choices= sample_list,
                # selected = "" ,
                multiple = FALSE)
  }
})


output$tag_antigen_radio_select <- renderUI({
  tag_antigens <- input$tag_antigens
  awesomeRadio("tag_antigen_radio",
               label = 'Select the antigen to visualize:',
               choices =  tag_antigens,
             selected = tag_antigens[[1]],
               inline=TRUE)

})




## tag subtracted data
dataCV_tag_reactive <- reactive({
  inFile <- input$tag_file
  if(input$chip_path_param!="" & !is.null(inFile) & input$tag_subtract_btn==TRUE ) {
    dataCV_best2 <- data_CV_best2_reactive()
    filenames <- filenames_reactive()
    tag_antigens <- input$tag_antigens
    tag_file <- tag_file_reactive()
    genepix_vars <- genepix_vars()
    batch_vars <- list(machine= genepix_vars$machine,
                       day =genepix_vars$date_process)


  dataCV_tag <- purrr::map(.x=dataCV_best2, .f=tag_subtract ,
                         tag_antigens=tag_antigens,
                         mean_best_CV_var="mean_best_CV",
                         tag_file=tag_file,
                         batch_vars=batch_vars)

  dataCV_tag <- set_names(dataCV_tag, purrr::map(filenames, name_of_files))
  dataCV_tag <- plyr::ldply(dataCV_tag)



  }else if(input$chip_path_param!="" & is.null(inFile) & input$tag_subtract_btn==FALSE) {
    dataCV_best2 <- data_CV_best2_reactive()
    filenames <- filenames_reactive()

    genepix_vars <- genepix_vars()
    batch_vars <- list(machine= genepix_vars$machine,
                       day =genepix_vars$date_process)
    dataCV_best2 <- set_names(dataCV_best2, purrr::map(filenames, name_of_files))
    dataCV_tag <- plyr::ldply(dataCV_best2)

    dataCV_tag <- dataCV_tag %>% ungroup() %>%
      dplyr::select(sampleID ,sample_array_ID, antigen , everything()) %>%
      mutate(mean_best_CV_tag=mean_best_CV,TAG_mfi=0,TAG=NA,TAG_name='') %>%
      mutate(machine=batch_vars[["machine"]] , day=batch_vars[["day"]])
  }else{
    dataCV_tag <- NULL
  }
  return(dataCV_tag)
})



dataCV_tag_specific_reactive <- reactive({
  dataCV_tag <- dataCV_tag_reactive()
  if(!is.null(dataCV_tag)){

    df <- dataCV_tag %>%
      filter(TAG_name==input$tag_antigen_radio)
  }else{
    df <- NULL
  }
 return(df)
})



output$antigen_tag_specific_select <- renderUI({
  df_tag_specific <- dataCV_tag_specific_reactive()
  antigens <- unique(df_tag_specific$antigen)
  antigen_list  <- list('Antigens'=antigens)
  pickerInput(inputId="tag_specific_antigens",
              label="Choose the TAG antigens to visualize:",
              choices= antigen_list,
              options = list(
                `actions-box` = TRUE,
                size = 10,
                `selected-text-format` = "count > 3"
              ),
              selected = paste(antigens),
              multiple = TRUE)
})



tag_data_reactive <- reactive({
  dataCV_tag <- dataCV_tag_reactive()
  if(!is.null(dataCV_tag)){
   tag_antigens <- input$tag_antigens
    tag_data_wide <- dataCV_tag %>%
      filter(antigen==tag_antigens[[2]])
    tag_data_wide <- tag_data_wide %>%
      select(.id, sampleID,tag_antigens) %>%
      gather(tag_antigen, MFI, -c(.id, sampleID))
  }else{
    tag_data_wide <- NULL
  }
  return(tag_data_wide)
})


### create a table and download the dataCV corrected data
## display the data
# table of a sample dataset of the file
output$tbl_data_cv<-  DT::renderDT({
  folder_choose <- parseDirPath(c(home = wd_this) ,input$folderChoose)
  if(!rlang::is_empty(sel_path)){#input$folderChoose!=""){
    dataCV_tag <- dataCV_tag_reactive()
    dataCV_tag <- dataCV_tag %>% mutate_if(is.numeric, ~round(., 3))

    write_csv(dataCV_tag, paste("processed_data/data_CV-", Sys.Date(), ".csv", sep=""))

    df <- DT::datatable(dataCV_tag)


  }
  #options = list(lengthChange = FALSE,
  #  initComplete = JS('function(setting, json) { alert("done"); }'))
  return(df)
}, filter = 'top')

## download all data
output$download_dataCV <- downloadHandler(
  filename = function() {
    paste("raw_data-", Sys.Date(), ".csv", sep="")
  },
  content = function(file) {
    dataCV_tag <- dataCV_tag_reactive()

    write_csv(dataCV_tag, file)
  }
)



output$tag_box <- renderPlot({
  tag_data <- tag_data_reactive()
  if(input$chip_path_param!="") {
    if(!is.null(tag_data)){
    p <- ggplot(tag_data, aes(tag_antigen,MFI)) +
        geom_violin(alpha=0.3) +
        geom_boxplot(width = 0.2) +
        geom_jitter(aes(color=.id),
                    position = position_jitter(width = .15, height=-0.7),
                    size=2)+
        theme_light()
    p

    }else{
      p <- empty_plot_error()
      p
    }
  }else{
    p <- empty_plot()
    p
  }
})

output$tag_box_sample <- renderPlot({
  tag_data <- tag_data_reactive()
  if(input$chip_path_param!="") {

    if(!is.null(tag_data)){
      tag_data <- tag_data %>%
        filter(sampleID %in% input$sample_tag)
      p <- ggplot(tag_data, aes(tag_antigen,MFI)) +
        geom_violin(alpha=0.3) +
        geom_boxplot(width = 0.2) +
        geom_jitter(aes(color=.id),
                    position = position_jitter(width = .15, height=-0.7),
                    size=2)+
        theme_light()
      p

    }else{
      p <- empty_plot_error()
      p
    }
  }else{
    p <- empty_plot()
    p
  }
})



output$tag_antigens_box  <- renderPlot({
  dataCV_tag_specific <- dataCV_tag_specific_reactive()
  if(!is.null(input$tag_specific_antigens)){
    df_plot <- dataCV_tag_specific %>%
      dplyr::select(.id,sampleID,antigen,before_tag=mean_best_CV,after_tag=mean_best_CV_tag) %>%
      filter(antigen %in% input$tag_specific_antigens)
  }else{
    df_plot <- dataCV_tag_specific %>%
      dplyr::select(.id,sampleID,antigen,before_tag=mean_best_CV,after_tag=mean_best_CV_tag)
  }
  df_plot <- df_plot %>%
    gather(measure,mfi,-c(.id:antigen))
  ggplot(df_plot,aes(as.factor(antigen),mfi,color=measure))  +
    geom_boxplot(aes(fill=measure),alpha=0.5)+
    theme_light() +
     xlab("antigen name")+
    ggtitle("Before and after TAG subtraction") +
    theme(axis.text.x = element_text(angle = 45, hjust = 1))
})

## info boxes to show the selected paramaeter
output$bg_correct_infobox <- output$bg_correct_infobox2 <- renderInfoBox({
  if(is.null(input$bg_correct)){
    infoBox("Select the background correct approach",
            subtitle = paste0("") ,
            # icon = shiny::icon("user-md"),
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = FALSE)
  }else{
    bg_correct_approaches <- c("None"="none",
                               "Local bakground subtraction"="subtract_local",
                               "Global background subtraction"="subtract_global",
                               "Moving mininimum"="movingmin_bg",
                               "Half Moving mininimum"="minimum_half",
                               "Log-linear background correction"="edwards",
                               "Normal Exponential (normexp)"="normexp")
    bg_app <- names(which(bg_correct_approaches == input$bg_correct))
    infoBox("Background correction",bg_app ,
            subtitle = paste0("") ,
            icon = shiny::icon("layer-group"),
            color = "aqua",width = 4,fill = FALSE)
  }

})


output$tag_infobox <- output$tag_infobox2 <- renderInfoBox({
  if(is.null(input$tag_antigens)){
    infoBox("Select the TAG antigens",
            subtitle = paste0("") ,
            # icon = shiny::icon("user-md"),
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = FALSE)
  }else{
    antigens <- paste(input$tag_antigens,collapse=" ,")
    infoBox("TAG antigens",antigens ,
            subtitle = '' ,
            icon = shiny::icon("tag"),
            color = "aqua",width = 4,fill = FALSE)
  }

})


output$cv_infobox <- output$cv_infobox2 <- renderInfoBox({
  if(is.null(input$cv_value)){
    infoBox("Select the CV cutoff",
            subtitle = paste0("") ,
            # icon = shiny::icon("user-md"),
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = FALSE)
  }else{
    infoBox("Selected CV cutoff",paste(input$cv_value,"% with") ,
            subtitle = paste0(input$lab_replicates," lab replicates") ,
            icon = shiny::icon("cut"),
            color = "aqua",width = 4,fill = FALSE)
  }

})

### hide the menu item for TAG visuals
### hide the menu item for TAG visuals
output$tag_plots <- renderMenu({
  req(input$tag_subtract_btn)
  if(input$tag_subtract_btn == TRUE)

    menuItem("Tag subtract", tabName = "tag_plots", icon = icon("bell")#,
             #menuSubItem("No option",tabName="RO_00"),
             #menuSubItem("Option 1",tabName="RO_01")
    )
})







#'
#'
#'@________________________________end_subtract_the_tag_values_______________________________________
#'
#'
#'@________________________________Normalisation_______________________________________
observe({ # called only once at app initialisation
  updateTabItems(session, "norm_tabs", "normalisation")
})


output$normalisation_select <- renderUI({
  normalisation_approaches <- c("Log2 Normalisation"="log2",
                                "VSN"="vsn",
                                "Cyclic Loess"="cyclic_loess",
                                "Cyclic Loess (log)"="cyclic_loess_log",
                                "RLM Normalisation"="rlm")
  selectInput(inputId = 'normalisation_method',
              label = 'Select normalisation method',
              selected = 'log2',
              choices = normalisation_approaches)
})

## select the control antigens to use with RLM
output$rlm_antigens_select <- renderUI({
  params_df <- params_display()
  antigens <-   params_df$antigen_list
  antigen_list  <- list('Antigens'=antigens)
  if(input$normalisation_method=="rlm") {
    selectInput(inputId="rlm_antigens",
                label="Choose the RLM antigens:",
                choices= antigen_list,
                multiple = TRUE)
  }
})


output$normalisation_infobox <- renderInfoBox({
  if(is.null(input$normalisation_method)){
    infoBox("Select the normalisation method",
            subtitle = paste0("") ,
            # icon = shiny::icon("user-md"),
            icon = icon("exclamation-triangle"),
            color = "red",width = 4,fill = FALSE)
  }else{
    normalisation_approaches <- c("Log2 Normalisation"="log2",
                                  "VSN"="vsn",
                                  "Cyclic Loess"="cyclic_loess",
                                  "Cyclic Loess (log)"="cyclic_loess_log",
                                  "RLM Normalisation"="rlm")
    normal_app <- names(which(normalisation_approaches == input$normalisation_method))
    infoBox("Normalisation approach",paste(normal_app) ,
            subtitle = "" ,
            icon = shiny::icon("cut"),
            color = "aqua",width = 4,fill = FALSE)
  }

})


## normalisation function
to_normalise_reactive <- reactive({
  dataCV_tag <- dataCV_tag_reactive()
  if(!is.null(dataCV_tag)){
    batch_vars_name <- c("machine","day")
    df_to_normalise <-  dataCV_tag  %>%
      ungroup() %>%
      select(slide=.id,sampleID,sample_array_ID,antigen,batch_vars_name,mean_best_CV) %>%
      group_by(sampleID,machine, slide)

    df_to_normalise$sample_index <- group_indices(.data =df_to_normalise )

    ###
    to_normalise <- df_to_normalise %>%
      ungroup() %>% select(-slide,-sampleID,-sample_array_ID) %>%
      select(antigen, machine,day,sample_index, everything()) %>%
      gather(variable, value, -(antigen:sample_index)) %>%
      unite(temp, antigen ) %>%
      select(-variable) %>%
      spread(temp, value)

    row.names(to_normalise) <- to_normalise$sample_index
  }else {
    to_normalise <- NULL
  }
  return(to_normalise)
})

array_matrix_reactive <- reactive({
  dataCV_tag <- dataCV_tag_reactive()
  if(!is.null(dataCV_tag) & !is.null(input$rlm_antigens)){
    batch_vars_name <- c("machine","day")
    df_to_normalise <-  dataCV_tag  %>%
      ungroup() %>%
      dplyr::select(slide=.id,sampleID,sample_array_ID,antigen,batch_vars_name,mean_best_CV) %>%
      group_by(sampleID,machine, slide)

    df_to_normalise$sample_index <- group_indices(.data =df_to_normalise )

    array_matrix <- df_to_normalise %>%
      filter(antigen==input$rlm_antigens[[1]]) %>%
      ungroup() %>%
      select(sample_array_ID,sample_index,slide)
  }else{
    array_matrix <- NULL
  }
  return(array_matrix)
})

## returning non normalised data
non_normalised_list_reactive <- reactive({
  to_normalise <- to_normalise_reactive()
  params_df <- params_display()
  antigens <-   params_df$antigen_list
  antigens <- antigens[antigens %in% names(to_normalise)]
  if(!is.null(to_normalise)){
    matrix_antigen <-  to_normalise %>%
      select(antigens) %>%
      as.matrix(.)
    normalise_list <- matrix_normalise(matrix_antigen=matrix_antigen,
                                       method = "none",
                                       array_matrix=array_matrix,
                                       return_plot = TRUE,
                                       control_antigens=control_antigens)
  }else {
    normalise_list <- NULL
  }


  return(normalise_list)

})

antigens_norm_react <- reactive({
  dataCV_tag <- dataCV_tag_reactive()
  if(!is.null(dataCV_tag)){
    batch_vars_name <- c("machine","day")
    df_to_normalise <-  dataCV_tag  %>%
      ungroup() %>%
      select(slide=.id,sampleID,sample_array_ID,antigen,batch_vars_name,mean_best_CV) %>%
      group_by(sampleID,machine, slide)
    to_normalise_antigens <- c(df_to_normalise$antigen)
  }
  return(to_normalise_antigens)
})

## returning a list of the normalised data
normalised_list_reactive <- reactive({
  to_normalise <- to_normalise_reactive()

  params_df <- params_display()
  antigens <-   params_df$antigen_list
  control_antigens <- input$rlm_antigens
  ## check consistency of names
  #antigens <- antigens[antigens %like% names(to_normalise)]
  antigens <- antigens_norm_react()


  if(!is.null(to_normalise) & input$normalisation_method!="rlm"){
    matrix_antigen <-  to_normalise %>%
    ## check consistency of names
    select(antigens) %>%
      as.matrix(.)

    normalise_list <- matrix_normalise(matrix_antigen=matrix_antigen,
                                       method = input$normalisation_method,
                                       array_matrix=array_matrix,
                                       return_plot = TRUE,
                                       control_antigens=control_antigens)
  }else if(!is.null(to_normalise) & input$normalisation_method=="rlm"){
    array_matrix <- array_matrix_reactive()
    print(row.names(to_normalise))
    matrix_antigen <-  to_normalise %>%
    #  rownames_to_column(.data, var = "rowname") %>%
      ## check consistency of names
    select(antigens) %>%
      as.matrix(.)
  ## check why it looses row names above
  row.names(matrix_antigen) <- row.names(to_normalise)
    normalise_list <- matrix_normalise(matrix_antigen=matrix_antigen,
                                       method = input$normalisation_method,
                                       array_matrix=array_matrix,
                                       return_plot = TRUE,
                                       control_antigens=control_antigens)
  }else {
    normalise_list <- NULL
  }


  return(normalise_list)

})

output$normalised_sd_plot <- renderPlot({
  normalised_list <- normalised_list_reactive()
  if(input$chip_path_param!="") {
    if(!is.null(normalised_list)){
      p <- normalised_list$plot_normalisation
      p

    }else{
      p <- empty_plot_error()
      p
    }
  }else{
    p <- empty_plot()
    p
  }
})


output$normalisation_drop_down <- renderUI({
  dropdownButton(
    tags$h3("List of Normalisation"),
    checkboxInput("log2", "Log2 Normalisation", TRUE),
    checkboxInput("vsn", "VSN", TRUE),
    checkboxInput("cyclic_loess_log", "Cyclic Loess (log)", TRUE),
    checkboxInput("rlm", "RLM Normalisation",FALSE),
    checkboxInput("cyclic_loess", "Cyclic Loess", FALSE)
  )
})


normalised_list_all_reactive <- reactive({
  to_normalise <- to_normalise_reactive()
  params_df <- params_display()
  antigens <-   params_df$antigen_list
  control_antigens <- input$rlm_antigens
  array_matrix <- array_matrix_reactive()
  ## due to different structures
 # antigens <- antigens[antigens %in% names(to_normalise)]
  antigens <- antigens_norm_react()
  matrix_antigen <-  to_normalise %>%
    select(antigens) %>%
    as.matrix(.)

  ## no normalisation
  normalise_list_none <- matrix_normalise(matrix_antigen=matrix_antigen,
                                         method = "none",
                                         array_matrix=array_matrix,
                                         return_plot = TRUE,
                                         control_antigens=control_antigens)
  names(normalise_list_none) <- c("matrix_antigen_none" ,"plot_none")

  ## log normalisation
  if(input$log2==TRUE){
    normalise_list_log <- matrix_normalise(matrix_antigen=matrix_antigen,
                                           method = "log2",
                                           array_matrix=array_matrix,
                                           return_plot = TRUE,
                                           control_antigens=control_antigens)
    names(normalise_list_log) <- c("matrix_antigen_log" ,"plot_log")
  }else if(input$log2==FALSE){
    p <- empty_plot_NS() + ggtitle("Log normalisation")
    normalise_list_log <- list(plot_log=p, matrix_antigen_log=NULL)
  }


  if(input$vsn==TRUE){
    normalise_list_vsn <- matrix_normalise(matrix_antigen=matrix_antigen,
                                           method = "vsn",
                                           array_matrix=array_matrix,
                                           return_plot = TRUE,
                                           control_antigens=control_antigens)
    names(normalise_list_vsn) <- c("matrix_antigen_vsn" ,"plot_vsn")
  }else if(input$vsn==FALSE){
    p <- empty_plot_NS() + ggtitle("VSN normalisation")
    normalise_list_vsn <- list(plot_vsn=p, matrix_antigen_vsn=NULL)
  }


  if(input$cyclic_loess_log==TRUE){
    normalise_list_cyclic_loess_log <- matrix_normalise(matrix_antigen=matrix_antigen,
                                                        method = "cyclic_loess_log",
                                                        array_matrix=array_matrix,
                                                        return_plot = TRUE,
                                                        control_antigens=control_antigens)
    names(normalise_list_cyclic_loess_log) <- c("matrix_antigen_cyclic_loess_log" ,"plot_cyclic_loess_log")

  }else if(input$cyclic_loess_log==FALSE){
    p <- empty_plot_NS() + ggtitle("Cyclic Loess Log")
    normalise_list_cyclic_loess_log <- list(plot_cyclic_loess_log=p, matrix_antigen_cyclic_loess_log=NULL)
  }
  if(input$cyclic_loess==TRUE){
    normalise_list_cyclic_loess <- matrix_normalise(matrix_antigen=matrix_antigen,
                                                    method = "cyclic_loess",
                                                    array_matrix=array_matrix,
                                                    return_plot = TRUE,
                                                    control_antigens=control_antigens)
    names(normalise_list_cyclic_loess) <- c("matrix_antigen_cyclic_loess" ,"plot_cyclic_loess")

  }else if(input$cyclic_loess==FALSE){
    p <- empty_plot_NS() + ggtitle("Cyclic Loess")
    normalise_list_cyclic_loess <- list(plot_cyclic_loess=p, matrix_antigen_cyclic_loess=NULL)
  }


  if(input$rlm==TRUE){
    normalise_list_rlm <- matrix_normalise(matrix_antigen=matrix_antigen,
                                                  method = "rlm",
                                                  array_matrix=array_matrix,
                                                  return_plot = TRUE,
                                                  control_antigens=control_antigens)
    names(normalise_list_rlm) <- c("matrix_antigen_rlm" ,"plot_rlm")

  }else if(input$rlm==FALSE){
    p <- empty_plot_NS() + ggtitle("RLM")
    normalise_list_rlm <- list(plot_rlm=p, matrix_antigen_rlm=NULL)
  }

  normalise_list <- c(normalise_list_none , normalise_list_log,normalise_list_vsn,normalise_list_cyclic_loess_log,
                      normalise_list_cyclic_loess,normalise_list_rlm)
  return(normalise_list)

})


## plot showing all the normalisation techniques
output$mutiple_plot <- renderPlot({
  normalised_list <- normalised_list_all_reactive()
  normalised_list_plot <- normalised_list[grepl("plot",names(normalised_list))]

  p <- do.call("grid.arrange", c(normalised_list_plot, ncol=2))
  p
})



## output for heatmap
output$select_heatmap<- renderUI({
  heat_map_c <- c("Plot both before and after normalisation"=TRUE,
              "Plot only normalised data"=FALSE)
  prettyRadioButtons(inputId="heat_both",
                     label = 'Plot with log or raw MFI:',
                     choices =  heat_map_c,
                     inline=TRUE,
                     selected = FALSE,
                     animation = "jelly",
                     status = "default",
                     shape = "curve",bigger = TRUE)

})

## heatmap plot
output$heatmap_normalised_see <- renderPlot({

  p3 <- pheatmap::pheatmap(cars)
  p3 <-ggplotify::as.ggplot(p3)
  p3 <- p3 +  theme_void()
  return(p3)
})


output$heatmap_normalised <- renderPlot({
  normalised_list <- normalised_list_reactive()
  non_normalised_list_reactive <- non_normalised_list_reactive()
  control_antigens <- input$rlm_antigens
  norm_df <- normalised_list$matrix_antigen_normalised
  norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))


  if(!is.null(normalised_list) & input$normalisation_method=="rlm"){
    norm_df <- normalised_list$matrix_antigen_normalised
    norm_df <- norm_df %>% select(-control_antigens)
    norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))
    non_norm_df <- non_normalised_list_reactive$matrix_antigen_normalised
    non_norm_df <- non_norm_df %>%  select(-control_antigens)
    non_norm_df <- data.frame(apply(non_norm_df, 2, function(x){rescale(x, to =c(0,1))}))

  }else{
    norm_df <- normalised_list$matrix_antigen_normalised
    norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))
    non_norm_df <- non_normalised_list_reactive$matrix_antigen_normalised
    non_norm_df <- data.frame(apply(non_norm_df, 2, function(x){rescale(x, to =c(0,1))}))
  }
  #test sections
  #print(paste(str(norm_df)))
  #print(paste(names(norm_df)))
  #print(paste(colSums(is.na(norm_df))))

  if(input$heat_both==TRUE){
    p_non_norm <- pheatmap::pheatmap(non_norm_df ,
                                     scale = "none",
                                     cluster_rows = FALSE ,
                                     main="Non normalised data",
                                     silent = TRUE)
    p_non_norm <- ggplotify::as.ggplot(p_non_norm)+ theme_void()
    p2 <- pheatmap::pheatmap(norm_df ,scale = "none", cluster_rows = FALSE ,
                             main=paste(input$normalisation_method,"Normalised Data"),
                             silent = TRUE)
    p2 <- ggplotify::as.ggplot(p2) + theme_void()
    #p <- gridExtra::grid.arrange(grobs = list(p2[[4]],p_non_norm[[4]]))
    p <- ggpubr::ggarrange(p_non_norm,p2, nrow = 2)

  }else{
    p3 <- pheatmap::pheatmap(norm_df ,scale = "none", cluster_rows = FALSE,
                            main=paste(input$normalisation_method,"Normalised Data"),
                            silent = TRUE)
    p3 <- ggplotify::as.ggplot(p3)
    p <- p3 +  theme_void()

  }

  return(p)

})


## output for heatmap
output$slider_pca<- renderUI({

  sliderInput(inputId = 'vars_pca', label = 'Select variables to plot on the PCA',
              value = 20,
              min = 10, max = 50)

})


## plot a PCA

output$PCA_normalised <- renderPlot({
  normalised_list <- normalised_list_reactive()
  non_normalised_list_reactive <- non_normalised_list_reactive()
  control_antigens <- input$rlm_antigens
  norm_df <- normalised_list$matrix_antigen_normalised
  norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))

  if(!is.null(input$vars_pca)){
    vars_visualise <- input$vars_pca
  }else{
    vars_visualise <- 20
  }

  if(!is.null(normalised_list) & input$normalisation_method=="rlm"){
    norm_df <- normalised_list$matrix_antigen_normalised
    norm_df <- norm_df %>% select(-control_antigens)
    norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))
    non_norm_df <- non_normalised_list_reactive$matrix_antigen_normalised
    non_norm_df <- non_norm_df %>%  select(-control_antigens)
    non_norm_df <- data.frame(apply(non_norm_df, 2, function(x){rescale(x, to =c(0,1))}))

  }else{
    norm_df <- normalised_list$matrix_antigen_normalised
    norm_df <- data.frame(apply(norm_df, 2, function(x){rescale(x, to =c(0,1))}))
    non_norm_df <- non_normalised_list_reactive$matrix_antigen_normalised
    non_norm_df <- data.frame(apply(non_norm_df, 2, function(x){rescale(x, to =c(0,1))}))
  }

  ## compute PCA
  res_pca <- prcomp( norm_df, scale = TRUE)
  var <- get_pca_var(res_pca)

  #Visualize the PCA
  p1 <- fviz_pca_ind(res_pca,
                    col.var = "contrib", # Color by contributions to the PC
                    gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
                    repel = TRUE     # Avoid text overlapping
  )+    theme_minimal()

  #p2 <- fviz_cos2(res_pca, choice='var',axes=1:2)
  # Select the top vars_visualise contributing variables
  p2 <-fviz_pca_biplot(res_pca, label="var",
                    select.var = list(contrib = vars_visualise)) +
    theme_minimal()

  # Total cos2 of variables on Dim.1 and Dim.2
  p3 <-     fviz_cos2(res_pca, choice = "var", axes = 1:2 , top = vars_visualise)


  # Color by cos2 values: quality on the factor map
 p4 <-  fviz_pca_var(res_pca, col.var = "cos2",
               gradient.cols = c("#00AFBB", "#E7B800", "#FC4E07"),
               select.var = list(contrib = vars_visualise),
               repel = TRUE # Avoid text overlapping
  )

  p <- gridExtra::grid.arrange(p1,p2,p3,p4, ncol=2 )
  p
})


# function to hide or show or hide tabs with loading
observe({

  tag_file_reactive <- tag_file_reactive()

  req(input$tag_subtract_btn)
  if(is.null(tag_file_reactive) & input$tag_subtract_btn==TRUE){
  hideTab(inputId = "all_tabs", target = "normalise_panel")
  }else  if(input$tag_subtract_btn==FALSE & is.null(tag_file_reactive)){
    showTab(inputId = "all_tabs", target = "normalise_panel")
  }else{
    showTab(inputId = "all_tabs", target = "normalise_panel")
  }
})


#if(input$chip_path_param==""){hideTab(inputId = 'all_tabs',target ='normalise_panel') }

# ### hide tabs
# observe({
#  # hide(selector = "#navbar li a[data-value=normalise_panel]")
#   if(input$chip_path_param==""){hideTab(inputId = 'all_tabs',target ='normalise_panel') }
# })
#
# observeEvent(input$variableXX=='am', {
#   toggle(selector = "#navbar li a[data-value=normalise_panel]")
# })

# output$trial <- renderPlotly({
#   if(input$chip_path_param!="") {
#     ## load the default parameted
#     p <- ggplot(cars,aes(speed,dist)) +geom_jitter()
#     p <- ggplotly(p)
#     p
#
#   }else{
#     p <- empty_plot()
#     p <- ggplotly(p)
#     p
#   }
#
# })
#
#
output$trial2 <- renderPrint({
  print(paste(str(input$tag_antigens), "seeeee", dataCV_tag_reactive()))
})

})

library(digest)
library(bit)

rawToInt <- function(bytes) {
  Reduce(function(left, right) {
    bitwShiftL(left, 8) + right
  }, as.integer(bytes), 0L)
}

# Quick and dirty bloom filter. The hashing "functions" are based on choosing
# random sets of bytes out of a single MD5 hash. Seems to work well for normal
# values, but has not been extensively tested for weird situations like very
# small n or very large p.
BloomFilter <- setRefClass("BloomFilter",
  fields = list(
    .m = "integer",
    .bits = "ANY",
    .k = "integer",
    .bytesNeeded = "integer",
    .bytesToTake = "matrix"
  ),
  methods = list(
    # @param n - Set size
    # @param p - Desired false positive probability (e.g. 0.01 for 1%)
    initialize = function(n = 10000, p = 0.001) {
      m <- (as.numeric(n) * log(1 / p)) / (log(2)^2)

      .m <<- as.integer(m)
      .bits <<- bit(.m)
      .k <<- max(1L, as.integer(round((as.numeric(.m)/n) * log(2))))

      # This is how many *bytes* of data we need for *each* of the k indices we need to
      # generate
      .bytesNeeded <<- as.integer(ceiling(log2(.m) / 8))
      .bytesToTake <<- sapply(rep_len(.bytesNeeded, .k), function(byteCount) {
        # 16 is number of bytes an md5 hash has
        sample.int(16, byteCount, replace = FALSE)
      })
    },
    .hash = function(x) {
      hash <- digest(x, "md5", serialize = FALSE, raw = TRUE)
      sapply(1:.k, function(i) {
        val <- rawToInt(hash[.bytesToTake[,i]])
        # Scale down to fit into the desired range
        as.integer(val * (as.numeric(.m) / 2^(.bytesNeeded*8)))
      })
    },
    has = function(x) {
      all(.bits[.hash(x)])
    },
    set = function(x) {
      .bits[.hash(x)] <<- TRUE
    }
  )
)

library(shiny)

# Define UI ----
ui <- fluidPage(
  titlePanel("About protGear"),
  sidebarLayout(
    sidebarPanel(
      h2("Installation"),
      p("protGear is available on BioConductor Github, so you can install it in the usual way
        from your R console:"),
      h3("Installation Bioconductor"),
      code('BiocManager::install("protGear")'),
      h3("Installation of development version Github"),
      code('remotes::install_github("keniajin/protGear")'),
      br(),
      br(),
      br(),
      br(),
      img(src = "protGear_logo.png"),
      br(),
      "protGear is a developed by ",
      span(a("Keniajin",href = "http://keniajin.com"), style = "color:blue")
    ),
    mainPanel(
      h1("Introducing protGear"),
      p("protGear is a package for protein micro-array data processing just
        before the main analysis. ",
        em("one-stop-shop  "),
        "pre-processing suite for protein microarrays that is compatible
        with data from the major protein microarray scanners."),
      br(),
      p("For an introduction examples, visit the ",
        a("protGear page.",
          href = "https://keniajin.github.io/protGear/")),
      br(),
      h2("Functionalities"),
      p("- Background correction"),
      p("- Normalization"),
      p("- Principal component Analysis"),
      p("- Heatmaps of normalised data")
    )
  )
)

shinyUI(
  navbarPage("Data process -->",id = 'all_tabs',
             theme = shinytheme("flatly"),

             # shinytheme("sandstone"),
             #'@_______________Background_and_structure____________________________________
             #'
             tabPanel("Array design structure",
                      dashboardPage(#theme = shinytheme("sandstone") ,
                        dashboardHeader(),
                        dashboardSidebar(width="20%",
                                         column(12,
                                                br(),
                                                h4("Select one of the .gpr/text file"),
                                                p(".gpr file extension was developed by Molecular Devices \n and is used for files created using the GenePix Pro Software"),
                                                uiOutput("gpr_file_load"),
                                                br(),
                                                sidebarMenu(id="structure_view",
                                                            menuItem("Define the array structure",
                                                                     tabName = "structure", icon = icon("dashboard")),
                                                            menuItem("Demo structure image",
                                                                     tabName = "structure_img", icon = icon("dashboard")),
                                                            menuItem("Spatial structure",
                                                                     tabName = "spatial_struct", icon = icon("dashboard"))

                                                )## end sidebar menu
                                          )),## end dashbaord side bar
                        dashboardBody(
                          shinyjs::useShinyjs(),
                          # add custom JS code
                          extendShinyjs(text = "shinyjs.hidehead = function(parm){
                                    $('header').css('display', parm);
                                }",functions = c("hidehead")),
                          tabItems(
                            tabItem("structure",
                                    column(9,
                                           fluidRow(
                                             column(4,
                                                    uiOutput("wavelength_struct")
                                             )),
                                    fluidRow(
                                      column(3,class='my_textinput',
                                             uiOutput("total_miniarray_struct")
                                      ),
                                      column(3,class='my_textinput',
                                             uiOutput("blockspersample_output")
                                      ),
                                      column(3,class='my_textinput',
                                             uiOutput("select_F_var")
                                      ),
                                      column(3,class='my_textinput',
                                             uiOutput("select_B_var")
                                      )
                                    ),
                                    fluidRow(
                                      box(width="100%",
                                          div(class="span6" , "Structure of the array map",
                                              plotOutput('structure_plot', width="100%",height = "500px"),
                                              style = "height:80vh;background-color: white;") %>%
                                            withSpinner(color="#0dc5c1")
                                      ))
                                    ),## end column 1
                                   column(3,
                                          h4(strong("Click here for", style = "font-family: 'times'; font-si16pt")),
                                          h4(menuItem("Demonstration of an array map structure",
                                                      tabName = "structure_img", icon = icon("dashboard")),
                                             style = "list-style-type: none;"),
                                          h4(strong("The image on the left shows the structure of array", style = "font-family: 'times'; font-si16pt")),
                                          p(strong("Blocks"),"The number blocks per mini array might vary as per the design of the experiment. A mini-array holds data
                                          for a single sample with unique features. The blocks in the same mini-array are grouped in their respective mini-array."),
                                          p(strong("Samples"),"Each slide must have a respective plate map or sampleID file. The sampleID file shows which sample is
                                            in each mini-array")
                                   )
                            ),## end of structure tabItem
                            tabItem("structure_img",
                                    img(src='image_slide.png', align = "center")
                            ),## end of structure_img tabItem
                            tabItem("spatial_struct",
                                  fluidRow(
                                      column(3#,class='my_textinput',
                                           #  uiOutput("select_spatial_var")
                                      ),
                                      column(3,class='my_textinput',
                                             uiOutput("select_spatial_var")
                                      ),
                                      column(3,class='my_textinput',
                                             uiOutput("select_spatial_type")
                                      ),
                                      column(3#,class='my_textinput',
                                            # uiOutput("select_B_var")
                                      )
                                  ),
                                  br(),
                                 fluidRow(
                                   column(12,
                                      # div(class="span6" , "Spatial visualization of the array MFI's",
                                      br(),
                                           conditionalPanel(
                                             condition = "input.spatial_type == 'point'",
                                             plotlyOutput('spatial_structure_plot', width="100%",height = "100vh")%>%
                                               withSpinner(color="#0dc5c1")
                                           ),
                                           conditionalPanel(
                                             condition = "input.spatial_type == '2d_array'",
                                             plotOutput('spatial_structure_plot_2d')
                                           ), style = "height:100vh;background-color: white;") %>%
                                     withSpinner(color="#0dc5c1")
                                  # )#div close
                                   )
                            )## end of spatial_struct tabItem
                        )## end tab items
                        )## end dashboard body
                      )## end dashboard page
             ),## end tab panel
             #'@_______________Data_load_and_processing____________________________________
             tabPanel("Data load and processing",
                      dashboardPage(#theme = shinytheme("sandstone") ,
                        dashboardHeader(),
                        dashboardSidebar(width="20%",
                                         column(12,
                                                shinyjs::useShinyjs(),
                                                uiOutput('dir_files'),
                                                br(),
                                                uiOutput("batch_select"),
                                                h4("Make sure this isthe folder with +sampleID's+"),
                                                uiOutput("sampleID_select"),
                                                br(),
                                                sidebarMenu(id="data_load_tabs",
                                                  menuItem("Dashboard", tabName = "dashboard", icon = icon("th")),
                                                  menuItem("Raw data", tabName = "rawdata", icon = icon("th"))
                                                )
                                         )),
                        dashboardBody(
                          shinyjs::useShinyjs(),
                          # add custom JS code
                          #js_file <- ,
                          #extendShinyjs("www/app.js"),
                          extendShinyjs(system.file("shiny-examples/protGear_interactive/www",
                                                    "app.js", package="protGear"),
                                        functions = c('hidehead','hideSidebar')),
                          tabItems(
                            tabItem("dashboard",
                                    # this Row picks the parametes
                                    fluidRow(
                                      column(1#,
                                             #uiOutput('blockspersample_output')
                                      ),
                                      column(3,
                                             uiOutput('channel_output')
                                      ),
                                      column(3,
                                          uiOutput('total_samples_output')
                                      ),

                                      column(2,
                                             uiOutput('machine_output')
                                      ),
                                      column(2,
                                             uiOutput('date_process_output')
                                      )

                                    ),
                                    fluidRow(
                                      column(3,
                                             uiOutput("files_in_batch")
                                      ),
                                      column(3,
                                             uiOutput("check_sampleID_out")
                                      ),
                                      column(3,
                                             uiOutput("antigens_count")
                                      ),
                                      column(3,
                                             uiOutput("blocks_count")
                                      )
                                    ),
                                    fluidRow(
                                      box(
                                        #   width = 8, status = "info", solidHeader = TRUE,
                                        #  title = "Popularity by package (last 5 min)",
                                        #  bubblesOutput("packagePlot", width = "100%", height = 600)
                                        div(class="span6" , "Buffer spots overview",
                                            plotOutput('data_process', width="100%")%>% withSpinner(color="#0dc5c1"),
                                            style = "height:80%;background-color: white;")
                                      ),
                                      box(
                                        div(class="span6" , "Data overview",
                                            plotlyOutput('data_box', width="100%")%>% withSpinner(color="#0dc5c1"),
                                            style = "height:80%;background-color: white;"),
                                        # slide bar for the
                                        sliderInput("slider_antigen", label = h3("Select the antigen range to view"), min = 0,
                                                    max = 120, value = c(40, 60))
                                      )
                                    )
                            ),
                            tabItem("rawdata",
                                    #  numericInput("maxrows", "Rows to show", 25),
                                    #verbatimTextOutput("rawtable"),
                                    #downloadButton("downloadCsv", "Download as CSV")
                                    downloadButton("download_Raw_Data", "Download raw data"),
                                    br(),
                                    DT::DTOutput("tbl_all_data") %>% withSpinner(color="#B80f0A")

                            )
                          )
                        )
                      )
             ),

             ### Background correction
             #'@_______________Background_correction____________________________________
             #'
             tabPanel("Background Correction",
                      dashboardPage(#theme = shinytheme("sandstone") ,
                        dashboardHeader(),
                        dashboardSidebar(width="20%",
                                         column(12,

                                                br(),
                                                h4("Select the slide to view the background"),
                                                uiOutput('slide_names_bg'),
                                                br(),
                                                uiOutput('bg_correction_select_1'),
                                                br(),
                                                sidebarMenu(id="bg_tabs",
                                                  menuItem("Background overview",
                                                           tabName = "bg_graphs", icon = icon("dashboard")),
                                                  menuItem("Perform background correction",
                                                           tabName = "plot_correct", icon = icon("th")),
                                                  menuItem("Foreground and Background plot",
                                                           tabName = "fg_bg_plot", icon = icon("th"))
                                                )
                                         )),## end dashbaord side bar
                        dashboardBody(width='80%',
                          shinyjs::useShinyjs(),
                          # add custom JS code
                          extendShinyjs(text = "shinyjs.hidehead = function(parm){
                                    $('header').css('display', parm);
                                }",functions = c("hidehead")),
                        tabItems(
                            tabItem("bg_graphs",
                                    fluidRow(
                                      column(1#,

                                      ),
                                      column(3,
                                             div(class="radioSelect",
                                                 #h4("  "),
                                                 uiOutput("select_log_MFI",height="auto")
                                             )
                                      ),
                                      column(3,
                                             div(class="radioSelect",
                                                 #h4(""),
                                                 uiOutput("select_block_antigen",height="auto"))
                                      )),
                                    fluidRow(
                                      column(8,
                                             div(class="span6" , textInput("value_block",""), #paste0("Background plot by ",  uiOutput('value_block')),
                                                 plotlyOutput('bg_plots')%>% withSpinner(color="#0dc5c1"),
                                                 style = "height:800px;background-color: white;width:auto;")
                                             ),
                                      column(4,class="sidebarItemExpanded",
                                       p("We have implement different approaches of background correction."),
                                      h2(strong("The implemented approaches are", style = "font-family: 'times'; font-si16pt")),
                                      p(strong("- Local bakground subtraction"),"This method involves subtracting the median of the background pixel
                                        intensities from the foreground signal of a specific spot.  Local background subtraction from the foreground
                                        intensity debatably gives in principle an unbiased estimator of the true signal since it is arguably an
                                        unbiased estimate of the local non-specific intensity "),
                                      p(strong("- Global background subtraction."),"Global background approach involves subtracting a single value for
                                        the background from each individual spot. The global value is the median of all the local background values for
                                        a given array. "),
                                      p(strong("- Moving minimum"),"The background intensities are replaced with the minimum of the local backgrounds
                                        in a given block/mini array. This is utilised mostly if there is a block artefact
                                        observed in the background intensities."),
                                      p(strong("- Half Moving minimum "),"Minimum-half involves a two-step background correction approach and it is
                                        designed to ensure only positive corrected intensities are produced.
                                        In the first place, the local background correction is implemented, then secondly any intensity which is zero or negative
                                        after local background subtraction is set equal to half the minimum of the positive corrected intensities in each block."),
                                      p(strong("- Log-linear background correction (Edwards)"),"The strategy involves adjusting values after local background subtraction to avoid negative intensities.
                                        Background subtraction is replaced by a smooth monotonic function when the difference between the foreground and the local background is
                                        large than a small threshold value "),
                                      p(strong("- Normal and exponential (normexp) background correction") ,"This approach corrects the background using a two mixture model of normal and exponential distribution for
                                        background signals and local background-subtracted foreground intensities respectively.")
                                )
                              )
                            ),## end of bg1 tabItem
                            tabItem("plot_correct",
                                    fluidRow(
                                      h2(strong("Select the BG correction option on the left. The red
                                              line is the zero line", style = "font-family: 'times'; font-si16pt")),
                                      column(12,
                                             div(class="span6" ,
                                                 plotOutput('bg_correct_graphs')%>% withSpinner(color="#0dc5c1"),
                                                 style = "height:800px;background-color: white;width:auto;")

                                      )
                                    )
                            ),## end of tabItem bg 2
                          tabItem("fg_bg_plot",
                              fluidRow(
                                column(12,
                                    plotlyOutput('fg_bg_plots')%>% withSpinner(color="#0dc5c1")

                                )
                              )
                            )## end of tabItem bg 3

                          )## end tab items
                        )## end dashboard body
                      )## end dashboard page
             ),## end tab panel

             ### Coefficient of variation plot
             #'@_______________Coefficient_of_variation____________________________________
             tabPanel("Coefficient of Variation (CV)",
                      dashboardPage(
                        dashboardHeader(),
                        dashboardSidebar(width="20%",
                                         column(12,

                                                sidebarMenu(id="cv_tabs",
                                                            menuItem("Background correction and CV",icon = icon("spinner"), tabName = "bg_correct_cv"),
                                                            #menuItem("Tag subtract", tabName = "tag_plots"),
                                                            menuItemOutput('tag_plots'),
                                                            menuItem("Download data", tabName = "download_bg_correct",icon = icon("download"))
                                                ),
                                                br(),
                                                helpText("Select the method of background correction"),
                                                uiOutput(class="bg_select" ,'bg_correction_select'),
                                                helpText("Select the button below if the proteins have purification TAGs"),
                                                uiOutput(class="btn_width" ,'tag_subtract_select'),
                                                uiOutput(class="bg_select" ,'tag_file_load'),
                                                uiOutput(class="drop_select" ,'tag_antigens_select'),
                                                hr(),
                                                uiOutput(class="bg_select" ,'cv_value_select'),
                                                uiOutput(class="bg_select" ,'replicates_select'),
                                                uiOutput(class="bg_select" ,'minimum_mfi_select')


                                         )),## end dashbaord side bar
                        dashboardBody(
                          tags$head(
                            # Include our custom CSS
                           # style_file <- ,
                            includeCSS(system.file("shiny-examples/protGear_interactive/www", "styles.css", package="protGear"))#"www/styles.css")
                          ),
                          shinyjs::useShinyjs(),
                          # add custom JS code
                          extendShinyjs(text = "shinyjs.hidehead = function(parm){
                                    $('header').css('display', parm);
                                }",functions = c("hidehead")),
                          tabItems(
                            tabItem("bg_correct_cv",
                                    fluidRow(
                                      column(6,
                                             div(class="span6" , "Correlation of data replicates coloured by CV",
                                                 plotOutput('cv_corr_plot', width="100%") %>% withSpinner(color="#0dc5c1"),
                                                 style = "height:80%;background-color: white;")
                                      ),
                                      column(6,
                                             div(class="span6" , "Violin plots showing the distribution of CV",
                                                 plotOutput('cv_violin_plot', width="100%")%>% withSpinner(color="#0dc5c1"),
                                                 style = "height:80%;background-color: white;")
                                      )) ,
                                    fluidRow(
                                      column(6,
                                             div(class="span6" , "Correlation of best 2 replicates ",
                                                 plotOutput('cv_violin_plot_best2', width="100%")%>% withSpinner(color="#0dc5c1"),
                                                 style = "height:80%;background-color: white;")
                                      ),
                                      column(6,
                                             div(class="table_view" , h3("A table showing the CV errors using",textOutput('cv_ui', inline = TRUE),"% cut off"),
                                                 DT::dataTableOutput("sample_best2_reactive_tbl") %>% withSpinner(color="#B80F0A"),
                                                 style = "height:100%;background-color: white;")
                                      ))

                            ),## end of cv1 tabItem
                            tabItem("tag_plots",
                                    fluidRow(
                                      column(3,
                                             uiOutput('bg_correct_infobox')
                                             ),
                                      column(3,
                                             uiOutput('tag_infobox')
                                      ),
                                      column(3,
                                             uiOutput('cv_infobox')
                                      ),
                                      column(2,
                                             uiOutput('sample_ID_select'),
                                             hr()
                                             )
                                    ),
                                    fluidRow(
                                      column(6,
                                           div(class="span6" , "Box plot of the TAG antigens",
                                               plotOutput('tag_box', width="100%") %>% withSpinner(color="#0dc5c1"),
                                          style = "height:80%;background-color: white;")
                                    ),
                                    column(6,
                                          div(class="span6" , "Box plot of the TAG antigens",
                                          plotOutput('tag_box_sample', width="100%") %>% withSpinner(color="#0dc5c1"),
                                          style = "height:80%;background-color: white;")

                                    )
                                    ),
                                    fluidRow(class='bg_gray',
                                      column(2
                                            ),
                                      column(3,
                                             uiOutput('tag_antigen_radio_select')

                                      ),
                                      column(3,
                                                uiOutput('antigen_tag_specific_select')
                                            ),
                                      hr()
                                      ),

                                    fluidRow(
                                      column(12,

                                             div(class="span6" , "Box plot of the TAG antigens",
                                                 plotOutput('tag_antigens_box', width="100%") %>% withSpinner(color="#0dc5c1"),
                                                 style = "height:100%;")
                                      )
                                    )

                            ),## end of ttag_plots
                            tabItem("download_bg_correct",
                                    fluidRow( downloadButton("download_dataCV", "Download CV corrected data")),
                                    br(),
                                    fluidRow(
                                      #DT::DTOutput("tbl_data_cv")
                                      DT::dataTableOutput('tbl_data_cv')  %>% withSpinner(color="#B80f0A")
                                      )
                            )## end of download_bg_correct
                          )## end tab items
                        )## end dashboard body
                      )## end dashboard page
             ),## end tab panel
             #'@_______________Normalisation____________________________________
             #'
             tabPanel("Normalisation",value = "normalise_panel",
                      dashboardPage(#theme = shinytheme("sandstone") ,
                        dashboardHeader(),
                        dashboardSidebar(width="20%",
                                         column(12,
                                             sidebarMenu(id="norm_tabs",
                                                 menuItem("Normalisation", tabName = "normalisation"),
                                                # menuItemOutput('normalisation'),
                                                  menuItem("Compare normalisations", tabName = "norm2"),
                                                  menuItem("Array heatmap of normalised data", tabName = "heatmap_norm"),
                                                  menuItem("PCA of normalised data", tabName = "pca_norm")
                                                )
                                         ),
                                         br(),
                                         helpText("Select the method of normalisation"),
                                         uiOutput(class="bg_select" ,'normalisation_select'),
                                         hr(),
                                         uiOutput(class="bg_select" ,'rlm_antigens_select'),
                                         #uiOutput(class="drop_select" ,'tag_antigens_select'),
                                         hr()
                                         ),## end dashbaord side bar
                        dashboardBody(
                          shinyjs::useShinyjs(),
                          # add custom JS code
                          extendShinyjs(text = "shinyjs.hidehead = function(parm){
                                    $('header').css('display', parm);
                                }",functions = c("hidehead")),
                          tabItems(
                            tabItem("normalisation",
                                    fluidRow(
                                      column(3,
                                           uiOutput('normalisation_infobox')
                                      ),
                                      column(3,
                                             uiOutput('bg_correct_infobox2')
                                      ),
                                      column(3,
                                             uiOutput('tag_infobox2')
                                      ),
                                      column(3,
                                             uiOutput('cv_infobox2')
                                      )
                                    ),
                                    fluidRow(
                                      column(9,
                                      box(width="100%",
                                          div(class="span6" , "Normalisation",
                                              plotOutput('normalised_sd_plot', width="100%",height = "600px"),
                                              style = "height:80%;background-color: white;")
                                      )),
                                      column(3,
                                             p("We have implement different approaches of normalisation."),
                                             h2(strong("The implemented approaches are", style = "font-family: 'times'; font-si16pt")),
                                             p(strong("- Log 2 Normalisation"),"The matrix of background-corrected signal levels is log2 transformed."),
                                             p(strong("- VSN"),"This approach calibrates for sample-to-sample variations through shifting and scaling
                                               using asinh function"),
                                             p(strong("- Cyclic Loess Normalisation"),"Cyclic loess performs pairwise normalization on all distinct pairs of
                                             arrays utlising MA plot and local regression (LOESS) smoothing. MA plot in single-colour microarrays for a pair
                                             of arrays is the scatter plot of average log intensity values [A] from both arrays vs.
                                               difference in log expression values [M] of the same arrays"),
                                             p(strong("- Cyclic Loess and Log Normalisation"),"This approach performs Cyclic Loess normalisation after log transformaing the data"),
                                             p(strong("- Robust Linear Model Normalisation"),'RLM was specifically developed for protein micro arrays.
                                             It incorporates a set of control proteins in the mini arrays to be used as normalization factor using a robust linear models')#,
                                            # p("- Normexp background correction")
                                      )
                                      )
                            ),## end of Norm1 tabItem
                            tabItem("norm2",
                                fluidRow(
                                  column(4

                                  ),
                                  column(3,
                                    uiOutput('normalisation_drop_down')
                                  )
                                ),
                                fluidRow(
                                      box(width="100%",
                                          div(class="span6" , "Normalisation techniques comparison",
                                              plotOutput('mutiple_plot', width="100%",height = "600px"),
                                              style = "height:80%;background-color: white;")
                                  ))
                            ),## end of tabItem
                            tabItem("heatmap_norm",
                                    fluidRow(class="slide_select",
                                      column(4

                                      ),
                                      column(3,
                                           uiOutput('select_heatmap')
                                      )
                                    ),
                                    fluidRow(
                                      column(12,
                                          div(class="span6" , "Heatmap of normalised data",
                                              plotOutput('heatmap_normalised', width="100%",height = "1000px"),
                                              style = "height:80%;background-color: white;")
                                      ))
                            ),## end of tabItem
                            tabItem("pca_norm",
                                    fluidRow(class="slide_select",
                                      column(4

                                      ),
                                      column(3,
                                             uiOutput(class="slide_select" ,'slider_pca')
                                             #uiOutput('select_pca')
                                      )
                                    ),
                                    fluidRow(
                                      box(width="100%",
                                          div(class="span6" , "PCA of normalised data",
                                              plotOutput('PCA_normalised', width="100%",height = "600px"),
                                              style = "height:80%;background-color: white;")
                                      ))
                            )## end of tabItem
                          )## end tab items
                        )## end dashboard body
                      )## end dashboard page
             ),## end tab panel
             #tags$li(actionLink("openModal", label = "", icon = icon("info")),class = "dropdown"),
             tabPanel("Overview", icon = icon("question"))
            )## end Nav bar page
)### end shiny UI

#' Coefficient of Variation
#' @title  cv_estimation
#'
#' @param dataC A dataset a data frame with feature variables to be used
#' @param lab_replicates A numeric value indicating the number of lab replicates
#' @param sampleID_var A character string containing the name of the sample 
#' identifier variable. Default set to 'sampleID'
#' @param antigen_var A character string containing the name of the 
#' features/protein variable. Default to 'antigen'
#' @param replicate_var A character string containing the name of the replicate
#'  variable. Default to 'replicate'
#' @param mfi_var A character string containing the name of the variable with 
#' MFI value.Assuming background correction is done already.
#' Default to 'FMedianBG_correct'
#' @param cv_cut_off Optional value indicating the cut off of flagging CV's. 
#' Default set at 20.
#' @description A function to calculate the CV for the technical lab replicates.
#'  The default values are set as per the object names generated by machine
#' @import dplyr tidyr
#' @importFrom  rlang sym
#' @importFrom tidyr gather spread
#' @return A data frame where CV's of the replicates have been calculated
#' @export
#'
#' @examples
#' dataC <- readr::read_csv(system.file("extdata", 
#' "dataC.csv", package="protGear"))
#' ## this file has 3 lab replicates and the default names
#' cv_estimation(dataC  ,lab_replicates=3)
cv_estimation <-
  function(dataC  ,
           lab_replicates ,
           sampleID_var = 'sampleID',
           antigen_var = 'antigen' ,
           replicate_var = 'replicate' ,
           mfi_var = 'FMedianBG_correct' ,
           cv_cut_off = 20) {
    if (lab_replicates == 1) {
      dataC <- dataC %>%
        dplyr::filter(
          !grepl(
            '^[Ll][Aa][Nn][Dd][Mm][Aa][Rr][Kk]|^[Bb][Uu][Ff][Ff][Ee][Rr]',
            !!antigen_var
          )
        )   %>%
        filter(replicate <= lab_replicates)

      # combine Name and replicate
      #tidyr::unite(col=antigen,antigen,replicate)
      # dataC <- dataC %>%
      #   dplyr::select(sampleID,sample_array_ID, 
      #antigen,iden,FMedianBG_correct)
      ## get the name of the file
      iden <- unique(dataC$iden)
      # pick the values after the last underscore
      ## to get the replicate
      #dataC$replicate <- sub(".*_(.*)", "\\1", dataC$antigen)

      ## create a wide data to
      if (length(unique(dataC$replicate)) > lab_replicates)  {
        try(stop("Some antigens seems to be repeated in a mini array for", iden) 
            , outFile = stdout())
        error_replicates(iden)
      }

    } else if (lab_replicates > 1) {
      ## Exclude the land mark and Buffer
      dataC <- dataC %>%
        dplyr::filter(
          !grepl(
            '^[Ll][Aa][Nn][Dd][Mm][Aa][Rr][Kk]| ^[Bb][Uu][Ff][Ff][Ee][Rr]',
            !!antigen_var
          )
        )   %>%
        dplyr::filter(!!sym(replicate_var) <= lab_replicates)

      #combine Name and replicate
      #tidyr::unite(col=antigen,antigen,replicate)

      ## get the name of the file
      iden <- unique(dataC$iden)
      # pick the values after the last underscore
      ## to get the replicate
      # dataC$replicate <- sub(".*_(.*)", "\\1", dataC$antigen)
      ### pick the nname of the antigen
      ## values until the last underscore
      #dataC$antigen <- sub("\\_[^\\_]*$" , "", dataC$antigen)

      ## create a wide data to

      if (length(unique(dataC$replicate)) > lab_replicates)  {
        try(stop("The replicates per antigen per sample are more than
                 expected for ",
                 iden) , outFile = stdout())
        error_replicates(iden)
      } else
        (warning("The replicates are as expected per sample per antigen"))


      ## reshaping the data
      Data3 <- dataC %>%
        dplyr::select(contains('iden'),
                      contains('sample_array_ID'),
                      !!sampleID_var, !!mfi_var,
                      !!replicate_var,!!antigen_var) %>%
        spread(replicate, FMedianBG_correct)


      ### group the antigens and calculate their CV values
      ## the CV values > 20, get the minimum mean of each of the two mfi values
      #and use it to calculate CV
      ## select the one with lowest CV
      dataC <- dataC %>%
        dplyr::group_by_at(c(antigen_var,sampleID_var)) %>%
        ###mean and then for each grouping of 2
        summarize(meanX=mean(get(mfi_var) , na.rm=TRUE),
                  meanX2_X3=mean(get(mfi_var)[-1] , na.rm=TRUE),
                  meanX1_X3=mean(get(mfi_var)[-2] , na.rm=TRUE),
                  meanX1_X2=mean(get(mfi_var)[-3] , na.rm=TRUE),
                  ###standard deviation and then for each grouping of 2
                  sdX=round(sd(get(mfi_var) , na.rm=TRUE),2),
                  sdX2_X3=round(sd(get(mfi_var)[-1] , na.rm=TRUE),2),
                  sdX1_X3=round(sd(get(mfi_var)[-2] , na.rm=TRUE),2),
                  sdX1_X2=round(sd(get(mfi_var)[-3] , na.rm=TRUE),2),
                  ### cv
                  CVX=(round(sdX/meanX,4))*100 ,
                  CVX2_X3=(round(sdX2_X3/meanX2_X3,4))*100 ,
                  CVX1_X3=(round(sdX1_X3/meanX1_X3,4))*100 ,
                  CVX1_X2=(round(sdX1_X2/meanX1_X2,4))*100) %>%
        mutate(cvCat_all = ifelse(CVX>=0 & CVX<=cv_cut_off ,
                                  paste0("CV <= ",cv_cut_off),
                                  ifelse(CVX >cv_cut_off & CVX <101,
                                         paste0("CV > ",cv_cut_off),
                                         "Others")))   %>%
        mutate(cvSelected_all = ifelse(CVX>=0 & CVX<=cv_cut_off , CVX,
                                       ifelse(CVX >cv_cut_off | CVX<=0 ,
                                              pmin(CVX2_X3, CVX1_X3,CVX1_X2 ,
                                                   na.rm = TRUE),NA)))
      dataC <- dataC %>%
        left_join(Data3, by=c(antigen_var,sampleID_var)) %>%
        select(antigen, sampleID, sample_array_ID,everything())
    }
    return(dataC)
  }


#' Select set of replicates with the best CV
#' @title best CV estimation
#'
#' @param dataCV A data frame
#' @param slide_id A character string containing the identifier of the data
#'  frame variable.
#' @param cv_cut_off a numeric value for the CV cut off. Should be between 0-100
#' @param lab_replicates A numeric value indicating the 
#' number of lab replicates.
#'
#' @description A function to select the best CV by combining the replicates in 
#' duplicates. The function has been build for up to to 3 replicates so far
#' @import dplyr tidyr
#' @importFrom tidyr gather spread
#' @importFrom dplyr mutate select
#' @importFrom readr read_csv
#' @importFrom  plyr .
#' @return A data frame with the best CV's estimated
#' @export
#'
#' @examples
#' dataC <- readr::read_csv(system.file("extdata",
#'  "dataC.csv", package="protGear"))
#' ## this file has 3 lab replicates and the default names
#' dataCV <- cv_estimation(dataC  ,lab_replicates=3)
#' best_CV_estimation(dataCV,slide_id = "iden", lab_replicates = 3,
#'  cv_cut_off = 20)
best_CV_estimation <-
  function(dataCV,
           slide_id,
           lab_replicates ,
           cv_cut_off) {
    if (lab_replicates > 1) {
      ### Get the mean that corresponds to the lowest CV
      iden <- unique(dataCV[[slide_id]])
      data_best_CV <- as.data.frame(dataCV)

      ## changing NaN values to 0 to facilitate computation
      is.nan.data.frame <-
        function(x)
          do.call(cbind, lapply(x, is.nan))
      data_best_CV[is.nan.data.frame(data_best_CV)] <- 0

      ## get the minumum cv and put the value of the as a string variable
      ## ie CVX3 selected meanX3 value will be created
      ## might bring issues here if the subtraction has a NA or its missing for
     # the prescan
      data_best_CV <- data_best_CV %>%
        mutate(x = colnames(.[, c("CVX2_X3", "CVX1_X3", "CVX1_X2")])
               [apply(.[, c("CVX2_X3", "CVX1_X3", "CVX1_X2")], 1, 
                      which.min)])  %>%
        dplyr::mutate(xbar = paste0("meanX", gsub("CVX", "", x)))

## get the actual value of the mean that corresponds to that
#http://stackoverflow.com/questions/43762869/
      #get-row-value-corresponding-to-a-column-name
      data_best_CV <- data_best_CV %>%
        mutate(row = 1:n()) %>%
        # select(-c(`1`,`2`,`3`,iden)) %>%
        gather(prop, val, meanX1_X2:meanX2_X3) %>%
        group_by(row) %>%
        mutate(selected = val[xbar == prop]) %>%
        spread(prop, val) %>% dplyr::select(-row)


      ##create the final selected mean in the data set
      data_best_CV <-  data_best_CV %>%
        mutate(
          meanSelected = ifelse(
            CVX >= 0 & CVX <= cv_cut_off ,
            meanX,
            ifelse((CVX > cv_cut_off &
                      CVX < 101)  | CVX <= 0 , selected, NA)
          ),
          mean_best_CV = selected
        ) %>%
        dplyr::select(-xbar, -selected)
      data_best_CV <-  data_best_CV %>%
        mutate(
          best_CV = pmin(CVX2_X3, CVX1_X3, CVX1_X2, na.rm = TRUE) ,
          best_CV_cat  = ifelse(
            best_CV >= 0 & best_CV <= cv_cut_off ,
            paste0("CV <= ", cv_cut_off),
            ifelse(
              best_CV > cv_cut_off &
                best_CV < 101,
              paste0("CV > ", cv_cut_off),
              "Others"
            )
          )
        )

    } else if (lab_replicates == 1) {
      ### Get the mean that corresponds to the lowest CV
      iden <- unique(dataCV[[slide_id]])
      data_best_CV <- as.data.frame(dataCV)

      ## changing NaN values to 0 to facilitate computation
      is.nan.data.frame <-
        function(x)
          do.call(cbind, lapply(x, is.nan))
      data_best_CV[is.nan.data.frame(data_best_CV)] <- 0

      ### generate the variabrl
      data_best_CV <- data_best_CV %>%
        mutate(mean_best_CV = FMedianBG_correct)
    }

    ## return the dataset of interest
    return(data_best_CV)
  }
#'
#'
#'         \\\_End_Function_\\\         #
#'
#'




#' Subtract the purification TAG data
#' @title tag_subtract
#'
#' @param tag_antigens A character vector with the names of proteins or antigens
#'  used as TAG.
#' @param mean_best_CV_var A character string containing the identifier of the 
#' variable with the MFI values.
#' @param dataC_mfi A dataframe
#' @param sampleID_var A character string containing the name of the sample 
#' identifier variable. Default set to 'sampleID'
#' @param antigen_var A character string containing the name of the
#'  features/protein variable. Default to 'antigen'
#' @param batch_vars A list of characters identifying variables in dataC_mfi 
#' for indicating  batch.
#' @param tag_file A data frame with variables \code{antigen, TAG, TAG_name } 
#' to show the TAG for the different antigens or proteins in dataC_mfi
#' @import dplyr tidyr
#' @importFrom tidyr gather spread
#' @importFrom rlang := !! UQ
#' @return A data frame of TAG values subtracted
#' @export
#'
#' @examples
#' tag_file <- readr::read_csv(system.file("extdata", "TAG_antigens.csv", 
#' package="protGear"))
#' tag_antigens <- c("CD4TAG", "GST", "MBP")
#' batch_vars <- list(machine = "m1", day = "0520")
#' dataC <- readr::read_csv(system.file("extdata", "dataC.csv",
#'  package="protGear"))
#' ## this file has 3 lab replicates and the default names
#' dataCV <- cv_estimation(dataC  ,lab_replicates=3)
#' dataCV_best2 <- best_CV_estimation(dataCV,slide_id = "iden", 
#' lab_replicates = 3, cv_cut_off = 20)
#' tag_subtract(dataCV_best2,tag_antigens=tag_antigens, 
#' mean_best_CV_var="mean_best_CV",
#'  tag_file = tag_file,antigen_var = "antigen", batch_vars = batch_vars)
tag_subtract <-
  function(dataC_mfi,
           tag_antigens,
           mean_best_CV_var,
           tag_file,
           batch_vars,
           sampleID_var = 'sampleID',
           antigen_var = 'antigen') {
    mean_best_CV_var <-  rlang::sym(paste0(mean_best_CV_var))
    sampleID_var <-  rlang::sym(paste0(sampleID_var))
    antigen_var <-  rlang::sym(paste0(antigen_var))
    ## remove the tag for the data

    dataC_mfi_tags <- dataC_mfi %>%
      dplyr::filter(UQ(antigen_var) %in% tag_antigens) %>%
      ungroup() %>%
      dplyr::select(!!sampleID_var,!!antigen_var ,!!mean_best_CV_var) %>%
      ##change all the negative TAG values to zero
      mutate(!!mean_best_CV_var := ifelse(!!mean_best_CV_var < 0, 
                                          0, !!mean_best_CV_var)) %>%
      spread(!!antigen_var ,!!mean_best_CV_var)

    ## join the data with the fused antigen name for subtraction
    dataC_mfi <-
      left_join(dataC_mfi , tag_file, by = paste(antigen_var))

    dataC_mfi <- dataC_mfi %>% ungroup() %>%
      dplyr::select(
        !!sampleID_var ,
        contains('sample_array_ID'),
        !!sampleID_var ,
        TAG,
        everything() ,
        -row
      )
    dataC_mfi <- dataC_mfi %>% ungroup() %>%
      left_join(x = dataC_mfi ,
                y = dataC_mfi_tags ,
                by = paste(sampleID_var)) %>%
      mutate(TAG_mfi = NA)
    vars_in <- names(dataC_mfi)


    ## mutate the tag_mfi variable
    for (i in tag_antigens) {
      tag_var <- rlang::sym(paste0(i))
      dataC_mfi <- dataC_mfi %>%
        mutate(TAG_mfi = ifelse(TAG_name == i &
                                  is.na(TAG_mfi), !!tag_var, TAG_mfi))
    }

    ## subtract the TAG values depending on the TAG of
    mean_best_CV_tag_var <-
      rlang::sym(paste0(mean_best_CV_var, "_tag"))
    dataC_mfi <- dataC_mfi %>%
      mutate(TAG_mfi = ifelse(is.na(TAG_mfi), 0, TAG_mfi)) %>%
      mutate(!!mean_best_CV_tag_var := !!mean_best_CV_var - TAG_mfi)  %>%
      dplyr::select(c(vars_in, paste(mean_best_CV_tag_var))) %>%
      ## add the batch variables to the data
      mutate(machine = batch_vars[["machine"]] , day = batch_vars[["day"]])
    return(dataC_mfi)
  }

---
title: "protGear: Protein micro array data management suite"
output:
  flexdashboard::flex_dashboard:
  theme: cosmo
orientation: rows
vertical_layout: scroll
runtime: shiny
---



```{r setup, include=FALSE}
## Inside the working directory create a sampleID folder to contain information of the sampleIDs
## they should be named according to the sampleID names
library(ggpubr)
library(limma)
library(vsn)
library(ggpubr)
library(gtools)
library(plotly)
library(shinythemes)
library(shinyjs)
library(shinyWidgets)
library(GGally)
library(DT)
library(shinycssloaders)
library(gridExtra)
library(pheatmap)
library(scales)
library(shinyFeedback)
library(shinyalert)
library(data.table)
library(stringr)
library(trend)
library(flexdashboard)
library(shiny)
library(factoextra)
library(shinyFiles)

#### save all the files as objects
#Increasing the default maximum upload restriction of Shiny to 10GB can add to globally
if(Sys.getenv('SHINY_PORT') == "") options(shiny.maxRequestSize=10000*1024^2)


jsToggleFS <- 'shinyjs.toggleFullScreen = function() {
    var element = document.documentElement,
      enterFS = element.requestFullscreen || element.msRequestFullscreen || element.mozRequestFullScreen || element.webkitRequestFullscreen,
      exitFS = document.exitFullscreen || document.msExitFullscreen || document.mozCancelFullScreen || document.webkitExitFullscreen;
    if (!document.fullscreenElement && !document.msFullscreenElement && !document.mozFullScreenElement && !document.webkitFullscreenElement) {
      enterFS.call(element);
    } else {
      exitFS.call(document);
    }
  }'

#useShinyjs(),
#extendShinyjs(text = jsToggleFS)
```


Data Processing {data-width=300}
======================================================================

```{r, height = "100vh"}
dir_app <- system.file("shiny-examples/protGear_interactive/load_data_dashboard/", package = "protGear")
fluidRow(class="sidebarItemExpanded",
    column(12,div(style = "height: 100vh;margin:0,padding:0;",
shinyAppDir(dir_app, options = list(width = "100%", height = "100vh"))
)))
```



About protGear {data-width=300}
======================================================================
 

```{r, height = "100%"}
about_dir_app <- system.file("shiny-examples/protGear_interactive/about_app/", package = "protGear")
fluidRow(
    column(12,div(style = "height:1000px;background-color: white;margin: 0;padding: 0;",
shinyAppDir(about_dir_app, options = list(width = "100%", height = "100%"))
)))
```


Help 
======

```{r}
dropdownMenu(type = "notifications",headerText = strong("HELP"), 
                                          icon = icon("question"), 
                                          badgeStatus = NULL,
                                          notificationItem(
                                            text = (steps$text[1]),
                                            icon = icon("spinner")
                                          ),
                                          notificationItem(
                                            text = steps$text[2],
                                            icon = icon("address-card")
                                          ),
                                          notificationItem(
                                            text = steps$text[3],
                                            icon = icon("calendar")
                                          ),
                                          notificationItem(
                                            text = steps$text[4],
                                            icon = icon("user-md")
                                          ),
                                          notificationItem(
                                            text = steps$text[5],
                                            icon = icon("ambulance")
                                          ),
                                          notificationItem(
                                            text = steps$text[6],
                                            icon = icon("flask")
                                          ),
                                          notificationItem(
                                            text = strong(steps$text[7]),
                                            icon = icon("exclamation")
                                          )  )
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{plot_bg}
\alias{plot_bg}
\title{Plot background}
\usage{
plot_bg(df, x_axis = "antigen", bg_MFI = "BG_Median", log_mfi = TRUE)
}
\arguments{
\item{df}{A default dataset to use for plot.}

\item{x_axis}{The variable on the x axis}

\item{bg_MFI}{A numeric \code{variable} describing which is the 
background MFI}

\item{log_mfi}{a logical value indicating whether the MFI values should be
log transformed or not.}
}
\value{
A ggplot of background values
}
\description{
A generic function for plotting of R objects.
}
\examples{
## Not run:
#After extracting the background using \code{\link{extract_bg}} 
#we plot the data using
allData_bg <- readr::read_csv(system.file("extdata", "bg_example.csv",
 package="protGear"))
plot_bg(allData_bg,
x_axis = "antigen",
bg_MFI = "BG_Median",  log_mfi = TRUE
)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/launch_protGear_interactive.R
\name{launch_protGear_interactive}
\alias{launch_protGear_interactive}
\title{launch_protGear_interactive}
\usage{
launch_protGear_interactive()
}
\value{
launches the shiny interactive protGear app
}
\description{
This is Function is to launch the shiny application
}
\examples{
app <- system.file("shiny-examples", "protGear_interactive",
"protGear_interactive.Rmd", package = "protGear")
 if (app!=""){
 ## run this
 #launch_protGear_interactive()
 }

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation_functions.R
\name{rlm_normalise_matrix}
\alias{rlm_normalise_matrix}
\title{Nomrmalise using RLM}
\usage{
rlm_normalise_matrix(matrix_antigen, array_matrix, control_antigens)
}
\arguments{
\item{matrix_antigen}{A matrix with antigen data}

\item{array_matrix}{A matrix with control antigen data}

\item{control_antigens}{the control antigens for RLM normalisation}
}
\value{
A RLM normalised data frame
}
\description{
A function for \code{method='rlm'} from 
\code{\link{matrix_normalise}}.
}
\examples{
matrix_antigen <- readr::read_csv(system.file("extdata",
 "matrix_antigen.csv", package="protGear"))
# rlm_normalise_matrix(matrix_antigen=matrix_antigen,
 #array_matrix=array_matrix,
# control_antigens=control_antigens)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_estimation_tag_functions.R
\name{best_CV_estimation}
\alias{best_CV_estimation}
\title{best CV estimation}
\usage{
best_CV_estimation(dataCV, slide_id, lab_replicates, cv_cut_off)
}
\arguments{
\item{dataCV}{A data frame}

\item{slide_id}{A character string containing the identifier of the data
frame variable.}

\item{lab_replicates}{A numeric value indicating the 
number of lab replicates.}

\item{cv_cut_off}{a numeric value for the CV cut off. Should be between 0-100}
}
\value{
A data frame with the best CV's estimated
}
\description{
A function to select the best CV by combining the replicates in 
duplicates. The function has been build for up to to 3 replicates so far
}
\details{
Select set of replicates with the best CV
}
\examples{
dataC <- readr::read_csv(system.file("extdata",
 "dataC.csv", package="protGear"))
## this file has 3 lab replicates and the default names
dataCV <- cv_estimation(dataC  ,lab_replicates=3)
best_CV_estimation(dataCV,slide_id = "iden", lab_replicates = 3,
 cv_cut_off = 20)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{check_sampleID_files}
\alias{check_sampleID_files}
\title{\\\_End_Function_\\\         #
Check existing sample ID names}
\usage{
check_sampleID_files(genepix_vars)
}
\arguments{
\item{genepix_vars}{A list of specific definitions of the experiment design.
See \code{\link{array_vars}}.}
}
\value{
A file with missing corresponding sample ID files
}
\description{
A generic function to check if the file(s) 
with the MFI values have a corresponding sample ID file. Sample ID file is
a file with the identifiers for the samples in array file.
}
\examples{
genepix_vars <- array_vars(
channel = "635",
chip_path = system.file("extdata", "array_data/machine1/", 
package="protGear"),
totsamples = 21,
blockspersample = 2,
mig_prefix = "_first",
machine = 1,
date_process = "0520"
)
check_sampleID_files(genepix_vars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_estimation_tag_functions.R
\name{cv_estimation}
\alias{cv_estimation}
\title{cv_estimation}
\usage{
cv_estimation(
  dataC,
  lab_replicates,
  sampleID_var = "sampleID",
  antigen_var = "antigen",
  replicate_var = "replicate",
  mfi_var = "FMedianBG_correct",
  cv_cut_off = 20
)
}
\arguments{
\item{dataC}{A dataset a data frame with feature variables to be used}

\item{lab_replicates}{A numeric value indicating the number of lab replicates}

\item{sampleID_var}{A character string containing the name of the sample 
identifier variable. Default set to 'sampleID'}

\item{antigen_var}{A character string containing the name of the 
features/protein variable. Default to 'antigen'}

\item{replicate_var}{A character string containing the name of the replicate
variable. Default to 'replicate'}

\item{mfi_var}{A character string containing the name of the variable with 
MFI value.Assuming background correction is done already.
Default to 'FMedianBG_correct'}

\item{cv_cut_off}{Optional value indicating the cut off of flagging CV's. 
Default set at 20.}
}
\value{
A data frame where CV's of the replicates have been calculated
}
\description{
A function to calculate the CV for the technical lab replicates.
 The default values are set as per the object names generated by machine
}
\details{
Coefficient of Variation
}
\examples{
dataC <- readr::read_csv(system.file("extdata", 
"dataC.csv", package="protGear"))
## this file has 3 lab replicates and the default names
cv_estimation(dataC  ,lab_replicates=3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation_functions.R
\name{output_trend_stats}
\alias{output_trend_stats}
\title{Trend test using Cox–Stuart (C–S) and Mann–Kendall (M–K) trend tests}
\usage{
output_trend_stats(name, p_val, z_val)
}
\arguments{
\item{name}{Name of the test}

\item{p_val}{p value from the test}

\item{z_val}{the Z value of the test}
}
\value{
A statistics of mean standard deviation trend
}
\description{
Trend test using Cox–Stuart (C–S) and Mann–Kendall (M–K) trend tests
}
\examples{
output_trend_stats(name="t.test",p_val=0.001, z_val=5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{bg_correct}
\alias{bg_correct}
\title{bg_correct}
\usage{
bg_correct(iden, Data1, genepix_vars, method = "subtract_local")
}
\arguments{
\item{iden}{A character indicating the name of the object to be 
used under Data1}

\item{Data1}{A data frame with sample identifiers merged with micro 
array data.}

\item{genepix_vars}{A list of specific definitions of the experiment design.
See \code{\link{array_vars}}.}

\item{method}{a description of the background correction to be used. 
 Possible values are \code{"none","subtract_local",
"subtract_global","movingmin_bg","minimum_half","edwards" or "normexp"}. 
The default is \code{"subtract_local"}.}
}
\value{
A data frame with background corrected data
}
\description{
A generic function to perform background correction.
}
\details{
Background correction

 The function implements background correction methods developed 
by \code{\link[limma]{backgroundCorrect}}. But the
\code{minimum_half and movingmin_bg} uses the block of the protein array as
the grid. If method="movingmin_bg" the minimum
background value within a  block is subtracted.
If method="minimum_half" then any intensity which is negative after 
background subtraction is reset to be equal to half the 
minimum positive value in
a block.  If method="movingmin_value" then any intensity which is negative 
after background subtraction is reset to the minimum positive value
in a block. For \code{edwards} we implement a similar algorithm with 
\code{limma::backgroundCorrect(method="edwards")} and for \code{'normexp'}
we use  the saddle-point approximation to maximum likelihood, 
\code{\link[limma]{backgroundCorrect}} for more details.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_estimation_tag_functions.R
\name{tag_subtract}
\alias{tag_subtract}
\title{tag_subtract}
\usage{
tag_subtract(
  dataC_mfi,
  tag_antigens,
  mean_best_CV_var,
  tag_file,
  batch_vars,
  sampleID_var = "sampleID",
  antigen_var = "antigen"
)
}
\arguments{
\item{dataC_mfi}{A dataframe}

\item{tag_antigens}{A character vector with the names of proteins or antigens
used as TAG.}

\item{mean_best_CV_var}{A character string containing the identifier of the 
variable with the MFI values.}

\item{tag_file}{A data frame with variables \code{antigen, TAG, TAG_name } 
to show the TAG for the different antigens or proteins in dataC_mfi}

\item{batch_vars}{A list of characters identifying variables in dataC_mfi 
for indicating  batch.}

\item{sampleID_var}{A character string containing the name of the sample 
identifier variable. Default set to 'sampleID'}

\item{antigen_var}{A character string containing the name of the
features/protein variable. Default to 'antigen'}
}
\value{
A data frame of TAG values subtracted
}
\description{
\\\_End_Function_\\\         #
}
\details{
Subtract the purification TAG data
}
\examples{
tag_file <- readr::read_csv(system.file("extdata", "TAG_antigens.csv", 
package="protGear"))
tag_antigens <- c("CD4TAG", "GST", "MBP")
batch_vars <- list(machine = "m1", day = "0520")
dataC <- readr::read_csv(system.file("extdata", "dataC.csv",
 package="protGear"))
## this file has 3 lab replicates and the default names
dataCV <- cv_estimation(dataC  ,lab_replicates=3)
dataCV_best2 <- best_CV_estimation(dataCV,slide_id = "iden", 
lab_replicates = 3, cv_cut_off = 20)
tag_subtract(dataCV_best2,tag_antigens=tag_antigens, 
mean_best_CV_var="mean_best_CV",
 tag_file = tag_file,antigen_var = "antigen", batch_vars = batch_vars)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation_functions.R
\name{plot_normalised}
\alias{plot_normalised}
\title{Comparison of normalised data by sample}
\usage{
plot_normalised(exprs_normalised_df, method, batch_correct)
}
\arguments{
\item{exprs_normalised_df}{a normalised data frame}

\item{method}{the method of normalisation used}

\item{batch_correct}{the batch correction}
}
\value{
A ggplot of normalised data
}
\description{
Comparison of normalised data by sample
}
\examples{
matrix_antigen <- readr::read_csv(system.file("extdata", 
"matrix_antigen.csv", package="protGear"))
normlise_vsn <- matrix_normalise(as.matrix(matrix_antigen),
method = "vsn",
return_plot = FALSE
)
plot_normalised(normlise_vsn,method="vsn",batch_correct=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{extract_bg}
\alias{extract_bg}
\title{extract bg}
\usage{
extract_bg(iden, data_files, genepix_vars = genepix_vars)
}
\arguments{
\item{iden}{A character indicating the name of the object to be
used under data_files.}

\item{data_files}{A list of data objects with names utilised by iden.}

\item{genepix_vars}{A list of specific definitions of the experiment design.
See \code{\link{array_vars}}.}
}
\value{
A data frame of background values
}
\description{
A generic function to extract the background 
data for micro array data.
}
\details{
Extract the background values
}
\examples{
## Not run:
genepix_vars <- array_vars(
channel = "635",
chip_path = system.file("extdata", "array_data/machine1/", 
package="protGear"),
totsamples = 21,
blockspersample = 2,
mig_prefix = "_first",
machine = 1,
## optional
date_process = "0520"
)
#Define the data path
data_path <- paste0(genepix_vars$chip_path)
# List the file names to use
filenames <- list.files(genepix_vars$chip_path,
                       pattern = '*.txt$|*.gpr$', full.names = FALSE
)
data_files <- purrr::map(
 .x = filenames,
  .f = read_array_files,
  data_path = data_path,
  genepix_vars = genepix_vars
)
data_files <- purrr::set_names(data_files, 
purrr::map(filenames, name_of_files))
names(data_files)
extract_bg(iden ="KK2-06" , data_files=data_files,genepix_vars=genepix_vars)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cv_by_sample_estimation.R
\name{cv_by_sample_estimation}
\alias{cv_by_sample_estimation}
\title{cv by sample}
\usage{
cv_by_sample_estimation(
  dataCV,
  cv_variable,
  lab_replicates,
  sampleID_var = "sampleID"
)
}
\arguments{
\item{dataCV}{A dataframe}

\item{cv_variable}{A character string containing the identifier 
of the variable with CV values.}

\item{lab_replicates}{A numeric value indicating the number of lab 
replicates.}

\item{sampleID_var}{A character string containing the name of the 
sample identifier variable. Default set to 'sampleID'}
}
\value{
A data frame of CV calculated by sample
}
\description{
A function to give the summary of 
the CV's by the sampleID
}
\details{
Summarise CV by samples
}
\examples{
dataC <- readr::read_csv(system.file("extdata", 
"dataC.csv", package="protGear"))
## this file has 3 lab replicates and the default names
dataCV <- cv_estimation(dataC  ,lab_replicates=3)
cv_by_sample_estimation(dataCV, cv_variable = "cvCat_all",
 lab_replicates = 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{visualize_slide}
\alias{visualize_slide}
\title{Visualize the slide mimicking the original scan image.}
\usage{
visualize_slide(infile, MFI_var, interactive = FALSE, d_f = NA)
}
\arguments{
\item{infile}{a .gpr file to be used to visualize the expression 
intensities of the slide spots}

\item{MFI_var}{the MFI variable to plot, can be either the
background or foreground value}

\item{interactive}{a logical to specify whether an interactive 
graph is returned or not}

\item{d_f}{a data frame with array data}
}
\value{
A ggplot of slide foreground values
}
\description{
Visualize the slide mimicking the original scan image.
}
\examples{
## Not run:
visualize_slide(
infile = system.file("extdata", "/array_data/machine1/KK2-06.txt",
 package="protGear"),
MFI_var = "B635 Median"
)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{minpositive}
\alias{minpositive}
\title{Get the minimum positive value}
\usage{
minpositive(x)
}
\arguments{
\item{x}{A numeric vector or variable}
}
\value{
Returns the minimum positive value in an object
}
\description{
Get the minimum positive value
}
\examples{
minpositive(c(-1,-2,3,5,6,7,8,9,10))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{error_replicates}
\alias{error_replicates}
\title{\\\_Start_Function_For Error\\\         #}
\usage{
error_replicates(iden)
}
\arguments{
\item{iden}{An id for the file with replicates error}
}
\value{
a log file showing the replicate errors
}
\description{
A generic function to write into the log 
file with a replicate check error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{visualize_slide_2d}
\alias{visualize_slide_2d}
\title{Visualize the slide mimicking the original scan image using a 2d plot.}
\usage{
visualize_slide_2d(infile, MFI_var, d_f = NA)
}
\arguments{
\item{infile}{- a .gpr file to be used to visualize the expression 
intensities of the slide spots}

\item{MFI_var}{the MFI variable to plot, can be either the 
background or foreground value}

\item{d_f}{a data frame with array data}
}
\value{
A 2d plot of either the background or foreground values
}
\description{
Visualize the slide mimicking the original scan image using a 2d plot.
}
\examples{
## Not run:
visualize_slide_2d(
infile = system.file("extdata", "/array_data/machine1/KK2-06.txt", 
package="protGear"),
MFI_var = "B635 Median"
)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation_functions.R
\name{rlm_normalise}
\alias{rlm_normalise}
\title{RLM normalisation}
\usage{
rlm_normalise(rlm_normalise_df)
}
\arguments{
\item{rlm_normalise_df}{rlm normalised data frame}
}
\value{
an elist of RLM normalisation to be utilised by 
\code{\link{rlm_normalise_matrix}}
}
\description{
A function for \code{method='rlm'} from 
\code{\link{matrix_normalise}}.
}
\examples{
matrix_antigen <- readr::read_csv(system.file("extdata", 
"matrix_antigen.csv", package="protGear"))
#rlm_normalise_df <- rlm_normalise_matrix(matrix_antigen=matrix_antigen, 
#array_matrix=array_matrix,
# control_antigens=control_antigens)
# rlm_normalise(rlm_normalise_df)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalisation_functions.R
\name{matrix_normalise}
\alias{matrix_normalise}
\title{Normalize  Arrays}
\usage{
matrix_normalise(
  matrix_antigen,
  method = "log2",
  batch_correct = FALSE,
  batch_var1,
  batch_var2 = day_batches,
  return_plot = FALSE,
  plot_by_antigen = TRUE,
  control_antigens = NULL,
  array_matrix = NULL
)
}
\arguments{
\item{matrix_antigen}{An object of class matrix with  features/proteins as 
columns and samples as the rows}

\item{method}{character string specifying the normalization method.
 Choices are \code{"none","log2","vsn","cyclic_loess"}
\code{"cyclic_loess_log" ,"rlm"}}

\item{batch_correct}{A logical value indicating whether batch
correction should be done or not}

\item{batch_var1}{A character or factor vector of size similar to rows 
of \code{matrix_antigen} indicating the first batch.}

\item{batch_var2}{A character or factor vector of size similar to rows 
of \code{matrix_antigen} indicating the second batch.}

\item{return_plot}{A logical value indicating whether a plot is returned 
to show the results of normalisation.}

\item{plot_by_antigen}{Logical to indicate whether to plot by antigen or not
slide name for the matrix_antigen object.}

\item{control_antigens}{logical vector specifying the subset of spots 
which are non-differentially-expressed control spots,
for use with \code{method="rlm"}}

\item{array_matrix}{An object of class dataframe or matrix used with
\code{method='rlm'} indicating the sample index and}
}
\value{
A data frame of normalised values
}
\description{
Normalize  Arrays
}
\examples{
matrix_antigen <- readr::read_csv(system.file("extdata", 
"matrix_antigen.csv", package="protGear"))
#VSN
normlise_vsn <- matrix_normalise(as.matrix(matrix_antigen),
method = "vsn",
return_plot = TRUE
)
## log
normlise_log <- matrix_normalise(as.matrix(matrix_antigen),
method = "log2",
return_plot = TRUE
)
## cyclic_loess_log
normlise_cylic_log <- matrix_normalise(as.matrix(matrix_antigen),
method = "cyclic_loess_log",
return_plot = TRUE
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{read_array_files}
\alias{read_array_files}
\title{Read array files}
\usage{
read_array_files(i, data_path, genepix_vars)
}
\arguments{
\item{i}{The name of the file which the data are to be read from.}

\item{data_path}{The path where the file with the data  is located}

\item{genepix_vars}{A list of specific definitions of the experiment design.
See \code{\link{array_vars}}.}
}
\value{
a number of data frames in the global environment
}
\description{
This helps to read the chip file(s).
}
\details{
Read multiple array files
}
\examples{
## Not run:
genepix_vars <- array_vars(
channel = "635",
chip_path = system.file("extdata", "array_data/machine1/", 
package="protGear"),
totsamples = 21,
blockspersample = 2,
mig_prefix = "_first",
machine = 1,
date_process = "0520"
)
file_read <- "KK2-06.txt"
read_array_files(i=file_read,
data_path=system.file("extdata", "array_data/machine1/",
package="protGear"), genepix_vars=genepix_vars)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{name_of_files}
\alias{name_of_files}
\title{Object names of a list}
\usage{
name_of_files(i)
}
\arguments{
\item{i}{- a list filenames with .txt or .gpr extension}
}
\value{
a list of file names

name
}
\description{
A generic function returning a vector with the names of 
files in the same directory. Removes the file extension
}
\examples{
name_of_files("KK2-06.txt")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{merge_sampleID}
\alias{merge_sampleID}
\title{Merge sample ID with the array data}
\usage{
merge_sampleID(iden, data_files, genepix_vars, method)
}
\arguments{
\item{iden}{A character indicating the name of the object to be used under
data_files.}

\item{data_files}{A list of data objects with names utilised by iden.}

\item{genepix_vars}{A list of specific definitions of the experiment design.
See \code{\link{array_vars}}.}

\item{method}{A description of the background correction to be used. 
See \code{\link{bg_correct}}.}
}
\value{
a data frame merged with corresponding sample ID's.
The sample ID are specified in the sample ID files
}
\description{
A generic function that merges the protein data with the
 sample identifiers or sample names. If the file
does not have sample identifiers the function generates it automatically.
}
\examples{
## Not run:
### Define the genepix_vars
genepix_vars <- array_vars(
  channel = "635",
  chip_path = system.file("extdata", "array_data/machine1/",
   package="protGear"),
  totsamples = 21,
  blockspersample = 2,
  mig_prefix = "_first",
  machine = 1,
  ## optional
  date_process = "0520"
)

## the path where the micro-array data is located
data_path <- paste0(genepix_vars$chip_path)
filenames <- list.files(genepix_vars$chip_path,
                        pattern = "*.txt$|*.gpr$", full.names = FALSE
)
## create a list of all the files
data_files <- purrr::map(
 .x = filenames,
  .f = read_array_files,
  data_path = data_path,
  genepix_vars = genepix_vars
)
data_files <- purrr::set_names(data_files, 
purrr::map(filenames, name_of_files))
## merge the lab data with samples and perform bg correction
merge_sampleID(iden = "KK2-06", data_files = data_files,
               genepix_vars =genepix_vars,method = "subtract_global" )
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buffer_spot_functions.R
\name{buffer_spots}
\alias{buffer_spots}
\title{Extract buffer spots of data}
\usage{
buffer_spots(Data1, buffer_spot = "buffer")
}
\arguments{
\item{Data1}{An object of the class data frame}

\item{buffer_spot}{A character string containing the name of the buffer
spots.}
}
\value{
A data frame of the buffer control spots
}
\description{
A function to extract the buffer spots data. A buffer spot only
 has the solution for
 proprietary ingredients for stabilizing protein and minimizing evaporation.
}
\examples{
bg_correct_df <- readr::read_csv(system.file("extdata", "Data1_sample.csv",
package="protGear"))
buffer_spots(Data1 = bg_correct_df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/launch_protGear_interactive.R
\name{launch_select}
\alias{launch_select}
\title{launch_select}
\usage{
launch_select(theApp)
}
\arguments{
\item{theApp}{accepts one of the folders containing the shiny appplication}
}
\value{
launches the app defined under theApp
}
\description{
This is Function is to launch mutiple shiny applications for protGear
}
\examples{
validExamples <-
 list.files(system.file("shiny-examples", package = "protGear"))
#launch_select(validExamples[[1]])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{plot_FB}
\alias{plot_FB}
\title{plot_FB}
\usage{
plot_FB(
  df,
  antigen_name = "antigen",
  bg_MFI = "BG_Median",
  FG_MFI = "FBG_Median",
  log_mfi = FALSE
)
}
\arguments{
\item{df}{An object containing the data to which the plot is done.}

\item{antigen_name}{The \code{variable} describing which features/proteins/
antibodies in the data should be used to plot}

\item{bg_MFI}{A numeric \code{variable} describing which is the 
background MFI}

\item{FG_MFI}{A numeric \code{variable} describing which is the 
foreground MFI}

\item{log_mfi}{a logical value indicating whether the MFI values should be
log transformed or not.}
}
\value{
a ggplot of foreground vs background MFI values
}
\description{
A generic function for plotting the background and foreground
 values.
}
\details{
Plot foreground and background values
}
\examples{
## Not run:
#After extracting the background using \code{\link{extract_bg}} 
#we plot the data using
allData_bg <- readr::read_csv(system.file("extdata", 
"bg_example.csv", package="protGear"))
plot_FB(allData_bg,
antigen_name = "antigen",
bg_MFI = "BG_Median", FG_MFI = "FBG_Median", log = FALSE
)
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buffer_spot_functions.R
\name{plot_buffer}
\alias{plot_buffer}
\title{Plot the buffer values}
\usage{
plot_buffer(
  df = buffers,
  buffer_names = "antigen",
  buffer_mfi = "FMedianBG_correct",
  slide_id = ".id"
)
}
\arguments{
\item{df}{A data frame to be used to plot}

\item{buffer_names}{A character string containing the name of the variable
with buffer spots. Default set to 'antigen'.}

\item{buffer_mfi}{A character string containing the name of the variable with
MFI value.Assuming background correction is done already.
Default to 'FMedianBG_correct'}

\item{slide_id}{A character string containing the name of the slide/array
identifier variable.}
}
\value{
plot of buffer spots
}
\description{
Plot the buffer values
}
\examples{
buffers <- readr::read_csv(system.file("extdata", "buffers_sample2.csv",
package="protGear"))
plot_buffer(df=buffers,buffer_names = "sampleID")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_files_functions.R
\name{read_array_visualize}
\alias{read_array_visualize}
\title{Read a gpr file to visualize}
\usage{
read_array_visualize(infile)
}
\arguments{
\item{infile}{a .gpr file to be used to visualize the expression intensities
of the slide spots}
}
\value{
a data frame to visualize the background or foreground values
}
\description{
Read a gpr file to visualize
}
\examples{
## Not run:
read_array_visualize(infile = system.file("extdata",
"/array_data/machine1/KK2-06.txt", package="protGear"))
## End(Not run)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{array_vars}
\alias{array_vars}
\title{List the array structure variables}
\usage{
array_vars(
  channel = "635",
  totsamples,
  FG = "",
  BG = "",
  FBG = "",
  blockspersample,
  chip_path = "data/array_data",
  sampleID_path = "data/array_sampleID/",
  mig_prefix = "_first",
  machine = "",
  date_process = ""
)
}
\arguments{
\item{channel}{A character indicating the channel that the data was scanned 
at. It is mostly included in the MFI variable names.}

\item{totsamples}{A numeric value indicating teh number of samples on a slide.}

\item{FG}{Optional:A character indicating the name of the foreground
variable name. if not specified its created as 
\code{paste0("F",channel,".Median")}}

\item{BG}{Optional:A character indicating the name of the background 
variable name.  if not specified its created as 
\code{paste0("B",channel,".Median")}}

\item{FBG}{Optional:A character indicating the name of the 
foreground - background variable name.  if not specified its 
created as \code{paste0("F",channel,".Median...B",channel)}}

\item{blockspersample}{A numeric value indicating the numer of blocks in a 
mini-array. The \code{".gal"} file can help in getting this}

\item{chip_path}{A character indicating the path of the folder  location with
the array data.}

\item{sampleID_path}{A character indicating the path of the folder location 
with the sample identifiers matching the array structure.}

\item{mig_prefix}{Optional: A character indicating the identifier of an MIG
dilution file}

\item{machine}{Optional:A character indicating the machine used to process
the data in the folder}

\item{date_process}{Optional:A character indicating the date
when the samples were processed.}
}
\value{
a list of parameters required to process the data

genepix_vars
}
\description{
A generic function returning a list with the data structure.
}
\examples{
## specify the the parameters to process the data
genepix_vars <- array_vars(
## the channel the data was processed in
  channel = "635",
  ## folder where the array data is stored
  chip_path = "data/array_data",
  ## the number of samples per slide or in as single run
  totsamples = 21,
  ## How many blocks each sample occupies
  blockspersample = 2,
  ## folder where the array data samples id files are stored
  sampleID_path = "data/array_sampleID/",
  ## optional
  mig_prefix = "_first",
  machine = 1,
  date_process = "0520"
)
genepix_vars
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/global_functions.R
\name{create_dir}
\alias{create_dir}
\title{Title Create directory function}
\usage{
create_dir(path)
}
\arguments{
\item{path}{folder location to create a directory}
}
\value{
created directory
}
\description{
creating a directory
}
\examples{
create_dir("data/sample_folder")
}
